% C. Scheidiger, 01/13/2012
% Reads in existing SPDBs (Speech Data Base) and converst them to 
% structures and saves them as EDS (Experiment Data Structure)


%% Convert SPDBs (cell arrays) to EDS (structures)
clear all
addpath ./SPDBs ./EDCs ./bin

%--------------------------------------------------------------------------
% Modification for changing the experiment must be made in this section
for exp_name = { 'CV06SWN', 'CV06WN','HIExp2', 'HIExp4', 'HL07', ...
       'HL08', 'MN16R', 'MN64', 'SL05', 'SL06', 'SL07', 'SL07EN', ...
       'SL07L', 'TR06', 'TR07', 'TR08' } ;% 
    exp_name = char(exp_name);
    %load SPDB
    eval(['load SPDB_' exp_name])
    %load SPDB_CV06WN %SPDB_CV07SWN, SPDB_CV07WN
    load listener_master
    
    eval(['colHeadings = fieldnames(Index_' exp_name ');'])
    colHeadings = colHeadings';
    if ~(strcmp(exp_name, 'HIExp2') || strcmp(exp_name, 'HIExp4'))
        colHeadings{7} = 'l1';
    else
        colHeadings{5} = 'l1';
    end
    %{'exp' 'utter' 'talker' 'listener' 'l1' 'age' ...
    %    'gender' 'stim' 'resp' 'snr' 'time' 'repeat' 's_level' 'int_level'...
    %    'audiogram' 'hit'}
    eval(['edc = cell2struct(SPDB_' exp_name ',colHeadings,2);'])
    %edc = cell2struct(SPDB_CV06WN,colHeadings,2);
    
    %--------------------------------------------------------------------------
    
    edc = rmfield(edc, 'exp');
    edc = rmfield(edc, 'utter');
    edc.talker = char(edc.talker);
    listener_master.(genvarname(exp_name)) = unique(edc.listener);
    edc.listener = ReduceRedundancy(edc.listener);
    if ~(strcmp(exp_name, 'HIExp2') || strcmp(exp_name, 'HIExp4'))
        edc.hearing = ReduceRedundancy(edc.hearing);
    end
    
    edc.l1 = ReduceRedundancy(edc.l1);
    if ~(strcmp(exp_name, 'HIExp2') || strcmp(exp_name, 'HIExp4'))
        edc.accent = ReduceRedundancy(edc.accent);
    end
    
    edc.age = int8(edc.age);

    edc.gender = ReduceRedundancy(edc.gender);

    edc.stim=ConvertXtoUppercase(edc.stim);
    edc.stim = ReduceRedundancy(edc.stim);
    if ~(strcmp(exp_name, 'HIExp2') || strcmp(exp_name, 'HIExp4'))
        edc.compensate = ReduceRedundancy(edc.compensate);
        edc.trunctime = ReduceRedundancy(edc.trunctime);
        edc.frequency = ReduceRedundancy(edc.frequency);
    end
    edc.resp=ConvertXtoUppercase(edc.resp);
    edc.resp = ReduceRedundancy(edc.resp);
    % TR07 has an 'a' as possible resp and therefore needs to be converted
    % to char manually
    if strcmp(exp_name,'TR07')
        edc.resp = char(edc.resp);
    end
    edc.snr = ReduceRedundancy(edc.snr);
    % Replace Quiet with 100db SNR
    max_snr=max(edc.snr);
    if (max_snr > 12 && max_snr ~= 100)
        prompt = sprintf('Max SNR is %i dB. Do you want to replace it with 100 dB?Y/N [Y]: ',max_snr);
        reply = input(prompt, 's');
        if isempty(reply)
            reply = 'Y';
        end
        if reply == 'Y'
            idx_maxsnr = (edc.snr == max_snr);
            edc.snr(idx_maxsnr)=100;
        end
    end
    if ~(strcmp(exp_name, 'HIExp2') || strcmp(exp_name, 'HIExp4'))
        edc.noise = ReduceRedundancy(edc.noise);
    end
    edc.repeat = ReduceRedundancy(edc.repeat);
    edc.s_level = ReduceRedundancy(edc.s_level);
    if ~(strcmp(exp_name, 'HIExp2') || strcmp(exp_name, 'HIExp4'))
        edc.n_level = ReduceRedundancy(edc.n_level);
        edc.r_level = ReduceRedundancy(edc.r_level);
    end
    if strcmp(exp_name,'HIExp2')
        SPDB_HIExp2=GetAudiogram(SPDB_HIExp2, SPDB_HIExp2old);
    end
    edc.audiogram = Audiogram(edc.audiogram);
    edc = rmfield(edc, 'hit');
    if ~(strcmp(exp_name, 'HIExp2') || strcmp(exp_name, 'HIExp4'))
        edc.comment = ReduceRedundancy(edc.comment);
        
        if length(edc.comment) == 0
            edc = rmfield(edc, 'comment');
        end
    end
    eval([exp_name ' = edc'])
    prompt = sprintf('Save above listed structure as EDS_%s.mat? Y/N [Y]: ',exp_name);
    reply = input(prompt, 's');
    if isempty(reply)
        reply = 'Y';
    end
    
    if reply == 'Y'
        save(['./EDSs/EDS_' exp_name '.mat'], exp_name);
        save('listener_master', 'listener_master');
    end
    
end

%% Split EDS into three structures personalinfo, subjects and the trails named after the Experiment

clear all
addpath ./bin ./EDSs
n=1;
load listener_master
% order in which the subject codes get assigned
exp_order = {'MN64', 'SL05', 'SL06', 'CV06SWN', 'CV06WN', 'HL07', ... %,
    'HL08', 'MN16R',   'SL07', 'SL07EN', ...
    'SL07L', 'TR06', 'TR07', 'TR08', 'HIExp2', 'HIExp4'}

% get all names in one list and create look-up table for listener codes
exps = fieldnames(listener_master);
list_cnt=1;
ex=0; %exist flag
for exp_n = 1 : length(exps) % loop over experiments
    % count of listeners to be added
    len = length(listener_master.(genvarname(exps{ exp_n })));
    if list_cnt ~= 1 % if it isn't the first experiment to be added to the list
        for i=1:len % loop over the listeners to be added
            listener2ba = char(listener_master.(genvarname(exps{ exp_n }))(i));
            for l=1:(list_cnt-1) % loop over already existing listener to see wheter the listener to be added already exists
                listener_in_list = char(listeners(l));                
                if strcmp(listener2ba, listener_in_list)
                    s = sprintf('%s listenr %d in exp %s is equal to a listener in an earlier exp',...
                        listener_master.(genvarname(exps{ exp_n })){i}, (i), exps{exp_n});
                    disp( s );
                    ex = 1; % set exist flag
                    break  % leave loop
                end
            end
            if ~ex % listener is added to the list if doesn't already exist
                % see wheter or not listener is the same as the one before just the
                % other ear
                if listener2ba( length(listener2ba) ) == 'L' || listener2ba( length(listener2ba) ) == 'R'
                    prevlistener = char(listeners(list_cnt - 1)); % listener last added to the list
                    % if previous listener and listener to be added are the
                    % same except the last digit (L and R)
                    if strcmp(listener2ba(1:length(listener2ba)-1), prevlistener(1:length(prevlistener)-1))
                        list_cnt = list_cnt - 1;
                        listeners( list_cnt+1 , 1) = { listener2ba };
                        listeners{list_cnt+1,2} = num2str(list_cnt, '%03d');
                        list_cnt = list_cnt + 2;
                    else
                        listeners( list_cnt , 1) = { listener2ba };
                        listeners{list_cnt,2} = num2str(list_cnt, '%03d');
                        list_cnt = list_cnt + 1;
                    end
                else
                    listeners( list_cnt , 1) = { listener2ba };
                    listeners{list_cnt,2} = num2str(list_cnt, '%03d');
                    list_cnt = list_cnt + 1;
                end
                
            end
            ex = 0;   % reset exist flag
        end
    else % the first listener is added without any constraints
        listeners( list_cnt , 1) = listener_master.(genvarname(exps{ exp_n }))(1);
        % assign the listener code to the new added listeners 
        listeners{list_cnt,2} = num2str(list_cnt, '%03d');
        list_cnt=list_cnt+1;
    end
    
end

listener_lookup = listeners;

for exp_name =  exp_order;
    
    % load eds and listener list
    exp_name = char(exp_name);
    eval(['load EDS_' exp_name ';']);
    
    if strcmp( exp_name(1:length(exp_name)-1), 'HIExp' )
        eval([exp_name '.hearing=''HI''']);
        eval([exp_name '.accent=''''']);
    end
    
    % add listener code
    listeners=listener_master.(genvarname(exp_name));
    for i=1:length(listeners)
        
        listener2ba = char( listeners(i) );
        idx_listener = find( strcmp( listener_lookup(:,1), listener2ba ));
        
        % add listener code
        listener_code = listener_lookup(idx_listener,2);
        if listener2ba( length(listener2ba) ) == 'L' || listener2ba( length(listener2ba) ) == 'R'
           listener_ear = listener2ba( length(listener2ba) );
        else
           listener_ear = '';
        end            
        listeners(i,2) = listener_code;
        
        
        % replace listener names by code
        eval(['eds = ' exp_name ';']);
        if strcmp(class(eds.listener),'char')
            eds.listener = cellstr(eds.listener);
        end
        
        idx = find( strcmp( eds.listener, listeners(i) ) );
        eds.listener( idx )= cellstr( repmat( listener_code, length(idx) ,1) );
        % add ear field if necessary
        if ~isempty( listener_ear )
            %disp(exp_name);
            ears(idx,1) = cellstr( repmat( listener_ear,length(idx) ,1 ) );
        end
        if size(eds.l1,1) == 1
            listeners(i,3) = cellstr(eds.l1(1,:));
        else
            listeners(i,3) = cellstr(eds.l1(idx(1),:));
        end
        if strcmp( exp_name(1:length(exp_name)-1), 'HIExp' ) || size( eds.accent ,1 ) == 0 
            listeners( i ,4 ) = {''};
        elseif size( eds.accent ,1 ) == 1
            listeners( i ,4 ) = cellstr( eds.accent(1,:) );
        else
            listeners(i,4) = cellstr( eds.accent(idx(1) ,:) );
        end
        listeners(i,5) = cellstr( eds.gender( idx(1) ) );
        listeners(i,6) = cellstr( num2str( eds.age( idx(1) ) ) );
        n=n+1;
    end
    % format eds and remove listener dependent fields
    eds.listener = char(eds.listener);
    if exist('ears')
        length(ears)
        length(eds.listener)
        eds.ear = char(ears);
        clear ears;
    end
    eds = rmfield(eds ,'hearing');
    eds = rmfield(eds ,'audiogram');
    eds = rmfield(eds ,'l1');
    eds = rmfield(eds ,'accent');
    eds = rmfield(eds ,'gender');
    eds = rmfield(eds ,'age');
    eval( [exp_name '= eds;'] );
    
    % convert listener information to struct
    colHeadings = {'name', 'listener', 'l1', 'accent', 'gender','age'};
    c= cell(1,size(listeners,2));
    for i=1:size(listeners,2)
        c{1,i} = listeners(:,i);
    end
    
    subjects = cell2struct(c,colHeadings,2);
    subjects.listener = char(subjects.listener);
    for i=1:length(subjects.age)
        age(i,1) = str2num(subjects.age{i});
    end
    subjects.age=int8(age);
    
    % extract personal information to a sperate structure
    personalinfo.name=subjects.name;
    personalinfo.listener=char(subjects.listener);
    subjects = rmfield(subjects, 'name');
    
    save(['./EDSs/EDS_' exp_name '.mat'], exp_name, 'subjects', 'personalinfo');
end
save(['../listener_master.mat'], 'listener_master', 'listener_lookup');
clear all