% this script make a scatter plot of the properties of the events of the
% stop consonants. Peak Frequency of Burst vs Duration between Burst and 
% Voice Onset Time (VOT)

clear all; clc; close all;

%-------------------Freq----Dur----Strength
prop_burst_stop = [ ...
                    0.55,   73,     -1;      % pa_m118, t=15.18 cs
                    0.48,   70,     10;      % pa_m114, t=19.2 cs
                    0.40,   47,     2;     % pa_m104,t=27
                    0.60,   62,     0;     % pa_f103, t=21.8
                    0.25,   50,     5;      % pa_f106, t=25
                    0.50,   48,     8;     % pa_f109, t=17.18
                    0.50,   -18,    13;     % ba_m118, t=20.4
                    0.50,   -12,    10;     % ba_m107, t=27.4
                    0.48,   -20,    10;     % ba_m111, t=28.6
                    NaN,    NaN,    NaN;     % ba_f119
                    0.50,   -14,    0;     % ba_f105, t=27.88
                    0.52,   -10,    -1;      % ba_f101, t=27.9
                    4.1,    54,     -10;     % ta_m115, t=27.65
                    4.2,    34,     0;      % ta_m112, t=27.64
                    2.7,    75,     -1;      % ta_m104, t=27.92
                    5.2,    43,     -2;      % ta_f105, t=27.89
                    5.4,    76,     2;      % ta_f108, t=27.66
                    3.5,    66,     4;      % ta_f106, t=15.43
                    5.2,     4,     3;      % da_f103, t=27.89
                    2.8,     0,     -4;      % da_m118, t=26.65
                    4.2,     0,     10;     % da_m111, t=26.5
                    4.8,   -14,     -4;      % da_f119, t=28.14
                    2.7,    10,     -4;      % da_m104, t=28.17
                    3.6,    4,      10;     % da_m115, t=26.1
                    1.6,    68,     -8;      % ka_m111, t=21.41
                    1.3,    88,     -3;      % ka_m118, t=27.66
                    1.3,    68,     4;      % ka_m114, t=20.41
                    1.6,    59,     -4;      % ka_f103, t=16.68
                    1.6,    44,     0;      % ka_f105, t=22.2
                    1.6,    51,     -3;      % ka_f119, t=16.67
                    1.8,    12,     -5;      % ga_m111, t=19.67
                    1.5,     0,     -1;      % ga_m104, t=27.2
                    1.7,    18,     -2;      % ga_m112, t=27.14
                    2.0,    23,     -3;      % ga_f119, t=27.88
                    2.0,    5,      11;     % ga_f108, t=26
                    1.7,    16,     1];    % ga_f109, t=19.2
              
% group = {'pa','pa','pa','pa','pa','pa',...
%          'ba','ba','ba','ba','ba','ba',...
%          'ta','ta','ta','ta','ta','ta',...
%          'da','da','da','da','da','da',...
%          'ka','ka','ka','ka','ka','ka',...
%          'ga','ga','ga','ga','ga','ga'};

group = [1,1,1,1,1,1, 2,2,2,0,2,2, 3,3,3,3,3,3, ...
         4,4,4,4,4,4, 5,5,5,5,5,5, 6,6,6,6,6,6];
     
% make scatter plots for the six consonants
figure;
%gscatter(prop_burst_stop(:,2), prop_burst_stop(:,1), group, 'mcrgbk', '+o*^sp', ...
%         8, 'on', 'Duration between burst and vowel [ms]', 'Center Frequency [kHz]');

markers = {'s', 's', 'o', 'o', '^', '^'};
clr = [1 0 0; 1 0 0; 0 0 0; 0 0 0; 0 0 1; 0 0 1];
facecolor = {'none', 'r', 'none', 'k', 'none', 'b'};

% for k=1:6
%     n = [(k-1)*6+1:(k-1)*6+1];
%     marksize = (-prop_burst_stop(n,3)+12)*0.5+4;
%     hC{k} = semilogy(-prop_burst_stop(n,2), prop_burst_stop(n,1), [markers{k} ], 'Color', clr(k, :),...
%                     'MarkerFacecolor', facecolor{k},'Markersize', marksize); hold on;
% end
% legend('pa', 'ba', 'ta', 'da', 'ka', 'ga', 'FontSize', 12, 'Location', 'EastOutside');

semilogy([0 0],[0.2 8], 'Linestyle', '--', 'Color', [0,1,0], 'Linewidth', 2); hold on
text(3, 1, 'vowel /a/', 'Fontsize', 14, 'Fontweight', 'Bold');

for k=1:6
    for m=1:6
        n = (k-1)*6+m;
        marksize = (-prop_burst_stop(n,3)+12)*0.5+4;
        if ~isnan(marksize)
            hC{k} = semilogy(-prop_burst_stop(n,2), prop_burst_stop(n,1), [markers{k} ], 'Color', clr(k, :),...
                    'MarkerFacecolor', facecolor{k}, 'Markersize', marksize); hold on;
            %text(-prop_burst_stop(n,2)+3.3, prop_burst_stop(n,1), num2str(prop_burst_stop(n,3)), 'Fontsize', 8);
            text(-prop_burst_stop(n,2), prop_burst_stop(n,1)*1.08, num2str(prop_burst_stop(n,3)), 'Fontsize', 8);
        end
    end
end


xlim([-110, 30]); ylim([0.2, 8]);
%set(gca, 'XTick', [-100:10:20], 'XTickLabel', [100:-10:-20], 'FontSize', 12);
set(gca, 'XTick', [-100:20:20], 'FontSize', 12);
set(gca, 'YTick', [0.2,0.4,0.8,1.6,3.2,6.4], 'FontSize', 12);
xlabel('Time [ms]', 'FontSize', 14);
ylabel('Frequency [kHz]', 'FontSize', 14);