function [ level3, level1,  detection_thresholds, st_dev, rate ] = bm_filter_erbaligned( signal, channels, noise_or_speech, detection_threshold, return_l1 )

% [ level3, level1,  detection_thresholds, st_dev, rate ] = bm_filter_erboldmod( signal, channels, noise_or_speech, detection_threshold, return_l1 )
%
% This function filters the signal with the basilar membrane filterbank. This
% particular version of it uses a very sharp elliptic filter on the high side
% and a butterworth fitler with the proper slope on the low side.  The phase
% is somewhat close to linear in the passband.
%
% Each row of level3 corresponds to a different frequency, the columns
% correspond to time at a rate of 400/second.
%
% signal is the input signal at a rate of 16000.
%
% type indicates whether the 99% threshold and the std
%      of the noise need to be computed.  This is to save
%      time in the event that the signal is speech and not
%      noise.


level1 = [];

delay_data = [317   336   333   331   329   327   325   341   339   336   334   332   330   344   341   339   337   335   333   331   342   340   338   336   335   344   342   340   338   336   345   343   341   339   337   345   343   341   339   337   344   334   332   338   329   335   341   339   330   336   334   339   331   329   334   339   331   329   328   338   325   335   328   321   325   334   322   326   320   328   322   330   324   327   330   320   323   326   312   319   318   313   327   322   317   323   311   317   316   318   317   319   305   304   300   305   316   315   314  1215  1234  1219  1215  1222  1218  1204  1211  1217  1194  1200  1197  1203  1190  1196  1184  1181  1178  1175  1180  1177  1182  1179  1176  1173  1163  1153  1172  1155  1166  1150  1154  1145  1149  1134  1138  1142  1140  1143  1135  1133  1125  1123  1132  1119  1133  1110  1103  1122  1105  1108  1111  1114  1112  1101  1104  1098  1092  1099  1089  1096  1082  1085  1072  1086  1081  1076  1071  1077  1076  1071  1070  1069  1061  1070  1056  1055  1051  1069  1065  1052  1051  1053  1055  1057  1045  1047  1049  1043  1050  1039  1041  1038  1042  1039  1036  1038  1035  1030  1036  1029];

high_rate_max = max(delay_data(100:200));
low_rate_max = max(delay_data(1:99));



% Make sure "signal" is always a column vector.
if size(signal,1) < size(signal,2)
   signal = signal';
end



profile = 0;  % turn this on to do profiling

if profile
   tic
end

if length(strfind(path,'./bin')) == 0 
   path(path,'./bin/')
end



filter_fs = 16000;  % this is the rate for the bm filters
speech_fs = 16000;
decimated_fs = 400;
D = 4000/400;
env_grd = 30;  % in samples

%if ~exist('','var')
%   detection_threshold = 0.99;
%end


%  if profile
%     disp(sprintf('Making filter stuff at +%f',toc))
%  end

how_many_filters = 200;
bmdist = linspace(7.35,30,how_many_filters);
frequencies = flipud(greenwood_inverse( bmdist, 'dist' )')';



envelope_filters_filename = './mat/envelope_filters.mat';
load(envelope_filters_filename);


%  if ~exist(envelope_filters_filename,'file')
%  
%     %let's make a better filter with a strictly positive impulse respones
%     [B1h,A1h] = butter(6,60*pi/filter_fs);  % 1 order will give us strictly positive
%     [B1l,A1l] = butter(10,50*pi*4/filter_fs);
%     [B2,A2] = butter(4,30*pi/400);
%     [B3,A3] = ellip(12,0.2,80,500*pi/8000);  % filter to try to solve the aliasing problem	
%     save(envelope_filters_filename,'B1h','A1h','B1l','A1l','B2','A2','B3','A3')
%  
%  else
%     load(envelope_filters_filename)
%  end


% These are the filters which make up the "auditory filterbank"
%load(filter_setup_file)






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% Filter loading and setup
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

filter_setup_file = './mat/cmsgram1_erb_bmfilters.mat';
%envelope_filters_filename = './mat/envelope_filters.mat';

%load(envelope_filters_filename)

% These are the filters which make up the "auditory filterbank"
load(filter_setup_file)

fsx = rate;










%load ./mat/bm_filters_beta_0830.mat
%load ./mat/env_filters_bmflapx.mat

%[B_lp,A_lp] = butter(4,70*2/decimated_fs);

[B_env,A_env] = ellip(12,0.25,90,0.09);

%keyboard

% Note, these filters may not have the correct width.  My thought on fixing that
% is this:
% 1)  Find filters with the correct width from the cat data.
% 2)  Modulate the filters up to the appropriate frequency.
% 3)  Bandpass filter to remove the "artifacts" from Neely's simulation,
%     as well as aliasing from the procedure we used.
%
% Note:  It would be better to do some of this in the filter design procedure,
%        but I don't have time or desire to do that right now.
%


last_rate = 0;

detection_thresholds = zeros(1,length(frequencies));


%  disp(sprintf('Done doing all that warmup stuff +%f',toc))

if profile
   cell_array_time = 0;
   src_time = 0;
   bmfilter_time = 0;
   make_envfilt_time = 0;
   envfilt_time = 0;
   stats_time = 0;
   resample_time = 0;
   filter_again_time = 0;
   put_in_struct_time = 0;
end



for n = channels %1:length(frequencies)

   if profile
      %disp(sprintf('Starting freq loop at +%f',toc))
      loop_start = toc;
   end

   % Some of the filters run at a different rate because
   % it's hard to make really narrow and stable filters.
   % If they run at a lower rate, then we need a decimated
   % version of the speech signal

   %[ fs, Bb, Ab, Be, Ae ] = make_auditory_filters_be( frequencies(n) );

   fs = fsx(n);
%   Bb = cell2mat(Bbx(n));
%   Ab = cell2mat(Abx(n));
%   Be = cell2mat(Bex(n));
%   Ae = cell2mat(Aex(n));

   if profile
      %disp(sprintf('Done fooling around with cell arrays +%f',toc))
      d_cell_array_time = toc;
   end


   if fs ~= speech_fs

      % We don't need to waste time decimating the same signal over and over
      if fs ~= last_rate
         tmpz = decimate( signal, speech_fs/fs );
         last_rate = fs;
      end
   else
      tmpz = signal;
   end
   

   if profile
      %disp(sprintf('Done sample rate converting +%f',toc))
      d_src_time = toc;
   end


   
   % compute the gain compensation

   %gain = abs( polyval( hx(n,:), exp( j*2*pi*frequencies(n)/fsx(n) ) ) );
   gain = 1;


   
   % This is the auditory filter
   if n < 101
      tmpa = filter([zeros(1,low_rate_max-delay_data(n)) Bk1(n,:)] ,Ak1(n,:), tmpz );
   else
      tmpa = filter([zeros(1,high_rate_max-delay_data(n)) Bk1(n,:)] ,Ak1(n,:), tmpz );
   end

   if exist('return_l1','var')
      if return_l1
         level1(n).data = tmpa;
      end
   end
  

   %tmpa = conv(tmpz,hx(n,:)) / gain;
   

   %keyboard
   %tmpa = filter(Bb,Ab, filter(Be,Ae, tmpz ) );

   
   if profile
      %disp(sprintf('Done filtering with the BM filter +%f',toc))
      d_bmfilter_time = toc;
   end

   

   %envelope_filter_B = env_filters_B(n,:);
   %envelope_filter_A = env_filters_A(n,:);


   
   if profile
      %disp(sprintf('Done making envelope filter +%f',toc))
      d_make_envfilt_time = toc;
   end
   
   %disp(sprintf('starting = %i',length(tmpa)))

   % This is the envelope detection stp
   if rate(n) ~= speech_fs
      %tmpb = abs( tmpa );
      %tmpb = filter(B_env,A_env, tmpb );
      %tmpb = tmpb / max(tmpb) * max(tmpa);
      %tmpb = tmpb(env_grd:end);
      tmpb = filter([zeros(1,0) B1l],A1l,half_wave_rectifier( tmpa ) );
      tmpb = tmpb * max(tmpa)/max(tmpb);
      %tmpb = [ tmpb(60:length(tmpb))' zeros(1,60) ]';
                        
      %tmpb = [ tmpb' zeros(1,4-mod(length(tmpb),4)) ]';
                        
   else

      %keyboard
      %tmpb = abs( tmpa );
      %tmpb = decimate( tmpb, speech_fs/4000 );
      %tmpb = filter(B_env,A_env, tmpb );
      %tmpb = tmpb / max(tmpb) * max(tmpa);
      %tmpb = tmpb(env_grd:end);
      tmpb = filter([ zeros(1,250) B1h],A1h,half_wave_rectifier( tmpa ) );
      tmpb = tmpb * max(tmpa)/max(tmpb);
      %tmpb = [ tmpb(150:length(tmpb))' zeros(1,200) ]';% was 151
   end

   %disp(sprintf('ending = %i',length(tmpb)))

   % envelope filtering
   %tmpb = filter(envelope_filter_B,envelope_filter_A, max(0,tmpa) );
   %tmpb  = tmpb * max(tmpa)/max(tmpb);

   %stage2(n).data = tmpb;

   tmpc = decimate( tmpb, 40*rate(n)/speech_fs);

   tmpd = filter(B2,A2,tmpc);   

   if profile
      %disp(sprintf('Done envelope detecting +%f',toc))
      d_envfilt_time = toc;
   end          
      
  
   % This is where we'll compute the detection threshold.  We still have the signal
   % at full rate, or near full rate.  For the lower frequencies, they will be at
   % much lower rates, eg 2000/sec which is not very many.  We may need some special
   % arrangement.

   %keyboard

   if exist('noise_or_speech','var')
      %disp('1')
      if strcmp(noise_or_speech,'noise')
         %disp('2')
         tmpb_sorted = sort( tmpb );
         detection_thresholds(n) = tmpb_sorted( round( detection_threshold*length(tmpb) ) );
         st_dev(n) = std(tmpb,0);
      end
   end


   % The rate of the signal is brought down to 400/second

   %if ~strcmp(noise_or_speech,'noise')
   %tmpc = decimate( tmpb, 40*rate(n)/speech_fs );
   
   %tmpc = tmpb(1:D:end);

   %else
   %   tmpc = tmpb;
   %end



   if profile
      d_stats_time = toc;
   end


   % sample rate convert to 400 hz
   %tmpc = resample( tmpb, decimated_fs / gcd(decimated_fs,fs), fs / gcd(decimated_fs,fs) );


   if profile
      %disp(sprintf('Done +%f',toc))
      d_resample_time = toc;
   end

   % lpf to 70 hz

   % dont' want to do this anymore
   %tmpd = filter(B_lp,A_lp, tmpc);

   %keyboard

   % Filter again, although I'm not sure why.  This may
   % be an unnecessary step.
   %tmpd = filter(B2,A2,tmpc);

   if profile
      d_filter_again_time = toc;
   end   


   %disp(length(tmpd))

   % level3 holds the envelope of the output of each signal.

   %disp(sprintf('n = %i, len(tmpb) = %i, %i',n,length(tmpb),length(tmpc)))

   if n == 1
      level3 = zeros(length(frequencies),length(tmpd));
      mlen = length(tmpd);
   elseif length(tmpd) ~= mlen
      tmpd = [ tmpd ;zeros(mlen - length(tmpd),1) ];   
   end
   %keyboard
  
   
   level3(n,:) = tmpd(1:mlen)';
  


   if profile
      d_put_in_struct_time = toc;

      cell_array_time = cell_array_time + d_cell_array_time - loop_start;
      src_time = src_time + d_src_time - d_cell_array_time;
      bmfilter_time = bmfilter_time + d_bmfilter_time - d_src_time;
      make_envfilt_time = make_envfilt_time + d_make_envfilt_time - d_bmfilter_time;
      envfilt_time = envfilt_time + d_envfilt_time - d_make_envfilt_time;
      stats_time = stats_time + d_stats_time - d_envfilt_time;
      resample_time = resample_time + d_resample_time - d_stats_time;
      filter_again_time = filter_again_time + d_filter_again_time - d_resample_time;
      put_in_struct_time = put_in_struct_time + d_put_in_struct_time - d_filter_again_time;



   end


   % This is where we estimate the mean and the variance
   % of the noise in each channel.

   % Note:  The noise is actually rayleigh distributed which means
   % this mu and sigma are wrong, but the wrongness is compensated for
   % so it's sorta okay.  In the end this should be changed.

     if 0 %mod(n,20) == 4
     %   keyboard
      figure
      [N,X]=hist(tmpb,100);
      plot(X,N)
      hold on
      plot(detection_thresholds(n)*[1 1],[ 0 100 ],'r')
      plot((mean(tmpb)+st_dev(n))*[1 1],[0 100],'k')
      plot((mean(tmpb)-st_dev(n))*[1 1],[0 100],'k')

   end
                
end

%keyboard
if ~strcmp(noise_or_speech,'noise')
   rate = decimated_fs * ones(1,how_many_filters);
end


%keyboard
if profile

   disp(sprintf('cell_array_time = %f',cell_array_time))
   disp(sprintf('scr_time = %f',src_time))
   disp(sprintf('bmfilter_time = %f',bmfilter_time))
   disp(sprintf('make_envfilt_time = %f',make_envfilt_time))
   disp(sprintf('envfilt_time = %f',envfilt_time))
   disp(sprintf('stats_time = %f',stats_time))
   disp(sprintf('resample_time = %f',resample_time))
   disp(sprintf('filter_again_time = %f',filter_again_time))
   disp(sprintf('put_in_struct_time = %f',put_in_struct_time))

   disp(sprintf('\nTotal time is %f',toc))

end

%keyboard

