function [ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn16r( speech_data )

% function [  speech_in, noise_std ] = prepare_speech_mn16r( speech_data )
% speech_data contains information necessary to determine the speech and noise.  It.
% may come in three different formats, which are:
%
% speech.fs = sample rate
%       .signal = the speech signal w/o noise
%       .noise = the noise signal at the correct snr
%       .type = 'separate'
%
% speech.fs = sample rate
%       .signal = the speech signal
%       .snr = snr in dB
%       .type = 'speechonly'
%
% speech.original = a copy of the original wav file
%       .snr = snr in dB
%       .type = 'premixed'
%       .nmpath = path to the noisemaker used to prepare the file, this option
%                 exists for beren because it had a "special" noisemaker which
%                 which is not like the original
%
% speech_in is the mixed speech signal suitable for processing
% noise_std is the standard deviation of the noise



% User editable parameters
fs = 16000;
flow = 180;
fhigh = 7400;
how_many_filters = 200;
pad_factor = 6;


if length(strfind(path,'./bin')) == 0 
   path(path,'./bin/')
end

mat_dir = './mat/';



cmsgram_output_rate = 400;  % the sample rate of the data from the cmsgram



if strcmp(speech_data.type,'separate')

   speech_in = speech_data.signal + speech_data.noise;

   if size(speech_in,1) < size(speech_in,2)
      speech_in = speech_in';
   else
      speech_in = speech_in;
   end

   speech_only = speech_data.signal;
   noise_only = speech_data.noise;

   noise_std = std(speech_data.noise,1);

end


if 0 % strcmp(speech_data.type,'premixed')


   addpath(speech_data.nmpath)
   [noise speechscale] = noisemaker_mn55(speech_data.original,speech_data.snr,16000,'','',0);
   %noise = noisemaker_MN16R( original_speech, snr, fs );  %, start_samples, stop_samples );
   rmpath(speech_data.nmpath)

   %disp(speechscale)

   speech_only = [];

   %  if the ramp is strong, this may not be quite right
   noise_only = noise(1:length(speech_data.signal));

 
   speech_in = speech_data.signal;
   noise_std = std(noise,1);

end








if strcmp(speech_data.type,'speechonly')


   
   %% This section will prepare the speech signal by
   % 1) scaling it to a uniform and reasonable level
   % 2) clipping off the irrelevant parts of the sound
   % 3) padding with zeros, so that the EM algorithm can converge
   
   % load the sound, and set up the noise level
  
   if size(speech_data.signal,1) < size(speech_data.signal,2)
      speech = speech_data.signal';
   else
      speech = speech_data.signal;
   end
   
   
   speech_level = 0.002;
   
   % 2)
   
   % stupid matlab, in-fucking-sane
   trash = which('label_begining_and_end2');
   
   [first_sample last_sample] = label_begining_and_end2( speech );
   cropped_speech = speech( max(1,first_sample-3000):min(length(speech),last_sample+3000) );
   
   % 1)
   % volume normalize the sound
   cropped_scaled_speech = cropped_speech/std(cropped_speech,1)*speech_level;
   
   % 3)
   
   % Note: pres_padding must be long enough that the amount of padding on each side
   % makes the sample at least 40000 in length.
   
   pres_padding = 0.025;  % seconds, space on each side to make it look nice, etc, let the filters ring, etc
   
   if 2*round( pres_padding * fs ) + length(cropped_speech) < 40000
      noisemaker_padding = ceil( ( 40000 - length(cropped_speech) ) / 2 / fs );
   end
   
   
   noise_samples = 100000;
   pres_padding_samples = round( pres_padding * fs );
   noisemaker_padding_samples = round( noisemaker_padding * fs );
   
   padded_cropped_scaled_speech = [  zeros(1,noise_samples) zeros(1,pres_padding_samples + noisemaker_padding_samples) ...
                                     cropped_scaled_speech' zeros(1,pres_padding_samples + noisemaker_padding_samples) ];
     
   % apply the headphone filter to the speech, as specified
   load ./mat/bpfilt_MN16R.mat
   processed_speech = filter(B_headphones,A_headphones,padded_cropped_scaled_speech);  % same, just to make the code more readable
   
   
   
   % Adding the noise.  Note!:  This file dresses up the file only with the MN64 noise.
   % If the speech sample is from another experiment, a different noisemaker is needed.
   
   % noise = eval(noisemaker_str);
   
   % Note:  That start_sample and stop_sample (last two arguments respectively) need
   % to be the first and last sample for the wave file.  noisemaker_MN64 knows how much
   % padding needs to be used so the padding is exactly the same as it was in the MN64
   % experiment.
   
   
   
   start_samples = noise_samples + pres_padding_samples + noisemaker_padding_samples;
   stop_samples = length(processed_speech) - pres_padding_samples - noisemaker_padding_samples;
   
   %keyboard
   
   noise = noisemaker_MN16R( processed_speech, speech_data.snr, fs );  %, start_samples, stop_samples );
   
   %keyboard
   
   noisy_speech = noise + processed_speech';
   
     
     
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     % code which deals with setting the levels of the noise
     % and the detection threshold, etc
     
     %snr = 10^(SNR/20);  % in amplitude not power
     
     % data which will help us determine what is "visible" above the noise
     % this contains a vector called 'band_power_level' which has the std
     % of the noise in each of the 200 bands we are using
     % band_power_mu
     % band_power_sigma
     %load hsr_swn_bandpower07.mat
     
     
     % this is a quantity says the ratio of the probe to masker where the
     % probe is just barely detectible.  For a tone, this value is unity.
     % for speech it is probably a little bit less.  This number is a guess
     % and will be determined by Andy Lovitt's work.
     % scratch
     % This is the number of sigma (the std of the noise) that the output
     % of the auditory filter must be over for that part of the speech
     % signal to be detectible.
     
   %speech_perception_threshold = 2.6;
     
     
   % Set up the auditory filterbank  
   bmdist = linspace(7.35,30,how_many_filters);
   freqs = flipud(greenwood_inverse( bmdist, 'dist' )')';
   bm_filters_filename = [ mat_dir 'standard.mat' ];
   
   
   
   %make_auditory_filters( freqs, bm_filters_filename );
   
   
   % This is where the signal is filter, noise is estimated, etc
   %[stage1, stage1_noise_info] = filter_detect_estimate( noisy_speech, bm_filters_filename );
   
   %keyboard
   
   % This has been changed to not include send noise because we're going to pregenerate the thresholds
   % and stds
   
   
   % This is the speech signal that we're putting into the ccgram
   % It should correspond the ccgram output
   speech_in = noisy_speech( noise_samples+noisemaker_padding_samples+1:end-noisemaker_padding_samples );
   noise_std = std(noise,1);

   speech_only = processed_speech( noise_samples+noisemaker_padding_samples+1:end-noisemaker_padding_samples )';
   noise_only = noise( noise_samples+noisemaker_padding_samples+1:end-noisemaker_padding_samples );

   
   % The noise statistics are pregenerated, since they will be the same for
   % any experiment here using the MN64 noise.
   
   
   % We need to scale it accoringly.  The noise used to generate the threshold and std was of
   % unity RMS, so we need to multiply the numbers by the rms of the noise we're using.

end