function fig_handle = make_figure( speech_data, experiment, sat_method, filterbank )

% function fig_handle = make_figure( speech_data, experiment, sat_method, filterbank )
%
% speech_data contains information necessary to determine the speech and noise.  It.
% may come in three different formats, which are:
%
% speech.fs = sample rate
%       .signal = the speech signal w/o noise
%       .noise = the noise signal at the correct snr
%       .type = 'separate'
%
% speech.fs = sample rate
%       .signal = the speech signal
%       .snr = snr in dB
%       .type = 'speechonly'
%
% speech.original = a copy of the original wav file
%       .snr = snr in dB
%       .type = 'premixed'
%       .nmpath = path to the noisemaker used to prepare the file, this option
%                 exists for beren because it had a "special" noisemaker which
%                 which is not like the original
%
% experiment is 'mn16r' or 'mn64' which indicate the noise configuration
%
% sat_method indicates which scaling/thresholding/etc method is used, valid
% options are 'nadeeka', 'ai', or 'old1206'.
% *- The 'nadeeka' method uses 1/3*log10( (a(t,n) / dt(n))^2 ) limited to +/- 1
% *- The 'ai' method uses 1/3*log10( a(t,n)^2 / E_t( at(t,n)^2 ) - a(t,n)/E_t( at(t,n) ) )
% *- 'old1206' - original method, well documented but too complicated to explain
%                here.
%
% filterbank is 'erboldmod' which is the samem filter bank always used based on
% sharp elliptical erb equiv filters.  'erbaligned' is the same just shifted in
% time to not have a frequency dependant phase delay.
%


fontsize = 16;

[ aigram, ait, time_axis, freq_axis ] = scaling_cc( speech_data, experiment, sat_method, filterbank );


S = load('./mat/standard.mat');
frequencies = S.frequencies;

[how_many_filters, how_many_samples] = size( aigram );

fig_handle = figure('visible','off');

% Draw the ccgram figure
subplot('position', [0.1 0.29 0.85 0.63])
        
image( aigram, 'CDataMapping','scaled')
set(gca,'CLim',[0 1])

colormap(flipud(gray))

h_colorbar = colorbar;
set(h_colorbar,'FontSize',fontsize);
set(h_colorbar,'YTick',[0:0.2:1])
set(h_colorbar,'YTickLabel',[0:0.2:1])        


set(gca,'YDir','normal')
set(gca,'FontSize',fontsize)
xlabel('Time [cs]','FontSize',fontsize)
ylabel('Frequency [kHz]','FontSize',fontsize)

%title(sprintf('SNR = %.2f dB',snr),'FontSize',fontsize)
        
set(gca,'YTick',linspace(1,how_many_filters,11));
set(gca,'YTickLabel',0.1*round(10*frequencies(round(linspace(1,how_many_filters,11)))/1000))

%set(gca,'XTick',40*[1 2 3 4 5]);
%set(gca,'XTickLabel',10*[1 2 3 4 5]);

set(gca,'XTick',40*[1:floor(how_many_samples/40)]);
set(gca,'XTickLabel',10*[1:floor(how_many_samples/40)]);
        
%if ischar(wav_file)        
%   set(gcf,'Name',sprintf('Auditory Filter Based Spectrogram for %s',wav_file));
%end
        

% Draw the channel capacity figure
subplot('position', [0.1 0.05 0.728 0.13])
        

aigram_output_rate = 400;  % per second

% The 100 is to convert from bits/sec to bits/cs
plot([1:length(ait)]/aigram_output_rate,ait)

a = axis;

a(3) = 0;
a(4) = ceil(max(ait));
a(2) = length(ait)/aigram_output_rate;
axis(a)


        
set(gca,'FontSize',fontsize)
set(gca,'XTick',40*[1 2 3 4 5]);
set(gca,'XTickLabel',[])
        
ylabel('Inst. AI','FontSize',fontsize)
