function [aigram, ait, time_axis, freq_axis ] = scaling_cc( speech_data, experiment, sat_method, filterbank )

% function [aigram, ait, time_axis, freq_axis, speech_data ] = scaling_cc( speech_data, experiment, sat_method, filterbank )
%
% speech_data contains information necessary to determine the speech and noise.  It.
% may come in three different formats, which are:
%
% speech.fs = sample rate
%       .signal = the speech signal w/o noise
%       .noise = the noise signal at the correct snr
%       .type = 'separate'
%
% speech.fs = sample rate
%       .signal = the speech signal
%       .snr = snr in dB
%       .type = 'speechonly'
%
% speech.original = a copy of the original wav file
%       .snr = snr in dB
%       .type = 'premixed'
%       .nmpath = path to the noisemaker used to prepare the file, this option
%                 exists for beren because it had a "special" noisemaker which
%                 which is not like the original
%
% experiment is 'mn16r' or 'mn64' which indicate the noise configuration
%
% sat_method indicates which scaling/thresholding/etc method is used, valid
% options are 'nadeeka', 'ai', or 'old1206'.
% *- The 'nadeeka' method uses 1/3*log10( (a(t,n) / dt(n))^2 ) limited to +/- 1
% *- The 'ai' method uses 1/3*log10( a(t,n)^2 / E_t( at(t,n)^2 ) - a(t,n)/E_t( at(t,n) ) )
% *- 'old1206' - original method, well documented but too complicated to explain
%                here.
%
% filterbank is 'erboldmod' which is the samem filter bank always used based on
% sharp elliptical erb equiv filters.  'erbaligned' is the same just shifted in
% time to not have a frequency dependant phase delay.
%


% first prepare the speech using the prepare_wav_[].m



% next run the auditory models



% next make the image

if strcmp(sat_method,'ai')
   [ speech_in, speech_only, noise_only, noise_std ] = eval(sprintf('prepare_speech_%s( speech_data )',experiment));
   [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_in, [1:200], ''speech'' , 0, 0 )' ]);
   [aigram, ait, time_axis, freq_axis ] = form_image_ai( level3, experiment, filterbank, noise_std );
end

if strcmp(sat_method,'aiwct')
   [ speech_in, speech_only, noise_only, noise_std ] = eval(sprintf('prepare_speech_%s( speech_data )',experiment));
   [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_in, [1:200], ''speech'' , 0, 0 )' ]);
   [aigram, ait, time_axis, freq_axis ] = form_image_aiwct( level3, experiment, filterbank, noise_std );
end


if strcmp(sat_method,'old1206')
   [ speech_in, speech_only, noise_only, noise_std ] = eval(sprintf('prepare_speech_%s( speech_data )',experiment));
   [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_in, [1:200], ''speech'' , 0, 0 )' ]);
   threshold_file = [ experiment '_' filterbank '_85_noise_stats.mat' ];
   [aigram, ait, time_axis, freq_axis ] = form_image_old1206( level3, threshold_file, noise_std );
end


if strcmp(sat_method,'nadeeka')
   [ speech_in, speech_only, noise_only, noise_std ] = eval(sprintf('prepare_speech_%s( speech_data )',experiment));
   [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_in, [1:200], ''speech'' , 0, 0 )' ]);
   threshold_file = [ experiment '_' filterbank '_85_noise_stats.mat' ];
   [aigram, ait, time_axis, freq_axis ] = form_image_nadeeka( level3, threshold_file, noise_std );
end


if strcmp(sat_method,'ai0607')
   if strcmp(speech_data.type,'separate')
      [ speech_in, speech_only, noise_only, noise_std ] = eval(sprintf('prepare_speech_%s( speech_data )',experiment));
      [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
      [ E_ni2, E_ni ] = compute_noise_stats( experiment, filterbank, noise_std );
      [aigram, ait, time_axis, freq_axis ] = form_image_ai0607( level3, experiment, filterbank, E_ni2 );

   else
      disp('sat method ai0607 requires speech_data.type = ''separate''')
   end
end



if strcmp(sat_method,'ai0607b')
   if strcmp(speech_data.type,'separate')
      [ speech_in, speech_only, noise_only, noise_std ] = eval(sprintf('prepare_speech_%s( speech_data )',experiment));
      [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
      [ E_ni2, E_ni ] = compute_noise_stats( experiment, filterbank, noise_std );
      [aigram, ait, time_axis, freq_axis ] = form_image_ai0607b( level3, experiment, filterbank, E_ni2 );

   else
      disp('sat method ai0607b requires speech_data.type = ''separate''')
   end
end



if strcmp(sat_method,'ai0607c')
   if strcmp(speech_data.type,'separate')
      [ speech_in, speech_only, noise_only, noise_std ] = eval(sprintf('prepare_speech_%s( speech_data )',experiment));
      [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
      [ E_ni2, E_ni ] = compute_noise_stats( experiment, filterbank, noise_std );
      [aigram, ait, time_axis, freq_axis ] = form_image_ai0607c( level3, experiment, filterbank, E_ni2, E_ni );

   else
      disp('sat method ai0607c requires speech_data.type = ''separate''')
   end
end

