% This program plots the average Pe over all stimuli as a function of snr

clear all; clc; close all;

load SPDB_SL07.mat;

lab_row = {'pa';'ta';'ka';'fa';'xta';'sa';'xsa';'ba';'da';'ga';'va';'xda';'za';'xza';'ma';'na'};
lab_col = {'pa';'ta';'ka';'fa';'xta';'sa';'xsa';'ba';'da';'ga';'va';'xda';'za';'xza';'ma';'na';'?'};

stims = {'pa';'ta';'ka';'fa';'Ta';'sa';'Sa';'ba';'da';'ga';'va';'Da';'za';'Za';'ma';'na'};

%% calculate Confusion Matrix for subject DC
%------------left ear, SNR = 100dB----------------
% left ear
data = query(SPDB_SL07, 'listener', 'DC_L', 'snr', 100, 'compensate', 'NaN');
cm = calc_CM(data, 'CM of DC-L at 100 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_L_100dB_noNAL.tex', lab_row, lab_col);
pi_cv_nonalr = diag(cm.table)/12; 

data = query(SPDB_SL07, 'listener', 'DC_L', 'snr', 100, 'compensate', 'NAL-R');
cm = calc_CM(data, 'CM of DC-L at 100 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_L_100dB_NAL.tex', lab_row, lab_col);
pi_cv_nalr = diag(cm.table)/12; 

disp(['DC-L at 100dB SNR: ' num2str(mean(pi_cv_nalr-pi_cv_nonalr))]);
% make bar plots for the comparison of NAL-R vs non NAL-R
order = [1,3,4,8,10,11,15,16, 7, 2,6,9,13,14];
figure; bar2=bar([1:14], pi_cv_nalr(order), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
set(bar2,'BarWidth',0.4); bar1=bar([1:14], pi_cv_nonalr(order)); set(bar1,'BarWidth',0.2);
cm = jet(4); ch1=get(bar1,'children'); idx = [2,2,2,2,2,2,2,2,1,4,4,4,4,4];
set(ch1,'facevertexcdata',idx.'); colormap(cm);
 legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
xlim([0, 15]); ylim([0, 1.05]); title('Percent Correctness of DC-L at 100 dB SNR');
xlabel('Consonants', 'FontSize', 12); ylabel('Percent Correct', 'FontSize', 12);
set(gca,'XTick',[1:14], 'XTickLabel', stims(order)); set(gca,'YTick',[0:0.2:1]);

%------------left ear, SNR = 6dB----------------
data = query(SPDB_SL07, 'listener', 'DC_L', 'snr', 6, 'compensate', 'NaN');
cm = calc_CM(data, 'CM of DC-L at 6 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_L_6dB_noNAL.tex', lab_row, lab_col);
pi_cv_nonalr = diag(cm.table)/12; 

data = query(SPDB_SL07, 'listener', 'DC_L', 'snr', 6, 'compensate', 'NAL-R');
cm = calc_CM(data, 'CM of DC-L at 6 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_L_6dB_NAL.tex', lab_row, lab_col);
pi_cv_nalr = diag(cm.table)/12; 

disp(['DC-L at 6dB SNR: ' num2str(mean(pi_cv_nalr-pi_cv_nonalr))]);
% make bar plots for the comparison of NAL-R vs non NAL-R
order = [2,3,7,8,9,10,13,15,16, 1,4,6,11,14];
figure; bar2=bar([1:14], pi_cv_nalr(order), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
set(bar2,'BarWidth',0.4); bar1=bar([1:14], pi_cv_nonalr(order)); set(bar1,'BarWidth',0.2);
cm = jet(4); ch1=get(bar1,'children'); idx = [2,2,2,2,2,2,2,2,2,4,4,4,4,4];
set(ch1,'facevertexcdata',idx.'); colormap(cm);
legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
xlim([0, 15]); ylim([0, 1.05]); title('Percent Correctness of DC-L at 6 dB SNR');
xlabel('Consonants', 'FontSize', 12); ylabel('Percent Correct', 'FontSize', 12);
set(gca,'XTick',[1:14], 'XTickLabel', stims(order)); set(gca,'YTick',[0:0.2:1]);

%--------------right ear, SNR = 100dB-------------
data = query(SPDB_SL07, 'listener', 'DC_R', 'snr', 100, 'compensate', 'NaN');
cm = calc_CM(data, 'CM of DC-R at 100 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_R_100dB_noNAL.tex', lab_row, lab_col);
pi_cv_nonalr = diag(cm.table)/12; 

data = query(SPDB_SL07, 'listener', 'DC_R', 'snr', 100, 'compensate', 'NAL-R');
cm = calc_CM(data, 'CM of DC-R at 100 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_R_100dB_NAL.tex', lab_row, lab_col);
pi_cv_nalr = diag(cm.table)/12; 

disp(['DC-R at 100dB SNR: ' num2str(mean(pi_cv_nalr-pi_cv_nonalr))]);
% make bar plots for the comparison of NAL-R vs non NAL-R
order = [1,3,4,6,8,10,11,14,15,16, 7,13, 2,9];
figure; bar2=bar([1:14], pi_cv_nalr(order), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
set(bar2,'BarWidth',0.4); bar1=bar([1:14], pi_cv_nonalr(order)); set(bar1,'BarWidth',0.2);
cm = jet(4); ch1=get(bar1,'children'); idx = [2,2,2,2,2,2,2,2,2,2,1,1,4,4];
set(ch1,'facevertexcdata',idx.'); colormap(cm);
 legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
xlim([0, 15]); ylim([0, 1.05]); title('Percent Correctness of DC-R at 100 dB SNR');
xlabel('Consonants', 'FontSize', 12); ylabel('Percent Correct', 'FontSize', 12);
set(gca,'XTick',[1:14], 'XTickLabel', stims(order)); set(gca,'YTick',[0:0.2:1]);

%--------------right ear, SNR = 6dB-------------
data = query(SPDB_SL07, 'listener', 'DC_R', 'snr', 6, 'compensate', 'NaN');
cm = calc_CM(data, 'CM of DC-R at 6 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_R_6dB_noNAL.tex', lab_row, lab_col);
pi_cv_nonalr = diag(cm.table)/12; 

data = query(SPDB_SL07, 'listener', 'DC_R', 'snr', 6, 'compensate', 'NAL-R');
cm = calc_CM(data, 'CM of DC-R at 6 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_R_6dB_NAL.tex', lab_row, lab_col);
pi_cv_nalr = diag(cm.table)/12; 

disp(['DC-R at 6dB SNR: ' num2str(mean(pi_cv_nalr-pi_cv_nonalr))]);
% make bar plots for the comparison of NAL-R vs non NAL-R
order = [1,3,4,7,10,11,13,14,16, 2,8,15, 6,9];
figure; bar2=bar([1:14], pi_cv_nalr(order), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
set(bar2,'BarWidth',0.4); bar1=bar([1:14], pi_cv_nonalr(order)); set(bar1,'BarWidth',0.2);
cm = jet(4); ch1=get(bar1,'children'); idx = [2,2,2,2,2,2,2,2,2,1,1,1,4,4];
set(ch1,'facevertexcdata',idx.'); colormap(cm);
 legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
xlim([0, 15]); ylim([0, 1.05]); title('Percent Correctness of DC-R at 6 dB SNR');
xlabel('Consonants', 'FontSize', 12); ylabel('Percent Correct', 'FontSize', 12);
set(gca,'XTick',[1:14], 'XTickLabel', stims(order)); set(gca,'YTick',[0:0.2:1]);

