% this script calculates the Pc functions for the 16 CVs used in the
% SNRLoss07 experiment. 

% compare the score of HI listener to that of the NH listeners

clear all; close all; clc

SL07 = load('SL07.mat'); % the hearing impaired data
MN64 = load('MN64sub.mat'); % the normal hearing data of the 96 utterances

markers_code = create_markers_code();

%% plot the PI functions for the 16 consonants
stims = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na'}; 

for i=1:length(stims)
    stims{i} = stims{i}(1:end-1);
    if stims{i}(1)=='x', 
        stims{i}=upper(stims{i}(2));
    end
    stims{i}=[stims{i}, 'a'];    
end

% calculate the PI functions for NH listeners
snr_NH = MN64.PE_CV{1}.conds;
snr_NH(end) = 24;
pi_NH = NaN(16, length(snr_NH)); 
for i=1:16
    pi_NH(i,:) = 1 - MN64.PE_CV{i}.score';
end

% calculate the score for HI listeners
for i=1:16
    snr_HI_L_NAL{i} = SL07.PE_CV_NAL{1,i}.conds; % left ear of Annette
    snr_HI_L_NAL{i}(end) = 24;
    pi_HI_L_NAL{i} = 1 - SL07.PE_CV_NAL{1,i}.score; 
    snr_HI_L_noNAL{i} = SL07.PE_CV_noNAL{1,i}.conds; % left ear of Annette
    snr_HI_L_noNAL{i}(end) = 24;
    pi_HI_L_noNAL{i} = 1 - SL07.PE_CV_noNAL{1,i}.score; 
end

% calculate the score for HI listeners
for i=1:16
    snr_HI_R_NAL{i} = SL07.PE_CV_NAL{2,i}.conds; % right ear of Annette
    snr_HI_R_NAL{i}(end) = 24;
    pi_HI_R_NAL{i} = 1 - SL07.PE_CV_NAL{2,i}.score; 
    snr_HI_R_noNAL{i} = SL07.PE_CV_noNAL{2,i}.conds; % right ear of Annette
    snr_HI_R_noNAL{i}(end) = 24;
    pi_HI_R_noNAL{i} = 1 - SL07.PE_CV_noNAL{2,i}.score;     
end

% make plots for Annette left ear
figure;
order = [1,2,12,11,14,15,10,7,3,13,8,9,4,16,5,6];
for i=1:16
    
    k = order(i);
    ax(k) = subplot(4,4,k);
    
    plot(snr_HI_L_NAL{i}, pi_HI_L_NAL{i}, '-rs', 'LineWidth', 1.5); hold on;
    plot(snr_HI_L_noNAL{i}, pi_HI_L_noNAL{i}, '--bo', 'LineWidth', 1.5); hold on;
    plot(snr_NH, pi_NH(i,:), '--k', 'LineWidth', 1.5); hold on;

    xlim([-24 26]); ylim([-0.05 1.05]);
    text(-20, 0.85, stims{i}); 
    
    set(gca, 'XTick', [-18:6:24], 'XTickLabel', '');
    set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', '');    
    
    if (k>12) 
        xlabel('SNR (dB)'); 
        set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
    end
    if (mod(k,4)==1)
        ylabel('Pc'); 
        set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'});            
    end

end
legend({'NAL-R'; 'no NAL-R'; 'Normal Hearing'});
sameaxis(ax, 4, 4);


% make plots for Annette right ear
figure;
order = [10,3,11,4,8,12,13,14,5,7,15,9,6,2,16,1];
for i=1:16
    k = order(i);
    ax(k) = subplot(4,4,k);
    
    plot(snr_HI_R_NAL{i}, pi_HI_R_NAL{i}, '-rs', 'LineWidth', 1.5); hold on;    
    plot(snr_HI_R_noNAL{i}, pi_HI_R_noNAL{i}, '--bo', 'LineWidth', 1.5); hold on;    
    plot(snr_NH, pi_NH(i,:), '--k', 'LineWidth', 1.5); hold on;

    xlim([-24 26]); ylim([-0.05 1.05]);
    text(-20, 0.85, stims{i}); 
    
    set(gca, 'XTick', [-18:6:24], 'XTickLabel', '');
    set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', '');    
    
    if (k>12) 
        xlabel('SNR (dB)'); 
        set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
    end
    if (mod(k,4)==1)
        ylabel('Pc'); 
        set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'});            
    end

end

legend({'NAL-R'; 'no NAL-R'; 'Normal Hearing'});

sameaxis(ax, 4, 4);

% %% classify the 16 consonants into several groups based on the Pc
% 
% % AS-L
% stims =       {'p','t','k','f','T','s','S','b','d','g','v','D','z','Z','m','n'}; 
% markercolor = ['g','b','r','r','y','g','g','r','b','r','r','y','g','g','r','g'];
% facecolor =   {'g','b','r','r','y','c','g','r','none','none','none','none','none','none','none','none'};
% marker = ['o','s','d','^','v','>','<','p','o','s','d','^','v','>','<','p'];
% 
% figure;
% for i=1:16
%     pi_NH_int = interp1(snr_NH, pi_NH(i,:), snr_HI_L_noNAL{i}, 'linear');
%     pi_lamda{i} = pi_HI_L_noNAL{i}./pi_NH_int;
%     plot(snr_HI_L_noNAL{i}, pi_lamda{i}, ['--' marker(i)], 'Color', markercolor(i), 'MarkerFacecolor', facecolor{i}, 'MarkerSize', 9); hold on;
% end
% xlim([-2 26]); ylim([-0.05 1.05]);
% xlabel('SNR (dB)', 'FontSize', 12); ylabel('Pc_{HI}/Pc_{NH}', 'FontSize', 12); 
% set(gca, 'XTick', [-12:6:24], 'XTickLabel', {'-12';'-6';'0';'6';'12';'18';'Q'});        
% set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'});            
% %text(0, 0.95, 'Subject AS (left ear)', 'Fontsize', 12, 'Fontweight', 'bold');
% title('Subject AS (left ear)', 'Fontsize', 12, 'Fontweight', 'bold');
% h = legend(stims,'Location','EastOutside'); set(h,'Interpreter','none');
% 
% % AS-R
% stims =       {'p','t','k','f','T','s','S','b','d','g','v','D','z','Z','m','n'}; 
% markercolor = ['g','b','g','g','y','g','g','g','b','r','g','y','b','g','g','g'];
% facecolor =   {'g','b','g','g','y','g','g','g','none','none','none','none','none','none','none','none'};
% marker = ['o','s','d','^','v','>','<','p','o','s','d','^','v','>','<','p'];
% 
% figure;
% for i=1:16
%     pi_NH_int = interp1(snr_NH, pi_NH(i,:), snr_HI_R_noNAL{i}, 'linear');
%     pi_lamda{i} = pi_HI_R_noNAL{i}./pi_NH_int;
%     plot(snr_HI_R_noNAL{i}, pi_lamda{i}, ['--' marker(i)], 'Color', markercolor(i), 'MarkerFacecolor', facecolor{i}, 'MarkerSize', 9); hold on;
% 
% end
% xlim([-2 26]); ylim([-0.05 1.05]);
% xlabel('SNR (dB)', 'FontSize', 12); ylabel('Pc_{HI}/Pc_{NH}', 'FontSize', 12); 
% set(gca, 'XTick', [-12:6:24], 'XTickLabel', {'-12';'-6';'0';'6';'12';'18';'Q'});        
% set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'});            
% %text(0, 0.95, 'Subject AS (right ear)', 'Fontsize', 12, 'Fontweight', 'bold');
% title('Subject AS (right ear)', 'Fontsize', 12, 'Fontweight', 'bold');
% h = legend(stims,'Location','EastOutside'); set(h,'Interpreter','none');
% 
% 
% % % make plots
% % %stims =      {'p','t','k','f','T','s','S','b','d','g','v','D','z','Z','m','n'}; 
% % markercolor = ['g','b','r','r','y','c','g','g','b','r','g','y','c','c','g','c'];
% % facecolor =   {'none','b','r','r','none','c','none','none','b','r','none','none','c','c','none','none'};
% % marker =      ['o','s','d','^','v','>','<','p','o','s','d','^','v','>','<','p'];
% % 
% % figure;
% % for i=1:16
% %     
% %     %plot_SNRloss(pi_HI(i,:), snr_HI, pi_NH(i,:), snr_NH, marker(i), markercolor(i), facecolor{i}, 0 ); hold on;
% % end
% % h = legend(stims,'Location','EastOutside'); set(h,'Interpreter','none');
% % t = text(-20,28, 'L1'); set(t,'FontSize',16)  
% % plot_SNRloss(pi_HI(i,:), snr_HI, pi_NH(i,:), snr_NH, marker(i), markercolor(i), facecolor{i}, 1 );

%% test the SNRLoss vs Pc_NH - Pc_HI function

stims =       {'p','t','k','f','T','s','S','b','d','g','v','D','z','Z','m','n'}; 
markercolor = ['g','b','r','r','r','g','g','g','b','r','g','r','b','b','r','g'];
facecolor =   {'g','b','r','r','r','g','g','g','none','none','none','none','none','none','none','none'};
marker =      ['o','s','d','^','v','>','<','p','o','s','d','^','v','>','<','p'];

% AS-L
figure;
for i=1:16
    plot_SNRLoss_vs_PILoss(pi_HI_L_noNAL{i}, snr_HI_L_noNAL{i}, pi_NH(i,:), snr_NH, stims{i}, marker(i), markercolor(i), facecolor{i}); hold on;
end
%h = legend(stims,'Location','EastOutside'); set(h,'Interpreter','none');
xlim([10 50]); ylim([-0.05 1.05]);
text(12, 0.95, 'AS-L', 'Fontsize', 14, 'Fontweight', 'bold');
%title('Subject AS (left ear)', 'Fontsize', 12, 'Fontweight', 'bold');

stims =       {'p','t','k','f','T','s','S','b','d','g','v','D','z','Z','m','n'}; 
markercolor = ['g','b','r','r','r','g','g','g','b','r','g','r','b','g','g','g'];
facecolor =   {'g','b','r','r','r','g','g','g','none','none','none','none','none','none','none','none'};
marker =      ['o','s','d','^','v','>','<','p','o','s','d','^','v','>','<','p'];

% AS-R
figure;
for i=1:16
    plot_SNRLoss_vs_PILoss(pi_HI_R_noNAL{i}, snr_HI_R_noNAL{i}, pi_NH(i,:), snr_NH, stims{i}, marker(i), markercolor(i), facecolor{i}); hold on;
end
%h = legend(stims,'Location','EastOutside'); set(h,'Interpreter','none');
xlim([10 50]); ylim([-0.05 1.05]);
text(12, 0.95, 'AS-R', 'Fontsize', 14, 'Fontweight', 'bold');
%title('Subject AS (right ear)', 'Fontsize', 12, 'Fontweight', 'bold');

