% this script calculates the Pc functions for the 16 CVs used in the
% SNRLoss07 experiment. 

% compare the score of HI listener to that of the NH listeners

clear all; close all; clc

SL07 = load('SL07.mat'); % the hearing impaired data
MN64 = load('MN64sub.mat'); % the normal hearing data of the 96 utterances

fontsize = 12;

markers_code = create_markers_code();

%% plot the PI functions for the 16 consonants
stims = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na'}; 

for i=1:length(stims)
    stims{i} = stims{i}(1:end-1);
    if stims{i}(1)=='x', 
        stims{i}=upper(stims{i}(2));
    end
    stims{i}=[stims{i}, 'a'];    
end

% calculate the PI functions for NH listeners
snr_NH = MN64.PE_CV{1}.conds;
snr_NH(end) = 24;
pi_NH = NaN(16, length(snr_NH)); 
for i=1:16
    pi_NH(i,:) = 1 - MN64.PE_CV{i}.score';
end

% calculate the score for HI listeners
for i=1:16
    snr_HI_L_NAL{i} = SL07.PE_CV_NAL{1,i}.conds; % left ear of Annette
    snr_HI_L_NAL{i}(end) = 24;
    pi_HI_L_NAL{i} = 1 - SL07.PE_CV_NAL{1,i}.score; 
    snr_HI_L_noNAL{i} = SL07.PE_CV_noNAL{1,i}.conds; % left ear of Annette
    snr_HI_L_noNAL{i}(end) = 24;
    pi_HI_L_noNAL{i} = 1 - SL07.PE_CV_noNAL{1,i}.score; 
end

% calculate the score for HI listeners
for i=1:16
    snr_HI_R_NAL{i} = SL07.PE_CV_NAL{2,i}.conds; % right ear of Annette
    snr_HI_R_NAL{i}(end) = 24;
    pi_HI_R_NAL{i} = 1 - SL07.PE_CV_NAL{2,i}.score; 
    snr_HI_R_noNAL{i} = SL07.PE_CV_noNAL{2,i}.conds; % right ear of Annette
    snr_HI_R_noNAL{i}(end) = 24;
    pi_HI_R_noNAL{i} = 1 - SL07.PE_CV_noNAL{2,i}.score;     
end

% make plots for pa, ta, ka
figure('Position',[1 1 600 700]); 
order = [1,2,3];
for k=1:length(order)
    
    i = order(k);
    ax(2*k-1) = subplot(2,3,2*k-1);
    
    plot(snr_HI_L_NAL{i}, pi_HI_L_NAL{i}, '-rs', 'LineWidth', 1.5); hold on;
    plot(snr_HI_L_noNAL{i}, pi_HI_L_noNAL{i}, '-bo', 'LineWidth', 1.5); hold on;
    plot(snr_NH, pi_NH(i,:), '--k', 'LineWidth', 1.5); hold on;
    if (k==1)
        title('AS-L', 'Fontsize',  fontsize+2);
    end
    xlim([-24 26]); ylim([-0.05 1.05]);
    text(-20, 0.9, stims{i}, 'Fontsize', fontsize);     

    ylabel('Pc', 'Fontsize',  fontsize);
    set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'});            
    set(gca, 'XTick', [-18:6:24], 'XTickLabel', '');
    
    if (k>2) 
        xlabel('SNR (dB)', 'Fontsize',  fontsize); 
        set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
    end
    
    ax(2*k) = subplot(2,3,2*k);
    plot(snr_HI_R_NAL{i}, pi_HI_R_NAL{i}, '-rs', 'LineWidth', 1.5); hold on;    
    plot(snr_HI_R_noNAL{i}, pi_HI_R_noNAL{i}, '--bo', 'LineWidth', 1.5); hold on;    
    plot(snr_NH, pi_NH(i,:), '--k', 'LineWidth', 1.5); hold on;    
    if (k==1)
        title('AS-R', 'Fontsize',  fontsize+2);
    end

    xlim([-24 26]); ylim([-0.05 1.05]);
    text(-20, 0.9, stims{i}, 'Fontsize',  fontsize); 
    
    set(gca, 'XTick', [-18:6:24], 'XTickLabel', '');
    set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', '');    
    
    if (k>2) 
        xlabel('SNR (dB)', 'Fontsize',  fontsize); 
        set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
    end

end
legend({'NAL-R'; 'no NAL-R'; 'Normal Hearing'});
% adjust the position and size of the axes
set(ax(1), 'Position', [0.1, 0.66, 0.4, 0.28]);
set(ax(2), 'Position', [0.5, 0.66, 0.4, 0.28]);    
set(ax(3), 'Position', [0.1, 0.38, 0.4, 0.28]);
set(ax(4), 'Position', [0.5, 0.38, 0.4, 0.28]);
set(ax(5), 'Position', [0.1, 0.10, 0.4, 0.28]);
set(ax(6), 'Position', [0.5, 0.10, 0.4, 0.28]);

%% make plots for ba, da, ga
figure('Position',[1 1 600 700]); 
order = [8,9,10];
for k=1:length(order)
    
    i = order(k);
    ax(2*k-1) = subplot(2,3,2*k-1);
    
    plot(snr_HI_L_NAL{i}, pi_HI_L_NAL{i}, '-rs', 'LineWidth', 1.5); hold on;
    plot(snr_HI_L_noNAL{i}, pi_HI_L_noNAL{i}, '-bo', 'LineWidth', 1.5); hold on;
    plot(snr_NH, pi_NH(i,:), '--k', 'LineWidth', 1.5); hold on;
    if (k==1)
        title('AS-L', 'Fontsize',  fontsize+2);
    end
    xlim([-24 26]); ylim([-0.05 1.05]);
    text(-20, 0.9, stims{i}, 'Fontsize', fontsize);     

    ylabel('Pc', 'Fontsize',  fontsize);
    set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'});            
    set(gca, 'XTick', [-18:6:24], 'XTickLabel', '');
    
    if (k>2) 
        xlabel('SNR (dB)', 'Fontsize',  fontsize); 
        set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
    end
    
    ax(2*k) = subplot(2,3,2*k);
    plot(snr_HI_R_NAL{i}, pi_HI_R_NAL{i}, '-rs', 'LineWidth', 1.5); hold on;    
    plot(snr_HI_R_noNAL{i}, pi_HI_R_noNAL{i}, '--bo', 'LineWidth', 1.5); hold on;    
    plot(snr_NH, pi_NH(i,:), '--k', 'LineWidth', 1.5); hold on;    
    if (k==1)
        title('AS-R', 'Fontsize',  fontsize+2);
    end

    xlim([-24 26]); ylim([-0.05 1.05]);
    text(-20, 0.9, stims{i}, 'Fontsize',  fontsize); 
    
    set(gca, 'XTick', [-18:6:24], 'XTickLabel', '');
    set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', '');    
    
    if (k>2) 
        xlabel('SNR (dB)', 'Fontsize',  fontsize); 
        set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
    end

end
legend({'NAL-R'; 'no NAL-R'; 'Normal Hearing'});
% adjust the position and size of the axes
set(ax(1), 'Position', [0.1, 0.66, 0.4, 0.28]);
set(ax(2), 'Position', [0.5, 0.66, 0.4, 0.28]);    
set(ax(3), 'Position', [0.1, 0.38, 0.4, 0.28]);
set(ax(4), 'Position', [0.5, 0.38, 0.4, 0.28]);
set(ax(5), 'Position', [0.1, 0.10, 0.4, 0.28]);
set(ax(6), 'Position', [0.5, 0.10, 0.4, 0.28]);


