% this script calculates the Pc functions for the 16 CVs used in the
% SNRLoss07 experiment. 

% compare the score of HI listener to that of the NH listeners

clear all; close all; clc

SL07 = load('SL07.mat'); % the hearing impaired data
MN64 = load('MN64sub.mat'); % the normal hearing data of the 96 utterances

%% plot the PI functions for the 16 consonants
%         1    2    3    4    5     6    7     8    9    10   11   12    13   14   15   16 
stims = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na'}; 

for i=1:length(stims)
    stims{i} = stims{i}(1:end-1);
    if stims{i}(1)=='x', 
        stims{i}=upper(stims{i}(2));
    end
    stims{i}=[stims{i}, 'a'];    
end

sublist = {'AS-L';'AS-R';'DC-L';'DC-R';'MC-L';'MC-R';'MJ-L';'MJ-R';'BD-L';'BD-R'};
NALexist = [1;     1;     1;     1;     1;     1;     0;     0;     0;     0    ];

cv_list = [2,9, 3,10, 1,8]; style = '32'; % stop consonants;
%cv_list = [4,11, 6,13, 7,14]; style = '32'; % fricative consonants (Ta and Da are not included);
%cv_list = [15,16]; style = '12'; % nasal consonants;

for j=1:length(sublist)
    disp(sublist{j});
    for i=1:length(cv_list)
        k = cv_list(i);

        Pc_NH{i}.stim = stims{k};
        Pc_NH{i}.snr = MN64.PE_CV{k}.conds; Pc_NH{i}.snr(end) = 24;
        Pc_NH{i}.pc = 1 - MN64.PE_CV{k}.score';
        Pc_NH{i}.stim = stims{k};
        Pc_HI{i}.snr = SL07.PE_CV_noNAL{j,k}.conds; Pc_HI{i}.snr(end) = 24;
        Pc_HI{i}.pc = 1 - SL07.PE_CV_noNAL{j,k}.score;
        if NALexist(j)
            Pc_HI{i}.snrNAL = SL07.PE_CV_NAL{j,k}.conds; Pc_HI{i}.snrNAL(end) = 24;
            Pc_HI{i}.pcNAL = 1 - SL07.PE_CV_NAL{j,k}.score; 
        else
            Pc_HI{i}.snrNAL = NaN;
            Pc_HI{i}.pcNAL = NaN; 
        end
    end
    hfig = plot_Pc_HI_vs_NH(Pc_NH, Pc_HI, style);
    set(hfig,'PaperPositionMode','auto');
    saveas(hfig, ['./results/' sublist{j} '-stop.fig'], 'fig'); 
    
    pause(0.5); close(hfig);
    clear Pc_NH Pc_HI
end
