% This program make the Pc vs num_of_presents function for the 16 consonant
% sounds at all SNR conditions.

close all; 
clear all; clc

load SPDB_SL07.mat

markers_code = create_markers_code();

data = query(SPDB_SL07, 'compensate', 'NAL-R');

stims = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na'}; 
%SNRs = [-12, -6, 0, 6, 12, 18, 100];
SNRs = [100, 18, 12, 6, 0, -6, -12];

nStim = length(stims);
nSNR = length(SNRs);
nRecords = length(data{1});
nPresent = 30;

hash = zeros(nRecords, 3); % the 1 - stim, 2 - nPresent, 3 - SNR

for i=1:length(stims)
    p = find(strcmp(data{11}, stims{i}));
    hash(p,1) = i;
end

for i=1:nSNR
    p = find(data{15}==SNRs(i));
    hash(p,3) = i;
end

stat_present = zeros(nStim, nSNR);
for i=1:nRecords
    row_no = hash(i,1); col_no = hash(i,3);
    stat_present(row_no, col_no) = stat_present(row_no, col_no) + 1;
    hash(i,2) = stat_present(row_no, col_no);
end

% create a 3D matrix for all the responses
R = NaN(16, 36, 7);
for i=1:nRecords
    R(hash(i,1), hash(i,2), hash(i,3)) = data{23}(i);
end

%%% create plots of Pc vs nPresentation under different SNR conditions
% for i=1:nSNR
%     figure(i);
%     pi_snr = cumsum(R(:,:,i), 2);
%     
%     for j=1:nStim
%         plot([1:36], pi_snr(j,:)./[1:36], markers_code{j}.shape, 'Color', markers_code{j}.color,...
%              'MarkerFacecolor', markers_code{j}.facecolor, 'MarkerSize', 4 ,'LineWidth',1.5 ); hold on
%     end
%     xlim([0, 36]); ylim([-0.05, 1.05]);
%     title(['SNR = ' num2str(SNRs(i)) '(dB)']);
%     legend(stims, 'Location','EastOutside');
% end

figure('Position', [50, 50, 550, 400]);

% create plots of Pc vs nPresentations for 16 CVs
for i=1:nStim

    R_stim = NaN(36, 7);
    for j=1:36
        for k=1:7
            R_stim(j,k) = R(i,j,k);
        end
    end
    
    pi_snr = cumsum(R_stim, 1);
    pi_snr = pi_snr';
    
    clf; 
    for j=1:nSNR
        plot([1:36], pi_snr(j,:)./[1:36], markers_code{j}.shape, 'Color', markers_code{j}.color,...
             'MarkerFacecolor', markers_code{j}.facecolor, 'MarkerSize', 4 ,'LineWidth',1.5 ); hold on
    end
    
    xlim([0, 36]); ylim([-0.05, 1.05]);
    xlabel('Num of Presentations'); ylabel('Pc(%)');
    title(fix_TSDZ(stims{i},1));
    legend(num2str(SNRs'), 'Location','EastOutside');
    
    saveas(gcf, ['./results/' stims{i} '.eps'], 'psc2');
end
            
