% this script measures the dB SPL of speech cue by comparing the peak 
% amplitude (STFT coefficient) of the cue to that of the 1 kHz tone

% dB SPL (cue peak) = 20*log10(abs(cue_peak_STFT)/abs(stimulus_ave_STFT)) +
% dB SPL(stimulus)

clear all; close all; clc;

% interpolate for the actual scalefactor
peak_amp_MCL = 0.1; 
wsize = 20; % 20ms

load vowel_start_time.mat;
utter_list = get_utter_list_SNRLoss07();

% generate the 1 kHz tone and normalize it to 78 dB SPL (amp = 0.1)
f = 1000; fs = 16000; dur = 5;
tone = generate_tone(f, dur, fs);
tone = 0.1*tone; % amplitude = 0.1

[T, Tlen] = STFT(tone, fs*wsize/1000); 

tone_peak_amp = max(max(abs(T))); % the STFT amplitude for 78 dB SPL

fp = fopen('cue_db_SPL_missed.txt','w');
fprintf(fp, 'utter   cue_dB_SPL    cue_freq  tone_dBS_SPL \n');
    
figure('Position',[1 1 800 600]); 
sound_list = [12,17,21,22,25,26,29,38,77,92,95];
for i=1:length(sound_list)
    utter = utter_list{sound_list(i)}; disp(utter); clf;
    
    [s, Fs] = wavread(['./wavs/s_' utter '.wav']);
    % get rid of the useless samples
    s = remove_dead_spots(s, Fs); 
    s = s/std(s)*peak_amp_MCL*0.707; % first scale it to the level of MCL of clean speech    

    [S, xlen] = STFT(s, Fs*wsize/1000); 
    S = abs(flipud(S')); 
    imagesc(S); hold on;
    title(utter);
    freqs = [8:-0.5:0];
    freq_pos = 512-freqs/8*512;
    set(gca, 'YTick', freq_pos, 'YTickLabel', freqs);
    
    % manually select the feature area
    xy = []; 
    n = 0;
    % Loop, picking up the points.
    but = 1;
    h_points = [];
    
    while but == 1
        [xi,yi,but] = ginput(1);
        h_points(end+1) = plot(xi,yi,'r.', 'MarkerSize', 6);
        n = n+1;
        xy(:,n) = round([xi;yi]);
    end

    cue = S([xy(2,1):xy(2,2)], [xy(1,1):xy(1,2)]);
    
    tone_peak_amp = tone_peak_amp; 
    cue_peak_amp = max(max(cue));
    
    cue_freq = (512 - mean(xy(2,:)))/512*8000;
    
    tone_dB_SPL = 78 + 2;
    cue_dB_SPL = 78 + 20*log10(cue_peak_amp/tone_peak_amp) + 2;
    
    fprintf(fp,'%d %s\n', i, [utter '   ' num2str(cue_dB_SPL) '	' num2str(cue_freq) '	' num2str(tone_dB_SPL)]);
    
end
fclose(fp);
