% this script analysis the perception data of enhanced speech (SNRLoss07RE1) for Normal Hearing listeners as a control.
%
% The SNRLoss07EN experiment is focused on the two sounds, /ka/s and /ga/s, that have been shown to be problematic for Annette
% by previous experiment. The purpose of the experiment is to see whether we can improve the recognition score for these two 
% consonants by manipulating the features. The experiment has been conducted for multiple rounds:
%
% Round-4: SNRLoss07RE1
% This experiment combines the round-2 and round-3 techniques, i.e, removing /ta/ or /da/ cues in /ka/ and /ga/ and boosting the 
% /ka/, /ga/ features.


close all; clear all; clc;

load SPDB_SL07EN.mat; % for detail of the 

SNRs = [-9, -3]; nSNR = 2;

NH_listeners = {'FL';'lobdellb';'AK'};
%NH_listeners = {'AK'};
nSub = length(NH_listeners);

%% create CM for the forth round: SNRLoss07RE1
% ka_1 means the /ta/ feature in /ka/ removed, same for ga_1; ka_2 means
% /ta/ feature removed plus /ka/ feature boosted by 2 (6dB); ka_4 means
% /ta/ feature removed plus /ka/ feature boosted by 4 (12dB).
stims = {'pa','ta','ka','ka_1','ka_2','ka_4','ba','da','ga','ga_1','ga_2','ga_4'}; 
resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'}; 

for k=1:nSub
    cur_sub = NH_listeners{k};
    for i=1:nSNR
        cur_snr = SNRs(i);
        
        % ---- calculate CM for the data with NAL-R ----
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr);	% retrieve data at a given snr

        if ~isempty(data{1})        
            % replace stims with utter for the enhanced speech, for example, ka to ka_10;
            for j=1:length(data{11})
                p = strfind(data{2}{j}, '_');
                data{11}{j} = data{2}{j}(p(2)+1:end);
            end
        
            CM{k,i} = calc_CM_en(data, stims, resps, ['SNRLoss07RE1-NH-SNR=' num2str(cur_snr) 'dB_' cur_sub]);  % calculate confusion matrix
            tab_mini = CM{k,i}.table(:,[1,2,3,8,9,10]); resps_mini = CM{k,i}.resps([1,2,3,8,9,10]);
            cm2tex(tab_mini, ['CM_mini_' CM{k,i}.title '.tex'], CM{k,i}.stims, resps_mini); % output to Latex document    
        end
        
    end
end

for i=1:nSNR
    tab_mini_all{i} = CM{1,i}.table(:,[1,2,3,8,9,10]);
    for k=2:nSub
        tab_mini_all{i} = tab_mini_all{i} + CM{k,i}.table(:,[1,2,3,8,9,10]);
    end
    cm2tex(tab_mini_all{i}, ['CM_mini_NH' num2str(SNRs(i)) '.tex'], CM{1,1}.stims, resps_mini); % output to Latex document    
end




