% this script analysis the perception data of enhanced speech.


load SPDB_SL07EN_old.mat; % SPDB_SL0707EN_old.mat contains Annette's data on enhanced speech with the gain being 0, 10, and 50, which proved to be too big.

% calculate the CMs for the data
stims = {'pa','ta','ka','ka_0','ka_10','ka_50','ba','da','ga','ga_0','ga_10','ga_50'}; % 10, 50 proved to be too big a gain
stims1 = {'pa','ta','ka','ka_2','ka_4','ka_6','ba','da','ga','ga_2','ga_4','ga_6'}; % 2, 4, 6 could be better
resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'}; 

nSNR = length(Index_SL07EN.snr);
nSub = length(Index_SL07EN.listener);

for k=1:nSub
    cur_sub = Index_SL07EN.listener{k};
    for i=1:nSNR
        cur_snr = Index_SL07EN.snr(i);
        
        % ---- calculate CM for the data without NAL-R ----
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NaN' );	% retrieve data at a given snr

        % replace stims with utter for the enhanced speech, for example, ka to ka_10;
        for j=1:length(data{11})
            data{11}{j} = data{2}{j}(6:end);
        end
        
        CM = calc_CM_en(data, stims, resps, ['SNRLoss07EN-No-NAL-R-SNR=' num2str(cur_snr) 'dB_' cur_sub]);  % calculate confusion matrix
        cm2tex(CM.table, ['CM' CM.title '.tex'], CM.stims, CM.resps); % output to Latex document    
        
        % ---- calculate CM for the data with NAL-R ----
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R' );	% retrieve data at a given snr

        % replace stims with utter for the enhanced speech, for example, ka to ka_10;
        for j=1:length(data{11})
            data{11}{j} = data{2}{j}(6:end);
        end
        
        CM = calc_CM_en(data, stims, resps, ['SNRLoss07EN-NAL-R-SNR=' num2str(cur_snr) 'dB_' cur_sub]);  % calculate confusion matrix
        cm2tex(CM.table, ['CM' CM.title '.tex'], CM.stims, CM.resps); % output to Latex document    
        
    end
end

%% compare the score of /ka/s and /ga/s from six talkers and find out which 
% are the /ka/s and /ga/s that the subject can hear. consider the original
% sounds only.

kas = { 'm111_ka',	'f103_ka',	'm118_ka',	'f105_ka',	'm114_ka',	'f119_ka'};
gas = { 'm111_ga',	'f119_ga',	'm104_ga',	'f108_ga',	'm112_ga',	'f109_ga'};  

for k=1:nSub
    
    pi_ka_noNAL = zeros(6,2); % pi of /ka/s from 6 talkers at 12 and 100dB SNR
    pi_ga_noNAL = zeros(6,2); % pi of /ga/s from 6 talkers at 12 and 100dB SNR
    
    pi_ka_NAL = zeros(6,2); % pi of /ka/s from 6 talkers at 12 and 100dB SNR
    pi_ga_NAL = zeros(6,2); % pi of /ga/s from 6 talkers at 12 and 100dB SNR    
    
    cur_sub = Index_SL07EN.listener{k};
    for i=1:nSNR
        cur_snr = Index_SL07EN.snr(i);
        for j=1:6
            cur_utter = kas{j};  % ka
            data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NaN', 'utter', cur_utter);
            pi_ka_noNAL(j,i) = sum(data{23})/(length(data{23})+eps);
            data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R', 'utter', cur_utter);
            pi_ka_NAL(j,i) = sum(data{23})/(length(data{23})+eps);
            
            cur_utter = gas{j};  % ga
            data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NaN', 'utter', cur_utter);
            pi_ga_noNAL(j,i) = sum(data{23})/(length(data{23})+eps);            
            data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R', 'utter', cur_utter);
            pi_ga_NAL(j,i) = sum(data{23})/(length(data{23})+eps);            
        end
    end
    
    pi_all_noNAL{k,1} = pi_ka_noNAL;
    pi_all_noNAL{k,2} = pi_ga_noNAL;
    
    pi_all_NAL{k,1} = pi_ka_NAL;
    pi_all_NAL{k,2} = pi_ga_NAL;
    
end

