% This program plots the average Pe over all stimuli as a function of snr
clear all; clc; close all;

load SPDB_SL07.mat;

lab_row = {'pa';'ta';'ka';'fa';'xta';'sa';'xsa';'ba';'da';'ga';'va';'xda';'za';'xza';'ma';'na'};
lab_col = {'pa';'ta';'ka';'fa';'xta';'sa';'xsa';'ba';'da';'ga';'va';'xda';'za';'xza';'ma';'na';'?'};

stims = {'pa';'ta';'ka';'fa';'Ta';'sa';'Sa';'ba';'da';'ga';'va';'Da';'za';'Za';'ma';'na'};

fontsize = 12;

%% calculate Confusion Matrix for subject DC
%------------left ear, SNR = 100dB----------------
% left ear
data = query(SPDB_SL07, 'listener', 'DC_L', 'snr', 100, 'compensate', 'NaN');
cm = calc_CM(data, 'CM of DC-L at 100 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_L_100dB_noNAL.tex', lab_row, lab_col);
pi_cv_nonalr = diag(cm.table)/12; 

data = query(SPDB_SL07, 'listener', 'DC_L', 'snr', 100, 'compensate', 'NAL-R');
cm = calc_CM(data, 'CM of DC-L at 100 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_L_100dB_NAL.tex', lab_row, lab_col);
pi_cv_nalr = diag(cm.table)/12; 

disp(['DC-L at 100dB SNR: ' num2str(mean(pi_cv_nalr-pi_cv_nonalr))]);
% make bar plots for the comparison of NAL-R vs non NAL-R
order = [1,8,3,10,2,9];
figure('Position', [100,100,450,350]); 
bar2=bar([1:6], pi_cv_nalr(order), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
set(bar2,'BarWidth',0.4); bar1=bar([1:6], pi_cv_nonalr(order)); set(bar1,'BarWidth',0.2);
cm = jet(4); ch1=get(bar1,'children'); idx = [2,2,2,2,4,4];
set(ch1,'facevertexcdata',idx.'); colormap(cm);
legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
xlim([0, 7]); ylim([0, 1.05]); title('Probability Correct of DC-L in Quiet', 'FontSize', 12);
xlabel('Consonants', 'FontSize', 12); ylabel('Probability Correct', 'FontSize', 12);
set(gca,'XTick',[1:6], 'XTickLabel', stims(order),'FontSize', 12); set(gca,'YTick',[0:0.2:1],'FontSize', 12);

%------------left ear, SNR = 6dB----------------
data = query(SPDB_SL07, 'listener', 'DC_L', 'snr', 6, 'compensate', 'NaN');
cm = calc_CM(data, 'CM of DC-L at 6 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_L_6dB_noNAL.tex', lab_row, lab_col);
pi_cv_nonalr = diag(cm.table)/12; 

data = query(SPDB_SL07, 'listener', 'DC_L', 'snr', 6, 'compensate', 'NAL-R');
cm = calc_CM(data, 'CM of DC-L at 6 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_L_6dB_NAL.tex', lab_row, lab_col);
pi_cv_nalr = diag(cm.table)/12; 

disp(['DC-L at 6dB SNR: ' num2str(mean(pi_cv_nalr-pi_cv_nonalr))]);
% make bar plots for the comparison of NAL-R vs non NAL-R
order = [1,8,3,10,2,9];
figure('Position', [100,100,450,350]); 
bar2=bar([1:6], pi_cv_nalr(order), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
set(bar2,'BarWidth',0.4); bar1=bar([1:6], pi_cv_nonalr(order)); set(bar1,'BarWidth',0.2);
cm = jet(4); ch1=get(bar1,'children'); idx = [4,2,2,2,2,2];
set(ch1,'facevertexcdata',idx.'); colormap(cm);
legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
xlim([0, 7]); ylim([0, 1.05]); title('Probability Correct of DC-L at 6 dB SNR', 'FontSize', 12);
xlabel('Consonants', 'FontSize', 12); ylabel('Probability Correct', 'FontSize', 12);
set(gca,'XTick',[1:6], 'XTickLabel', stims(order),'FontSize', 12); set(gca,'YTick',[0:0.2:1],'FontSize', 12);

%--------------right ear, SNR = 100dB-------------
data = query(SPDB_SL07, 'listener', 'DC_R', 'snr', 100, 'compensate', 'NaN');
cm = calc_CM(data, 'CM of DC-R at 100 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_R_100dB_noNAL.tex', lab_row, lab_col);
pi_cv_nonalr = diag(cm.table)/12; 

data = query(SPDB_SL07, 'listener', 'DC_R', 'snr', 100, 'compensate', 'NAL-R');
cm = calc_CM(data, 'CM of DC-R at 100 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_R_100dB_NAL.tex', lab_row, lab_col);
pi_cv_nalr = diag(cm.table)/12; 

disp(['DC-R at 100dB SNR: ' num2str(mean(pi_cv_nalr-pi_cv_nonalr))]);
% make bar plots for the comparison of NAL-R vs non NAL-R
order = [1,8,3,10,2,9];
figure('Position', [100,100,450,350]); 
bar2=bar([1:6], pi_cv_nalr(order), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
set(bar2,'BarWidth',0.4); bar1=bar([1:6], pi_cv_nonalr(order)); set(bar1,'BarWidth',0.2);
cm = jet(4); ch1=get(bar1,'children'); idx = [2,2,2,2,4,4];
set(ch1,'facevertexcdata',idx.'); colormap(cm);
legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
xlim([0, 7]); ylim([0, 1.05]); title('Probability Correct of DC-R in Quiet', 'FontSize', 12);
xlabel('Consonants', 'FontSize', 12); ylabel('Probability Correct', 'FontSize', 12);
set(gca,'XTick',[1:6], 'XTickLabel', stims(order),'FontSize', 12); set(gca,'YTick',[0:0.2:1],'FontSize', 12);

%--------------right ear, SNR = 6dB-------------
data = query(SPDB_SL07, 'listener', 'DC_R', 'snr', 6, 'compensate', 'NaN');
cm = calc_CM(data, 'CM of DC-R at 6 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_R_6dB_noNAL.tex', lab_row, lab_col);
pi_cv_nonalr = diag(cm.table)/12; 

data = query(SPDB_SL07, 'listener', 'DC_R', 'snr', 6, 'compensate', 'NAL-R');
cm = calc_CM(data, 'CM of DC-R at 6 dB SNR without NAL-R');
cm2tex(cm.table(:,1:17), 'CM_DC_R_6dB_NAL.tex', lab_row, lab_col);
pi_cv_nalr = diag(cm.table)/12; 

disp(['DC-R at 6dB SNR: ' num2str(mean(pi_cv_nalr-pi_cv_nonalr))]);
% make bar plots for the comparison of NAL-R vs non NAL-R
order = [1,8,3,10,2,9];
figure('Position', [100,100,450,350]); 
bar2=bar([1:6], pi_cv_nalr(order), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
set(bar2,'BarWidth',0.4); bar1=bar([1:6], pi_cv_nonalr(order)); set(bar1,'BarWidth',0.2);
cm = jet(4); ch1=get(bar1,'children'); idx = [2,2,2,2,2,4];
set(ch1,'facevertexcdata',idx.'); colormap(cm);
legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
xlim([0, 7]); ylim([0, 1.05]); title('Probability Correct of DC-R at 6 dB SNR', 'FontSize', 12);
xlabel('Consonants', 'FontSize', 12); ylabel('Probability Correct', 'FontSize', 12);
set(gca,'XTick',[1:6], 'XTickLabel', stims(order),'FontSize', 12); set(gca,'YTick',[0:0.2:1],'FontSize', 12);

