%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program calculate the Confusion Matrix.
% Input:    data - speech perception data to be analyzed
%           title - the name of the confusion matrix
% Output:   CM - confusion matrix
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-28
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function CM = calc_CM(data, title)

% the order introduced by MN55 paper
stims = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na'}; 
resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?','a'}; 

CM = NaN;

% replace Stimuli with their row num for fast processing.
row_pos = zeros(size(data)); 
for j=1:length(stims) % convert Resp to an address in Table
    p = find(strcmp(data{11}, stims{j}));
    row_pos(p) = j;
end
p = find(row_pos==0);   
if ~isempty(p)
    disp('error in generating CM! row_pos = 0! Please check data');
    CM = NaN;
    return;
end

% replace Resp with their colume no for fast processing. Those
% responses not included in Resps, for example, xja, are attributed to be ?
column_pos = zeros(size(data)); 
for j=1:length(resps) % convert Resp to an address in Table
    p = find(strcmp(data{20}, resps{j}));
    column_pos(p) = j;
end
p = find(column_pos==0);   
column_pos(p) = 17;    

% the confusion pattern table 
table = zeros(length(stims), length(resps));
nrecord = length(data{1});
for i=1:nrecord
    table(row_pos(i),column_pos(i)) = table(row_pos(i),column_pos(i)) + 1;
end

clear CM;

CM.title = title;
CM.stims = stims;
CM.resps = resps;
CM.table = table;


