%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program calculate the Pe for each utterance as a 
% function of filtering, truncation, snr conditions.
% Input:    data - speech perception data of an particular utterance to be analyzed
%           utter - the utterance to be analyzed
%           type - time, frequency or snr
% Output:   CP - confusion pattern as a function of time, frequency
%                or snr condition.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-28
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Pe = calc_Pe(data, cv, caption, type)

CP = calc_CP(data, caption, type);

target_col = find(strcmp(cv, CP.resps));

score = NaN(size(CP.conds));

for i=1:length(CP.conds)
    nPresent = sum(CP.table(i,:))+eps;
    if ~isnan(nPresent)
        score(i) = CP.table(i,target_col)/nPresent;
    end
end

Pe.npresent = nPresent;
Pe.caption = CP.caption;
Pe.conds = CP.conds;
Pe.xlabel = CP.xlabel;
Pe.ylabel = 'Percent Error';
Pe.score = 1-score;
