%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program calculate the feature mask for each utterance based on 
% the filtering and truncation data.
% Input:    utter - the utterance to be analyzed    
% Output:   feature - feature mask of the given utterance
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-28
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function calc_feature_mask(SPDB, utter, snr, vow_start_time)

cv = utter(6:end);
% first calculate the Pe as a function of time
data = query(SPDB, 'exp', 'trunc07', 'utter', utter);
pe_tem = calc_Pe(data, cv, utter, 'time');
score_time = pe_tem.score;
conds_time = pe_tem.conds;

% first calculate the Pe as a function of time
%data = query(SPDB, 'exp', 'trunc07', 'utter', utter);
cp_time = calc_CP(data, utter, 'time');
    
% then calculate the Pe as a function of high-pass and low-pass cutoff frequencies
data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'HP');
pe_tem = calc_Pe(data, cv, utter, 'hpf');
score_hpf = pe_tem.score;
freqs_hpf = pe_tem.conds;

data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'LP');
pe_tem = calc_Pe(data, cv, utter, 'lpf');
score_lpf = pe_tem.score;
freqs_lpf = pe_tem.conds;

data = query(SPDB, 'exp', 'MN16R', 'utter', utter);
pe_tem = calc_Pe(data, cv, utter, 'snr');
score_snr = pe_tem.score;
snrs = pe_tem.conds;
snrs(end) = 24;

% use the same frequencies for the HP and LP score
freqs_all = unique(sort([freqs_lpf; freqs_hpf]));

pe_lp = interp1(freqs_lpf, score_lpf, freqs_all, 'nearest', 'extrap');
pe_hp = interp1(freqs_hpf, score_hpf, freqs_all, 'nearest', 'extrap');

pe_lp = pe_lp+eps; pe_hp = pe_hp+eps;
% weights for all frequency segments
weights = NaN(2, length(freqs_all)); 
weights(1,2:end) = -log10(pe_lp(2:end)./pe_lp(1:end-1));    % lpf weights ekL
weights(2,1:end-1) = -log10(pe_hp(1:end-1)./pe_hp(2:end));    % hpf weights ekH

weights(1,1) =  weights(2,1);
weights(2,end) =  weights(1,end);

% set those less than zero to be zero
p = weights < 0; weights(p) = 0;
weight_sum = (weights(1,:) + weights(2,:))/2;    % weight_freq = weight_lpf + weight_hpf

x_freqs_all = greenwood_inverse(freqs_all);
x_mid_freqs = (x_freqs_all(1:end-1) + x_freqs_all(2:end))/2;

% map the frequency to auditory frequency bands
x_apex = greenwood_inverse(179.46);
x_base = greenwood_inverse(7520);

x_AI = linspace( x_base, x_apex, 200);

% frequency importance function 
importance_freq.weight = interp1(freqs_all, weight_sum, x_AI, 'nearest', 'extrap');
importance_freq.x = x_AI;

% then deal with the time importance function

    utterance = ['s_' utter];
    
    %%%%%%-------AI-gram v4.2--------%%%%%%
    [s, Fs, bits]= wavread(['./wavs/' utterance '.wav']);
    speech_data.fs = Fs;
    speech_data.type = 'speechonly';
    speech_data.snr = 0;
    speech_data.signal = s;
    
    cd('./aigram_v4.2');
        
    filterbank = 'erbaligned';
    %%1. Prepare the speech and generate the noise at 0 dB SNR like this:            
    [ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn16r( speech_data );  
    %%2. Compute the auditory representation of the speech-only signal like this:
    [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
    %%3. Compute the noise statistics:
    [ E_ni2, E_ni ] = compute_noise_stats('mn16r', filterbank, std(noise_only) );       

    %%4. Scale the noise statistics to the appropriate SNR:
    E_ni2_scaled = E_ni2 * 10^(-snr/10);
    E_ni_scaled = E_ni * 10^(-snr/20);            
    %%5. Make AI-gram out of existing information:
    [aigram, cc_d, time_axis, freq_axis ] = form_image_ai0607c( level3, 'mn16r', filterbank, E_ni2_scaled, E_ni_scaled);     
        
    cd('..');
    
    AIgram.ccgram = aigram;
    AIgram.time_axis = time_axis;
    AIgram.freq_axis = freq_axis;
    
    % align the time importance function with AIgram
    % This number only works when Fs = 16000, for other Fs re-calibration is
    % needed to get this number
    [shift, error] = speech_match(s, speech_only, Fs) ;
    delay = shift + 736;    % delay of the auditory filter introduced by AIgram 4.2
    timedelay = delay/Fs*100;
    cp_time.conds = cp_time.conds + timedelay;
    conds_time = conds_time + timedelay;
    
    start_time = min(AIgram.time_axis);
    end_time = max(AIgram.time_axis);
    
    [nband, nframe] = size(AIgram.ccgram);

    trunc_start_time = min(cp_time.conds)/100; 
    trunc_end_time = max(cp_time.conds)/100;

    trunc_start_col = round((trunc_start_time-start_time)/(end_time-start_time)*nframe);
    trunc_end_col = round((trunc_end_time-start_time)/(end_time-start_time)*nframe);
    
    importance_time.x = AIgram.time_axis;
    importance_time.weight = zeros(nframe, 1);
    for i=trunc_start_col:trunc_end_col
        importance_time.weight(i) = interp1(cp_time.conds/100, 1-score_time, importance_time.x(i), 'linear');
    end
                         
    %feature = sqrt(importance_freq.weight'*importance_time.weight);
    feature.mask = importance_freq.weight'*importance_time.weight';
    feature.time_axis = importance_time.x;
    feature.freq_axis = importance_freq.x;
    
%     subplot(211), plot_AI_gram(AIgram, utterance, snr); hold on; 
%     subplot(212), plot_feature(feature, utterance); %hold on; contour(flipud(feature));

    % make plots for feature hunting
    % convert the cut-off frequecies into auditory filter band number
    S = load('standard.mat');
    frequencies = S.frequencies;
    how_many_filters = length(frequencies);

    for i=1:length(freqs_all)
        temp = abs(frequencies-freqs_all(i));
        p = find(temp==min(temp));
        AI_BandNo(i) = p; % the AI band no corresponds to the cut-off frequency
    end

    ax(1) = subplot(221), plot_CP(cp_time);  ylim([1/16, 1.02]); grid off;
                  xlim([1*0.25, nframe*0.25]); title('TR07','FontSize',12);
                  set(gca, 'XTick', [0:10:round(nframe*0.25)], 'XTickLabel', '', 'FontSize', 9); 
                  set(gca, 'YTick', [0.2:0.2:1]); %grid on;
    ax(2) = subplot(222), plot(snrs, 1-score_snr, 'Linewidth', 1.5); 
                  xlim([-20, 26]);  ylim([1/16, 1.02]); title('MN16R','FontSize',12);
                  set(gca, 'XTick', [-12:6:24], 'XTickLabel', {'-12';'-6';'0';'6';'12';'18';'Q'}, 'FontSize', 9); 
                  set(gca, 'YTick', [0.2:0.2:1], 'YTickLabel', '' ); %grid on;                  
    ax(4) = subplot(224), plot(1-pe_lp, AI_BandNo, '-r', 1-pe_hp, AI_BandNo, '-b', 'Linewidth', 1.5); 
                  set(gca, 'XTick', [0.2:0.2:1]); 
                  set(gca, 'YAxisLocation','l','YTick',linspace(1,how_many_filters,11), 'YTickLabel', '', 'FontSize', 8);
                  %set(gca,'YTickLabel',0.1*round(10*frequencies(round(linspace(1,how_many_filters,11)))/1000))
                  ylim([1, 200]);  xlim([1/16, 1.02]); %grid on;
                  xlabel('HL07','FontSize', 12);
                  %ylabel('Frequency [kHz]','FontSize',12)
                  %title('Hi-Lo Data','FontSize',12);
                  %legend('L','H');
    ax(3) = subplot(223), plot_AI_gram(AIgram, utterance, snr); hold on; %grid on;
    
        con_start_time = min(conds_time)/100; 
        con_end_time = max(conds_time)/100;
        vow_start_time = vow_start_time/100 + delay/Fs;

        con_start_col = round((con_start_time-start_time)/(end_time-start_time)*nframe);
        con_end_col = round((con_end_time-start_time)/(end_time-start_time)*nframe);
        vow_start_col = round((vow_start_time-start_time)/(end_time-start_time)*nframe);

        line([con_start_col con_start_col], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' ); hold on;
        line([con_end_col con_end_col], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' );
        line([vow_start_col vow_start_col], [1 200], 'Color', 'g' ,'Linewidth', 1, 'Linestyle', '-' );    
        
    % cut the blank part of aigram
    ax(3) = subplot(223), xlim([con_start_col-5, vow_start_col+60]);
                          text(vow_start_col+55, 180, fix_TSDZ([fix_underscore(utter) '@' num2str(snr) 'dB'], 1), 'FontSize',14,...
                              'HorizontalAlignment','right','EdgeColor','black','BackgroundColor',[1 1 1]);
    ax(1) = subplot(221), xlim([(con_start_col-5)*0.25, (vow_start_col+60)*0.25]);
        
    set(ax(1), 'Position', [0.12, 0.68, 0.5, 0.22]);
    set(ax(2), 'Position', [0.64, 0.68, 0.3, 0.22]);    
    set(ax(3), 'Position', [0.12, 0.1, 0.5, 0.53]);
    set(ax(4), 'Position', [0.64, 0.1, 0.3, 0.53]);

    %saveas(gcf, ['./results/' utterance '_feature.eps'], 'eps');  

