%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program calculate the feature mask for each utterance based on 
% the filtering and truncation data.
% Input:    utter - the utterance to be analyzed    
% Output:   feature - feature mask of the given utterance
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-28
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function feature = calc_feature_mask(SPDB, utter)

% first calculate the Pe as a function of time
data = query(SPDB, 'exp', 'trunc07', 'utter', utter);
pe_tem = calc_Pe(data, utter, 'time');
score_time = pe_tem.score;
conds_time = pe_tem.conds;
    
% then calculate the Pe as a function of high-pass and low-pass cutoff frequencies
data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'HP');
pe_tem = calc_Pe(data, utter, 'hpf');
score_hpf = pe_tem.score;
freqs_hpf = pe_tem.conds;

data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'LP');
pe_tem = calc_Pe(data, utter, 'lpf');
score_lpf = pe_tem.score;
freqs_lpf = pe_tem.conds;

% use the same frequencies for the HP and LP score
freqs_all = unique(sort([freqs_lpf; freqs_hpf]));

pe_lp = interp1(freqs_lpf, score_lpf, freqs_all, 'nearest', 'extrap');
pe_hp = interp1(freqs_hpf, score_hpf, freqs_all, 'nearest', 'extrap');

% weights for all frequency segments
weights = NaN(2, length(freqs_all)-1); 
weights(1,:) = -(pe_lp(2:end) - pe_lp(1:end-1));    % lpf weights
weights(2,:) = -(pe_hp(1:end-1) - pe_hp(2:end));    % hpf weights

% set those less than zero to be zero
p = weights < 0; weights(p) = 0;
weight_sum = weights(1,:) + weights(2,:);    % weight_freq = weight_lpf + weight_hpf

x_freqs_all = greenwood_inverse(freqs_all);
x_mid_freqs = (x_freqs_all(1:end-1) + x_freqs_all(2:end))/2;

% map the frequency to auditory frequency bands
x_apex = greenwood_inverse(179.46);
x_base = greenwood_inverse(7520);

x_AI = linspace( x_base, x_apex, 200);

% frequency importance function 
importance_freq.weight = interp1(x_mid_freqs, weight_sum, x_AI, 'nearest', 'extrap');
importance_freq.x = x_AI;

% then deal with the time importance function

    utterance = ['s_' utter];
    
    %%%%%%-------AI-gram v4.2--------%%%%%%
    snr = 12;
    [s, Fs, bits]= wavread(['./wavs/' utterance '.wav']);
    speech_data.fs = Fs;
    speech_data.type = 'speechonly';
    speech_data.snr = snr;
    speech_data.signal = s;
    
    cd('./aigram_v4.2');
        
    filterbank = 'erbaligned';
    %%1. Prepare the speech and generate the noise at 0 dB SNR like this:            
    [ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn16r( speech_data );  
    %%2. Compute the auditory representation of the speech-only signal like this:
    [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
    %%3. Compute the noise statistics:
    [ E_ni2, E_ni ] = compute_noise_stats('mn16r', filterbank, std(noise_only) );       

    %%4. Scale the noise statistics to the appropriate SNR:
    E_ni2_scaled = E_ni2 * 10^(-snr/10);
    E_ni_scaled = E_ni * 10^(-snr/20);            
    %%5. Make AI-gram out of existing information:
    [aigram, cc_d, time_axis, freq_axis ] = form_image_ai0607c( level3, 'mn16r', filterbank, E_ni2_scaled, E_ni_scaled);     
        
    cd('..');
    
    AIgram.ccgram = aigram;
    AIgram.time_axis = time_axis;
    AIgram.freq_axis = freq_axis;
    
    % align the time importance function with AIgram
    % This number only works when Fs = 16000, for other Fs re-calibration is
    % needed to get this number
    R = 4; delay = 0; %delay = 736/R/Fs; % delay of the auditory filter introduced by AIgram 4.0
    %x_time = conds_time+delay*100;
    x_time = conds_time/100+delay;
    x_mid_time = (x_time(1:end-1)+x_time(2:end))/2;
    
    weight = score_time(2:end)-score_time(1:end-1);
    p = find(weight<0); weight(p) = 0;
    importance_time.weight = interp1(x_mid_time, weight, time_axis, 'nearest', 'extrap');
    importance_time.x = time_axis*100;

    %feature = sqrt(importance_freq.weight'*importance_time.weight);
    feature.mask = importance_freq.weight'*importance_time.weight;
    feature.time_axis = importance_time.x;
    feature.freq_axis = importance_freq.x;
    
    figure;
    subplot(211), plot_AI_gram(AIgram, utterance, snr); hold on; 
    subplot(212), plot_feature(feature, utterance); %hold on; contour(flipud(feature));
    
    saveas(gcf, ['./results/' utterance '_feature.emf'], 'emf');  

