%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program calculate the feature mask for each utterance based on 
% the filtering and truncation data.
% Input:    utter - the utterance to be analyzed    
% Output:   feature - feature mask of the given utterance
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-28
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pe_lp, pe_hp, pe_lphp] = calc_pe_lphp_cutfreq(SPDB, utter)

% then calculate the Pe as a function of high-pass and low-pass cutoff frequencies
data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'HP');
pe_tem = calc_Pe(data, utter(6:end), utter, 'hpf');
pe_hp.score = pe_tem.score;
pe_hp.freqs = pe_tem.conds;

data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'LP');
pe_tem = calc_Pe(data, utter(6:end), utter, 'lpf');
pe_lp.score = pe_tem.score;
pe_lp.freqs = pe_tem.conds;

% use the same frequencies for the HP and LP score
freqs_all = unique(sort([pe_lp.freqs; pe_hp.freqs]));

pe_lp_intp = interp1(pe_lp.freqs, pe_lp.score, freqs_all, 'nearest', 'extrap');
pe_hp_intp = interp1(pe_hp.freqs, pe_hp.score, freqs_all, 'nearest', 'extrap');

pe_lphp.score = sqrt(pe_lp_intp.*pe_hp_intp);
pe_lphp.freqs = freqs_all;

semilogx(pe_lp.freqs, pe_lp.score, '-ro','LineWidth',1.5 ); hold on
semilogx(pe_hp.freqs, pe_hp.score, '-bs','LineWidth',1.5 ); hold on

pe_lphp_int = pe_lp_intp.*pe_hp_intp;
semilogx(pe_lphp.freqs, pe_lphp_int, '--k','LineWidth',1.5);

grid off; xlim([200, 10000]); ylim([-0.02, 1+0.02]); 
title(fix_underscore( utter ), 'FontSize', 12);
set(gca,'XTick',freqs_all([1,3,5,7,9,11,13]),'YTick',[0:0.2:1], 'FontSize', 10);
xlabel('Cutoff Frequency (Hz)', 'FontSize', 12); 
ylabel('Pe', 'FontSize', 12); 

%legend({'Pe_{LP}'; 'Pe_{HP}'; '{(Pe_{LP}*Pe_{HP})}^{0.5}'}, 'Location', 'Best');

