% This program generates a cube of AIgram with the three dimensions 
% being time, frequency, and snr. 
% Input: wave_file -- name of the wave file
%        exp -- id of the experiment (64 for MN04, 16 for MN16R)
% Output: AI_cube -- the cube of aigram (v4)

function [AI_cube, x, speech_only, Fs] = creat_AI_cube(talker, sound, snrs, exp)

    wav_file = ['s_' talker '_' sound '.wav'];

    [x, Fs] = wavread(wav_file);

    S = load('standard.mat');
    
    if isempty(x)
        AI_cube = [];
        return;
    end

    %%%%%%-------AI-gram v4.2--------%%%%%%
    speech_data.fs = Fs;
    speech_data.type = 'speechonly';
    speech_data.snr = 0;
    speech_data.signal = x;
    
    filterbank = 'erbaligned';
    if (strcmp(exp,'64'))
            
        %%1. Prepare the speech and generate the noise at 0 dB SNR like this:
        [ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn64( speech_data );
        %%2. Compute the auditory representation of the speech-only signal like this:
        [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
        %%3. Compute the noise statistics:
        [ E_ni2, E_ni ] = compute_noise_stats('mn64', filterbank, std(noise_only) );  
        
    elseif (strcmp(exp,'16'))
            
        %%1. Prepare the speech and generate the noise at 0 dB SNR like this:            
        [ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn16r( speech_data );  
        %%2. Compute the auditory representation of the speech-only signal like this:
        [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
        %%3. Compute the noise statistics:
        [ E_ni2, E_ni ] = compute_noise_stats('mn16r', filterbank, std(noise_only) );                
        
    end     
    
    for i= 1:length(snrs)

        if (strcmp(exp,'64'))
            
            %%4. Scale the noise statistics to the appropriate SNR:
            E_ni2_scaled = E_ni2 * 10^(-snrs(i)/10);
            E_ni_scaled = E_ni * 10^(-snrs(i)/20);   
            %%5. Make AI-gram out of existing information:
            [aigram, cc_d, time_axis, freq_axis ] = form_image_ai0607c( level3, ...
            'mn64', filterbank, E_ni2_scaled, E_ni_scaled);   
        
        elseif (strcmp(exp,'16'))
            
            %%4. Scale the noise statistics to the appropriate SNR:
            E_ni2_scaled = E_ni2 * 10^(-snrs(i)/10);
            E_ni_scaled = E_ni * 10^(-snrs(i)/20);            
            %%5. Make AI-gram out of existing information:
            [aigram, cc_d, time_axis, freq_axis ] = form_image_ai0607c( level3, ...
            'mn16r', filterbank, E_ni2_scaled, E_ni_scaled); 
        
        end 

    % initialize the confusion patters
        
        AI(i).snr = snrs(i);
        AI(i).cc = cc_d;
        AI(i).ccgram = aigram;
        AI(i).time_axis = time_axis;
        AI(i).freq_axis = freq_axis;

    end
    
    utter = [ talker '_' sound ];

    AI_cube = AI;
