% This program generates multiple aigrams at different snrs and compare
% them in a single figure

function gen_aigrams(SPDB, utter, vow_start_time)

snrs = [-12, -6, 0, 6, 12, 18];
talker = utter(1:4); cv = utter(6:end); utterance = ['s_' utter];

%% calculate AI-grams for the utterance
cd('./aigram_v4.2');
[AI_cube, s, speech_only, Fs] = creat_AI_cube(talker, cv, snrs, '16');
cd('..');

%% calculate the time importance function
% align the time importance function with AIgram
% This number only works when Fs = 16000, for other Fs re-calibration is
% needed to get this number

% calculate Pe as a function of truncation time
data = query(SPDB, 'exp', 'trunc07', 'utter', utter);
pe_tem = calc_Pe(data, cv, utter, 'time');
eT = pe_tem.score;  % probability error of truncated utterance
t = pe_tem.conds;   % truncation time

delay_aigram = 736; % delay of the auditory filter introduced by AIgram 4.2

% find the shift in samples between the original speech and speech_only 
[shift, error] = speech_match(s, speech_only, Fs); 

delay_total = shift + delay_aigram;  % total delay of speech_only
t = t/100 + delay_total/Fs;

%% make plots for aigrams
S = load('standard.mat');
frequencies = S.frequencies;

for i=1:length(snrs)

        AIgram.ccgram = AI_cube(i).ccgram;
        AIgram.time_axis = AI_cube(i).time_axis;
        AIgram.freq_axis = AI_cube(i).freq_axis;
    
        [nband, nframe] = size(AIgram.ccgram);


        stime = min(AIgram.time_axis);
        etime = max(AIgram.time_axis);
        
        con_stime = min(t); 
        con_etime = max(t);
        
        vow_stime = vow_start_time/100 + delay_total/Fs;

        con_scol = round((con_stime-stime)/(etime-stime)*nframe);
        con_ecol = round((con_etime-stime)/(etime-stime)*nframe);
        vow_scol = round((vow_stime-stime)/(etime-stime)*nframe);
        
        ax(i) = subplot(1, length(snrs), i), 
        
        imagesc(AIgram.ccgram); hold on;
        colormap(flipud(gray));
        
        name = fix_underscore(utter);

        if (i==1), text(con_scol+5, 184, fix_TSDZ(name, 1), 'FontSize',14); end
        
        %line([con_scol con_scol], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' ); hold on;
        %line([con_ecol con_ecol], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' ); hold on;
        line([vow_scol vow_scol], [1 200], 'Color', 'g' ,'Linewidth', 1, 'Linestyle', '--' );                       

        if (i==1), ylabel('Frequency [kHz]','FontSize',12); end

        xlabel('Time [cs]','FontSize',12);
        
        set(gca,'YDir','normal')
        set(gca,'FontSize',12)        
        set(gca,'YAxisLocation','l','YTick',linspace(1,nband,11),'YTickLabel', '', 'FontSize', 8);
        if (i==1), set(gca,'YTickLabel',0.1*round(10*frequencies(round(linspace(1,nband,11)))/1000)); end
        
        set(gca,'XTick',40*[1:0.5:floor(nframe/40)],'FontSize', 8);
        set(gca,'XTickLabel',10*[1:0.5:floor(nframe/40)]);
        
        xlim([con_scol, con_ecol]);
        title([num2str(snrs(i)), ' dB'], 'FontSize', 12);
   
end
sameaxis(ax, length(snrs), 1);
