%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is used to plot the audiogram of hearing impaired
% listeners, Hearing Impaired.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2006-11-07
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_Audiogram(Audiogram, ear)

p = find(~isnan(Audiogram.audiogram.HL));
Freq = Audiogram.audiogram.Freq(p);
HL = Audiogram.audiogram.HL(p);
PTA = Audiogram.PTA;
SNRLoss = Audiogram.SNRLoss;
Listener = Audiogram.ID;

figure('Position', [100, 100, 450, 450]);

if strcmp(lower(ear), 'left')
    semilogx(Freq, -HL, '-xb', 'LineWidth', 2, 'Markersize', 8);
elseif strcmp(lower(ear), 'right')
    semilogx(Freq, -HL, '-ob', 'LineWidth', 2, 'Markersize', 8);
end
xlim([100,10000]); ylim([-120,10]);
xlabel('Frequency (Hz)', 'Fontsize', 12); 
ylabel('Hearing Level(dB HL)', 'Fontsize', 12);
set(gca,'XTick',[125,250,500,1000,2000,4000,8000],'XAxisLocation', 'top', 'Fontsize', 12);
set(gca,'YTick',[-120:10:10], 'YTickLabel',abs([-120:10:10]),'Fontsize', 12);
grid on; %title(['Audiogram of ' Listener], 'Fontsize', 12);
    
if ~isnan(PTA)
    text(130, 72, ['PTA: ' num2str(PTA) ' (dB)'], 'VerticalAlignment','baseline', 'Fontsize', 12);
end
if ~isnan(SNRLoss)
    text(130, 82, ['SNR Loss: ' num2str(SNRLoss) ' (dB)'], 'VerticalAlignment','baseline', 'Fontsize', 12); 
end
    
saveas(gcf,['Audiogram-' Listener '.eps'], 'psc2'); 
%saveas(gcf,['Audiogram-' Listener '.emf'], 'emf'); 
