%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is used to plot the confusion pattern of normal hearing
% Hearing Impaired 06.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2006-10-10
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_CP(CP)

caption = CP.caption;
resps = CP.resps;
xlab = CP.xlabel;
ylab = CP.ylabel;
conds = CP.conds;
table = CP.table;

for i=1:length(resps)
    if ~(strcmp(resps{i},'?')||strcmp(resps{i},'a')||strcmp(resps{i},'.'))
        %resps{i} = resps{i}(1:end-1);
        if (strcmp(resps{i}(1),'x'))
           resps{i} = upper(resps{i}(2:end));
        end
    end
end

markers_code = create_markers_code();
             
nCol = length(resps); nRow = length(conds);

%scrsz = get(0,'ScreenSize');
%figure('Position',[1 1 scrsz(3)/1.2 scrsz(4)/1.2]);
%figure
    
% convert the cut-off frequecies into auditory filter band number
if (strcmp(lower(CP.type), 'hpf')||strcmp(lower(CP.type), 'lpf')...
    ||strcmp(lower(CP.type), 'lpf08')||strcmp(lower(CP.type), 'hpf08'))
    S = load('standard.mat');
    frequencies = S.frequencies;
    how_many_filters = length(frequencies);

    for i=1:length(conds)
        temp = abs(frequencies-conds(i));
        p = find(temp==min(temp));
        AI_BandNo(i) = p; % the AI band no corresponds to the cut-off frequency
    end
end

p_chance = 1/16;
S = sum(table,2)+ eps; % total num of presentation 
pos = find(strcmp(resps, '?')); 
for j=1:nCol
    if (max(table(:,j))>1)&&(j~=pos) % skip the col of unknow
        pi_col = table(:,j)./S;
        idx = find(pi_col<p_chance);
        pi_col(idx) = p_chance;
        
        % remove uninteresting sounds (pi<1/8) from the Confusion Patterns plot
        if max(pi_col)>1/8
        
        if strcmp(lower(CP.type), 'snr')&&(conds(end)>32)
            conds(end) = 24;
        end
        %semilogy(conds, pi_row, markers_code{j}.shape, 'Color', markers_code{j}.color,...
        %    'MarkerFacecolor', markers_code{j}.facecolor, 'MarkerSize', 4 ,'LineWidth',1.5 ); hold on
        
        if strcmp(lower(CP.type), 'hpf')||strcmp(lower(CP.type), 'hpf08')
            plot(pi_col, AI_BandNo, 'LineStyle', '-', 'Color', markers_code{j}.color, 'LineWidth', 1.5 ); hold on
        elseif strcmp(lower(CP.type), 'lpf')||strcmp(lower(CP.type), 'lpf08')
            plot(pi_col, AI_BandNo, 'LineStyle', '-', 'Color', markers_code{j}.color, 'LineWidth', 1.5 ); hold on            
        else
            plot(conds, pi_col, 'Color', markers_code{j}.color, 'LineWidth', 1.5 ); hold on
        end
        
        p = max(find(table(:,j)==max(table(:,j))));  y = table(p,j)./S(p);
        if (y>p_chance)&&(y<1.1)
            if (strcmp(lower(CP.type), 'hpf')||strcmp(lower(CP.type), 'lpf')||...
                strcmp(lower(CP.type), 'hpf08')||strcmp(lower(CP.type), 'lpf08'))
                x = AI_BandNo(p); 
                text(y, double(x), resps(j), 'VerticalAlignment','baseline', 'Color', 'k', 'FontSize', 11, 'FontWeight','bold');
            else
                x = conds(p);
                text(double(x), y, resps(j), 'VerticalAlignment','baseline', 'Color', 'k', 'FontSize', 11, 'FontWeight','bold');
            end
        end
        
        end
    else
       %plot(conds, cm1(j,:)./S, '--o', 'Color', color_codes(j-1,:)); hold on
    end
end

% utterance = fix_underscore(title);
% if (utterance(5)=='x'), 
%     utterance(6)=upper(utterance(6));
%     utterance = [utterance(1:4) utterance(6:end)];
% end
% title(utterance, 'FontSize', 9);
%title(fix_underscore(caption), 'FontSize', 12);
grid on; 
%legend(resps,'Location','EastOutside');
% set the maximum snr to be 32 so that the plot looks better
if strcmp(lower(CP.type), 'snr')
    set(gca,'XTick',conds, 'XTickLabel', {conds(1:end-1), 'Q'});
    xlim([-18, 24]); ylim([1/16, 1.02]); 
    %ylabel('nResp/nPresents', 'FontSize', 12);   
    xlabel('SNR (dB)', 'FontSize', 12);   
elseif strcmp(lower(CP.type), 'time')||strcmp(lower(CP.type), 'tr08')
    xlim([conds(1),conds(end)]); ylim([1/16, 1.02]); 
    set(gca,'XTick',unique(floor(conds)),'YTick',[0:0.2:1], 'FontSize', 9);
    %xlabel('Truncation time [cs]', 'FontSize', 12); 
    ylabel('nResp/nPresents', 'FontSize', 12); 
elseif strcmp(lower(CP.type), 'lpf')||strcmp(lower(CP.type), 'lpf08')
    %set(gca,'XTick',conds(2:2:end), 'XTickLabel', conds(2:2:end));
    %set(gca,'YTick',[200,300,500,700,1000,1400,2000,2800,3900,5400,7400],'YTickLabel',{'0.2';'0.3';'0.5';'0.7';'1';'1.4';'2';'2.8';'3.9';'5.4';'7.4'}, 'FontSize', 9);   
    ylim([1,200]);  
    %ylabel('Cutoff Frequency [kHz]', 'FontSize', 12); 
    xlabel('nResp/nPresents', 'FontSize', 12); 
    xlim([1/16, 1.02]); 
    set(gca, 'XTick', [0.2:0.2:1]); 
    set(gca,'YAxisLocation','l','YTick',linspace(1,how_many_filters,11), 'YTickLabel', '', 'FontSize', 8);
    set(gca,'YTickLabel',0.1*round(10*frequencies(round(linspace(1,how_many_filters,11)))/1000))    
elseif strcmp(lower(CP.type), 'hpf')||strcmp(lower(CP.type), 'hpf08')
    %set(gca,'XTick',conds(1:2:end), 'XTickLabel', conds(1:2:end));
    %set(gca,'YTick',[200,300,500,700,1000,1400,2000,2800,3900,5400,7400],'YTickLabel',{'0.2';'0.3';'0.5';'0.7';'1';'1.4';'2';'2.8';'3.9';'5.4';'7.4'}, 'FontSize', 9);   
    ylim([1,200]);  
    %ylabel('Cutoff Frequency [kHz]', 'FontSize', 12); 
    xlabel('nResp/nPresents', 'FontSize', 12); 
    xlim([1/16, 1.02]);  
    set(gca, 'XTick', [0.2:0.2:1]); 
    set(gca,'YAxisLocation','l','YTick',linspace(1,how_many_filters,11), 'YTickLabel', '', 'FontSize', 8);
    %set(gca,'YTickLabel',0.1*round(10*frequencies(round(linspace(1,how_many_filters,11)))/1000))    
end
