% This script plot the AIgrams and CPs(Confusion Pattern) of an utterance as 
% a function of the truncation time, high-pass and low-pass cutoff
% frequency and signal-to-noise ratios.

function plot_CP_HL07_TR07_MN16_AIgram(SPDB, utter, snr, vow_stime)

%% ---- prepare data for the plots ----
% calculate the CP as a function of truncation time
data = query(SPDB, 'exp', 'trunc07', 'utter', utter);
cp_time = calc_CP(data, utter, 'time');
    
% calculate the CP as a function of high-pass and low-pass cutoff frequencies
data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'HP');
cp_hpf = calc_CP(data, utter, 'hpf');

data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'LP');
cp_lpf = calc_CP(data, utter, 'lpf');

% calculate the CP as a function of snr
data = query(SPDB, 'exp', 'MN16R', 'utter', utter);
cp_snr = calc_CP(data, utter, 'snr');

% calculate AI-gram for the utterance
% ---------------------------
utterance = ['s_' utter];
    
[s, Fs, bits]= wavread(['./wavs/' utterance '.wav']);
speech_data.fs = Fs;
speech_data.type = 'speechonly';
speech_data.snr = 0;
speech_data.signal = s;
    
cd('./aigram_v4.2');
        
filterbank = 'erbaligned';
%%1. Prepare the speech and generate the noise at 0 dB SNR like this:            
[ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn16r( speech_data );  
%%2. Compute the auditory representation of the speech-only signal like this:
[ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
%%3. Compute the noise statistics:
[ E_ni2, E_ni ] = compute_noise_stats('mn16r', filterbank, std(noise_only) );       
%%4. Scale the noise statistics to the appropriate SNR:
E_ni2_scaled = E_ni2 * 10^(-snr/10);
E_ni_scaled = E_ni * 10^(-snr/20);            
%%5. Make AI-gram out of existing information:
[aigram, cc_d, time_axis, freq_axis ] = form_image_ai0607c( level3, 'mn16r', filterbank, E_ni2_scaled, E_ni_scaled);     
        
cd('..');
    
AIgram.cc = cc_d;
AIgram.ccgram = aigram;
AIgram.time_axis = time_axis;
AIgram.freq_axis = freq_axis;
%---------------------------

%% align the CP of truncation data to the AIgram
delay_aigram = 736; % delay of the auditory filter introduced by AIgram 4.2

% find the shift in samples between the original speech and speech_only 
[shift, error] = speech_match(s, speech_only, Fs); 

delay_total = shift + delay_aigram;  % total delay of speech_only
cp_time.conds = cp_time.conds + delay_total/Fs*100;


%% make plots for the CPs and AIgram
[nband, nframe] = size(AIgram.ccgram);

stime = min(AIgram.time_axis);   % start time of the AI-gram
etime = max(AIgram.time_axis);   % end time of the AI-gram

con_stime = min(cp_time.conds)/100;         % start time of consonant
con_etime = max(cp_time.conds)/100;         % end time of consonant
vow_stime = (vow_stime + delay_total/Fs*100)/100; 	% start time of vowel

con_start_col = round((con_stime-stime)/(etime-stime)*nframe);
con_end_col = round((con_etime-stime)/(etime-stime)*nframe);
vow_start_col = round((vow_stime-stime)/(etime-stime)*nframe);

ax1 = subplot(231); plot_CP(cp_time); title('TR07', 'FontSize', 12); grid off
                    xlim([0, nframe*0.25]); ylim([1/16, 1.02]); hold on;
                    set(gca,'XTick',[0:10:round(nframe*0.25)], 'FontSize', 12);
ax2 = subplot(232); plot_CP(cp_snr); title('MN16R', 'FontSize', 12); grid off
ax5 = subplot(235); plot_CP(cp_lpf); title('HL07-LP', 'FontSize', 12); grid off
ax6 = subplot(236); plot_CP(cp_hpf); title('HL07-HP', 'FontSize', 12); grid off
ax3 = subplot(233); plot(AIgram.cc, 'LineWidth', 1.5 ); xlim([1,nframe]);
                    ylabel('[bits/cs]','FontSize',12);
                    set(gca,'XTick',40*[1:floor(nframe/40)], 'XTickLabel', '', 'FontSize', 12);
                    set(gca,'YTick', [0.2:0.2:1], 'FontSize', 12);
                    %set(gca,'XTickLabel',10*[1:floor(nframe/40)]);                    
ax4 = subplot(234); plot_AI_gram(AIgram, utterance, snr); hold on;

% plot lines on the AIgram indicating the start time and end time of
% consonant, and the start time of vowel

line([con_start_col con_start_col], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' ); hold on;
line([con_end_col con_end_col], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' );
line([vow_start_col vow_start_col], [1 200], 'Color', 'g' ,'Linewidth', 1, 'Linestyle', '-' );

% cut the blank part of aigram
ax4 = subplot(234), xlim([con_start_col-15, vow_start_col+60]); hold on;
    %text(vow_start_col-6, 180, fix_TSDZ([fix_underscore(utter) '@' num2str(snr) 'dB'], 1), 'FontSize',12);
    text(vow_start_col+57, 20, fix_TSDZ([fix_underscore(utter) '@' num2str(snr) 'dB'], 1), 'FontSize',12,...
         'HorizontalAlignment','right','EdgeColor','black','BackgroundColor',[1 1 1]);
    text(con_start_col-10, 180, 'c','FontSize',12, 'EdgeColor','black');
ax1 = subplot(231), xlim([(con_start_col-15)*0.25, (vow_start_col+60)*0.25]);
    text((con_start_col-10)*0.25, 0.85, 'a','FontSize',12, 'EdgeColor','black');
ax3 = subplot(233), xlim([(con_start_col-15), (vow_start_col+60)]);
    text((con_start_col-10), 0.2, 'b','FontSize',12, 'EdgeColor','black');
ax2 = subplot(232), text(-16, 0.85, 'd','FontSize',12, 'EdgeColor','black');
%ax5 = subplot(235), text(-16, 0.85, 'd','FontSize',12, 'EdgeColor','black');
%ax6 = subplot(236), text(-16, 0.85, 'd','FontSize',12, 'EdgeColor','black');
    
% set the position and size of the axes
set(ax1, 'Position', [0.1, 0.65, 0.4, 0.27]);
set(ax3, 'Position', [0.1, 0.50, 0.4, 0.10]);
set(ax2, 'Position', [0.55, 0.65, 0.4, 0.27]);
set(ax4, 'Position', [0.1, 0.1, 0.4, 0.38]);
set(ax5, 'Position', [0.55, 0.1, 0.19, 0.38]);
set(ax6, 'Position', [0.76, 0.1, 0.19, 0.38]);


