% this script make plot for the comparison of Normal Hearing and Hearing
% Impaired perceptual data.

function hfig = plot_Pc_HI_vs_NH(Pc_NH, Pc_HI, style)

switch lower(style)
    
    case {'32'} % 3*2 panel for stops and fricatives (Ta and Da are not included)
        
        hfig = figure('Position', [1, 1, 600, 800]);
        set(hfig,'PaperPositionMode','auto');
        for i=1:length(Pc_HI)
            ax(i) = subplot(3,2,i);
            plot_Pc_CV(Pc_NH{i}.snr, Pc_NH{i}.pc, Pc_HI{i}.snr, Pc_HI{i}.pc, Pc_HI{i}.snrNAL, Pc_HI{i}.pcNAL, Pc_NH{i}.stim); hold on;
            
            if (i>4) 
                xlabel('SNR (dB)'); 
                set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
            end
            if (mod(i,2)==1)
                ylabel('Pc'); 
                set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'});            
            end            
        end
       
        if ~isnan(Pc_HI{length(Pc_HI)}.snrNAL)
            legend({'NH'; 'HI'; 'HI(NAL-R)' }, 'Location', 'Best','Orientation', 'vertical', 'Fontsize', 10);
        else
            legend({'NH'; 'HI'}, 'Location', 'Best','Orientation', 'vertical', 'Fontsize', 10);
        end        
        
        set(ax(1), 'Position', [0.1, 0.66, 0.4, 0.28]);
        set(ax(2), 'Position', [0.5, 0.66, 0.4, 0.28]);    
        set(ax(3), 'Position', [0.1, 0.38, 0.4, 0.28]);
        set(ax(4), 'Position', [0.5, 0.38, 0.4, 0.28]);
        set(ax(5), 'Position', [0.1, 0.10, 0.4, 0.28]);
        set(ax(6), 'Position', [0.5, 0.10, 0.4, 0.28]);

    case {'12'} % 1*2 panel for nasals
        
        hfig = figure('Position', [1, 1, 600, 300]);
        set(hfig,'PaperPositionMode','auto');
        for i=1:length(Pc_HI)
            ax(i) = subplot(1,2,i);
            plot_Pc_CV(Pc_NH{i}.snr, Pc_NH{i}.pc, Pc_HI{i}.snr, Pc_HI{i}.pc, Pc_HI{i}.snrNAL, Pc_HI{i}.pcNAL, Pc_NH{i}.stim); hold on;
            xlabel('SNR (dB)');  ylabel('Pc'); 
            set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
            if (mod(i,2)==1) 
                set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'}); 
            end           
        end
       
        if ~isnan(Pc_HI{length(Pc_HI)}.snrNAL)
            legend({'NH'; 'HI'; 'HI(NAL-R)' }, 'Location', 'Best','Orientation', 'vertical', 'Fontsize', 10);
        else
            legend({'NH'; 'HI' }, 'Location', 'Best','Orientation', 'vertical', 'Fontsize', 10);
        end        
        
        set(ax(1), 'Position', [0.1, 0.15, 0.4, 0.72]);
        set(ax(2), 'Position', [0.5, 0.15, 0.4, 0.72]);    

    otherwise % 4*4 panel for all consonants
        
        hfig = figure('Position', [1, 1, 1000, 800]);
        set(hfig,'PaperPositionMode','auto');
        for i=1:length(Pc_HI)
            ax(i) = subplot(4,4,i);
            plot_Pc_CV(Pc_NH{i}.snr, Pc_NH{i}.pc, Pc_HI{i}.snr, Pc_HI{i}.pc, Pc_HI{i}.snrNAL, Pc_HI{i}.pcNAL, Pc_NH{i}.stim); hold on;
            
            if (i>12) 
                xlabel('SNR (dB)'); 
                set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'});        
            end
            if (mod(i,4)==1)
                ylabel('Pc'); 
                set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', {'0';'0.2';'0.4';'0.6';'0.8';'1.0'});            
            end            
        end
        sameaxis(ax, 4, 4);
        if ~isnan(Pc_HI{length(Pc_HI)}.snrNAL)
            legend({'NH'; 'HI'; 'HI(NAL-R)' }, 'Location', 'SouthOutside','Orientation', 'horizontal', 'Fontsize', 11);
        else
            legend({'NH'; 'HI' }, 'Location', 'SouthOutside','Orientation', 'horizontal', 'Fontsize', 11);
        end
end

function plot_Pc_CV(snr_NH, pi_NH, snr_HI, pi_HI, snr_HI_NAL, pi_HI_NAL, CV)

plot(snr_NH, pi_NH, '--k', 'LineWidth', 1.5); hold on;
plot(snr_HI, pi_HI, '-bo', 'LineWidth', 1.5); hold on;
plot(snr_HI_NAL, pi_HI_NAL, '-rs', 'LineWidth', 1.5); hold on;

xlim([-24 26]); ylim([-0.05 1.05]);
text(-21, 0.92, CV, 'Fontsize', 12); 
    
set(gca, 'XTick', [-18:6:24], 'XTickLabel', '');
set(gca, 'YTick', [-0:0.2:1], 'YTickLabel', '');    
