% this script plots the SNR Loss (consonant profile) of the 16 consonants
% for the hearing impaired listeners
% the idea of finding the SNRLoss of a given consonant is: 
% for a certain Pc_HI(snr_HI), find the snr_NH that has the same amount of
% Pc_HI. SNRLoss(i) = snr_HI - snr_NH.
% Input: listener - the name of the listener to be analyzed
%        compensate - with NAL-R or not
% Example: plot_SNRLoss(SPDB, 'SNRLoss07', 'schoenbe', 'NaN')
%          plot_SNRLoss(SPDB, 'SNRLoss07', 'schoenbe', 'NAL-R')
%   Author:      Feipeng Li revised from Marion's code
%   E-mail:      fli2@uiuc.edu
%   Time-stamp:  2007-12-02
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_SNRLoss(SPDB, exp, listener, compensate)

stims = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na'}; 

data_NH = query(SPDB, 'exp', 'MN16R');

snr_NH = unique(sort(data_NH{15})); snr_NH = snr_NH(2:end); % get rid of snr = -21dB
Pc_NH = zeros(16, length(snr_NH));

for i=1:length(snr_NH)
    data_NH_snr = query(data_NH, 'snr', snr_NH(i));
    CM_NH = calc_CM(data_NH_snr, num2str(snr_NH(i)));
    Pc_NH(:,i) = diag(CM_NH.table)./(sum(CM_NH.table,2)+eps);
end

%load SPDB_SL07.mat %%%%%%%%%!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%data_HI = query(SPDB_SL07, 'listener', listener, 'compensate', compensate);
data_HI = query(SPDB, 'listener', listener, 'compensate', compensate, 'comment', '');

snr_HI = unique(sort(data_HI{15}));
Pc_HI = zeros(16, length(snr_HI));

for i=1:length(snr_HI)
    data_HI_snr = query(data_HI, 'snr', snr_HI(i));
    CM_HI = calc_CM(data_HI_snr, num2str(snr_HI(i)));
    Pc_HI(:,i) = diag(CM_HI.table)./(sum(CM_HI.table,2)+eps);
end

%---------------------------
marker = {'s','^','d','v','p','<','o','>',  's','^','d','v','p','<','o','>'};
facecolor = {'none', 'none', 'none', 'none', 'none', 'none', 'none',...
             'k', 'k', 'k', 'k', 'k', 'k', 'k', 'none', 'none'};

clr = [1 0 0; 0.6 0.6 1; 0.3 1 0.3];

snr_max = length(snr_HI);

legend_str{1} = 'NaN';

for i = 1:16

    Pc_HI_con = Pc_HI(i,:);
    snr_HI_con = snr_HI; snr_HI_con(end) = 32;
    
    % the snr of NH Pc that is equal to a certain Pc_HI(snr_HI(i))
    snr_NH_equalPc = interp1(Pc_NH(i,:), snr_NH, Pc_HI_con, 'linear');
    
    slope =	100;
    if (snr_HI_con(end)-snr_HI_con(1))>0
    	slope =	atan((snr_NH_equalPc(end)-snr_NH_equalPc(1))/(snr_HI_con(end)-snr_HI_con(1)));
    end
    
	if slope > pi/3
		slope = 3;
	elseif slope < pi/6
		slope = 1;
	else
		slope = 2;
	end
	
    if ismember(i,9:16)
        plot(snr_HI_con,snr_NH_equalPc,[marker{i} '--'], 'Color', clr(slope,:),...
        'MarkerFacecolor', clr(slope,:), 'MarkerSize', 6);
    else
        plot(snr_HI_con,snr_NH_equalPc,[marker{i} '--'], 'Color', clr(slope,:),...
        'MarkerFacecolor', 'none', 'MarkerSize', 6);
    end
    
    if i==1
        legend_str{i} = stims{i};
    else
        legend_str{end+1} = stims{i};
    end
    
    xlim([-20 35]); ylim([-20 35]); hold on,

end

h = legend(legend_str,'Location','EastOutside');    
set(h,'Interpreter','none');
plot([-20 35], [-20 35], 'k--')
set(gca,'XTick',[-20:10:20, 32], 'XTickLabel', {-20:10:20, 'Q'})
set(gca,'YTick',[-20:10:20, 32], 'YTickLabel', {-20:10:20, 'Q'})
xlabel('SNR_{HI} [dB]','FontSize',18)
ylabel('SNR_{ANH} [dB]','FontSize',18)


%saveas(gca,['SNRloss_' listeners{l} '.fig'],'fig')
if strcmp(compensate, 'NAL-R')
    t = text(-17,16,'NAL-R'); set(t,'FontSize',18)
    print('-depsc2',['SNRloss07_NAL-R_' listener]);
    %saveas(gca, ['SNRloss07_NAL-R_' listener '.emf'], 'emf');
else
    print('-depsc2',['SNRloss07_' listener]);
    %saveas(gca, ['SNRloss07_' listener '.emf'], 'emf');    
end


