%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is used to retrieve data from the database given a
% query condition, SPDB.
%
% Usage: data = query(DB, 'Utter', 'm115_xta', 'Talker', 'f103' ...
%                     'Stim', 'xza', 'Resp', 'ba', 'SNR', 12 ...)
%
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2006-11-09, Modified 2007-09-16
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Data = query(DB, varargin)

Exp = '';        Utter = '';      Talker = '';     Listener = ''; 
Hearing = '';    Audiogram = '';  L1 = '';         Accent = ''; 
Gender = '';     Age = NaN;       Stim = '';       Compensate = ''; 
Trunctime = NaN; Frequency = NaN;  SNR = NaN;       Noise = ''; 
S_Level = NaN;   N_Level = NaN;   R_Level = NaN;   Resp = ''; 
Resptime = NaN;  Repeat = NaN;    Hit = NaN;       Comment = '';

for i=1:length(varargin)
    if strcmp(lower(varargin{i}),'exp'),        Exp = varargin{i+1};        end
    if strcmp(lower(varargin{i}),'utter'),      Utter = varargin{i+1};      end
    if strcmp(lower(varargin{i}),'talker'),     Talker = varargin{i+1};     end
    if strcmp(lower(varargin{i}),'listener'),   Listener = varargin{i+1};   end
    if strcmp(lower(varargin{i}),'hearing'),    Hearing = varargin{i+1};    end
    if strcmp(lower(varargin{i}),'audiogram'),  Audiogram = varargin{i+1};  end
    if strcmp(lower(varargin{i}),'l1'),         L1 = varargin{i+1};         end
    if strcmp(lower(varargin{i}),'accent'),     Accent = varargin{i+1};     end
    if strcmp(lower(varargin{i}),'gender'),     Gender = varargin{i+1};     end
    if strcmp(lower(varargin{i}),'age'),        Age = varargin{i+1};        end
    if strcmp(lower(varargin{i}),'stim'),       Stim = varargin{i+1};       end
    if strcmp(lower(varargin{i}),'compensate'), Compensate = varargin{i+1}; end
    if strcmp(lower(varargin{i}),'trunctime'),  Trunctime = varargin{i+1};  end
    if strcmp(lower(varargin{i}),'frequency'),  Frequency = varargin{i+1};  end
    if strcmp(lower(varargin{i}),'snr'),        SNR = varargin{i+1};        end
    if strcmp(lower(varargin{i}),'noise'),      Noise = varargin{i+1};      end
    if strcmp(lower(varargin{i}),'s_level'),    S_Level = varargin{i+1};    end
    if strcmp(lower(varargin{i}),'n_level'),    N_Level = varargin{i+1};    end
    if strcmp(lower(varargin{i}),'r_level'),    R_Level = varargin{i+1};    end
    if strcmp(lower(varargin{i}),'resp'),       Resp = varargin{i+1};       end
    if strcmp(lower(varargin{i}),'resptime'),   Resptime = varargin{i+1};   end
    if strcmp(lower(varargin{i}),'repeat'),     Repeat = varargin{i+1};     end
    if strcmp(lower(varargin{i}),'hit'),        Hit = varargin{i+1};        end
    if strcmp(lower(varargin{i}),'comment'),    Comment = varargin{i+1};    end
end

[nRow, nCol] = size(DB{1});
idx = zeros(nRow, 24);

if isempty(Exp)
    idx(:,1) = 1;
else
    p = find(strcmp(lower(DB{1}), lower(Exp)));
    idx(p,1) = 1;
end

if isempty(Utter)
    idx(:,2) = 1;
else
    p = find(strcmp(lower(DB{2}), lower(Utter)));
    idx(p,2) = 1;
end

if isempty(Talker)
    idx(:,3) = 1;
else
    p = find(strcmp(lower(DB{3}), lower(Talker)));
    idx(p,3) = 1;
end

if isempty(Listener)
    idx(:,4) = 1;
else
    p = find(strcmp(lower(DB{4}), lower(Listener)));
    idx(p,4) = 1;
end

if isempty(Hearing)
    idx(:,5) = 1;
else
    p = find(strcmp(lower(DB{5}), lower(Hearing)));
    idx(p,5) = 1;
end

if isempty(Audiogram)
    idx(:,6) = 1;
else
    p = find(strcmp(lower(DB{6}), lower(Audiogram)));
    idx(p,6) = 1;
end

if isempty(L1)
    idx(:,7) = 1;
else
    p =  find(strcmp(lower(DB{7}), lower(L1)));
    idx(p,7) = 1;
end

if isempty(Accent)
    idx(:,8) = 1;
else
    p =  find(strcmp(lower(DB{8}), lower(Accent)));
    idx(p,8) = 1;
end

if isempty(Gender)
    idx(:,9) = 1;
else
    p = find(strcmp(lower(DB{9}), lower(Gender)));
    idx(p,9) = 1;
end

if isnan(Age)
    idx(:,10) = 1;
else
    p = find(DB{10}==Age);
    idx(p,10) = 1;
end

if isempty(Stim)
    idx(:,11) = 1;
else
    p = find(strcmp(lower(DB{11}), lower(Stim)));
    idx(p,11) = 1;
end

if isempty(Compensate)
    idx(:,12) = 1;
else
    p = find(strcmp(lower(DB{12}), lower(Compensate)));
    idx(p,12) = 1;
end

if isnan(Trunctime)
    idx(:,13) = 1;
else
    p = find(DB{13}<=Trunctime);
    idx(p,13) = 1;
end

if isnan(Frequency)
    idx(:,14) = 1;
else
    p = find(DB{14}==Frequency);
    idx(p,14) = 1;
end

if isnan(SNR)
    idx(:,15) = 1;
else
    p = find(DB{15}==SNR);
    idx(p,15) = 1;
end

if isempty(Noise)
    idx(:,16) = 1;
else
    p = find(strcmp(lower(DB{16}), lower(Noise)));
    idx(p,16) = 1;
end

if isnan(S_Level)
    idx(:,17) = 1;
else
    p = find(DB{17}==S_Level);
    idx(p,17) = 1;
end

if isnan(N_Level)
    idx(:,18) = 1;
else
    p = find(DB{18}==N_Level);
    idx(p,18) = 1;
end

if isnan(R_Level)
    idx(:,19) = 1;
else
    p = find(DB{19}==R_Level);
    idx(p,19) = 1;
end

if isempty(Resp)
    idx(:,20) = 1;
else
    p = find(strcmp(lower(DB{20}), lower(Resp)));
    idx(p,20) = 1;
end

if isnan(Resptime)
    idx(:,21) = 1;
else
    p = find(DB{21}<=Resptime);
    idx(p,21) = 1;
end

if isnan(Repeat)
    idx(:,22) = 1;
else
    p = find(DB{22}==Repeat);
    idx(p,22) = 1;
end

if isnan(Hit)
    idx(:,23) = 1;
else
    p = find(DB{23}==Hit);
    idx(p,23) = 1;
end

if isempty(Comment)
    idx(:,24) = 1;
else
    p = find(strcmp(lower(DB{24}), lower(Comment)));
    idx(p,24) = 1;
end

idxc = idx(:,1).*idx(:,2).*idx(:,3).*idx(:,4).*idx(:,5).*idx(:,6).*...
       idx(:,7).*idx(:,8).*idx(:,9).*idx(:,10).*idx(:,11).*idx(:,12).*...
       idx(:,13).*idx(:,14).*idx(:,15).*idx(:,16).*idx(:,17).*idx(:,18).*...
       idx(:,19).*idx(:,20).*idx(:,21).*idx(:,22).*idx(:,23).*idx(:,24);
   
p = find(idxc > 0);

[nRow, nCol] = size(DB);
for j=1:nCol
    Data{j} = DB{j}(p);
end
