% This program is used to calculate the SNR loss per consonant for
% the Hearing Impaired listeners. 
% The idea: for any point on the HI PI curve, find out the snr of the point
% on the NH PI curve that has the same Pc value. 

function calc_SNRloss(HIPI, snrHI, NHPI, snrNH, marker, markercolor, facecolor)

% first pre-process the Normal Hearing data. 
% If max(HIPI) > max(NHPI), set max(HIPI) = max(NHPI)
% interpolate NHPI and make sure that min(NHPI) = 0 using linear prediction
% so that NHPI covers the range of HIPI.

%plot([-20 32], [-20 32], 'k--'); hold on;

if snrHI(end)>32, snrHI(end) = 32; end; % quiet condition
if snrNH(end)>32, snrNH(end) = 32; end;

if min(NHPI)>0
    snr0 = interp1(NHPI, snrNH, 0, 'linear', 'extrap');
    NHPI = [0, NHPI];
    snrNH = [snr0; snrNH];
end

p = find(HIPI>max(NHPI)); HIPI(p) = max(NHPI);

for i=1:length(HIPI)
    snr_NH4HI(i) = interp1(NHPI, snrNH, HIPI(i), 'linear');
end

plot(snrHI, snr_NH4HI, ['--' marker], 'Color', markercolor, 'MarkerFacecolor', facecolor, 'MarkerSize', 6); hold on;

xlim([-24, 36]); ylim([-24, 36]); 

set(gca,'XTick',[-20:10:20, 32], 'XTickLabel', {-20:10:20, 'Q'})
set(gca,'YTick',[-20:10:20, 32], 'YTickLabel', {-20:10:20, 'Q'})
xlabel('SNR_{HI} [dB]','FontSize',16)
ylabel('SNR_{ANH} [dB]','FontSize',16)
%t = text(-17,16,Listener{j}); set(t,'FontSize',16)  
