% calculate CM, CP, Pe for SNRLoss07 data

clear all; close all; clc

load SPDB_SL07.mat; % the hearing impaired data

nSub = length(Index_SL07.listener);
nSNR = length(Index_SL07.snr);

%% calculate CM for each subject (EAR) at each SNR with and without NAL-R
for i=1:nSub
    for j=1:nSNR
        cur_sub = Index_SL07.listener{i};
        cur_snr = Index_SL07.snr(j);
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R');
        if length(data{1})
            cm = calc_CM(data, [cur_sub ' at ' num2str(cur_snr) 'dB with ' 'NAL-R']);
            cm.sub = cur_sub; cm.snr = cur_snr; cm.NAL = 1;
            CM_CV_NAL{i,j} = cm;
        end
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NaN');        
        if length(data{1})        
            cm = calc_CM(data, [cur_sub ' at ' num2str(cur_snr) 'dB without ' 'NAL-R']);
            cm.sub = cur_sub; cm.snr = cur_snr; cm.NAL = 0;        
            CM_CV_noNAL{i,j} = cm;
        end
    end
end


%% calculate CP for each subject*CV with and without NAL-R
stims = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na'}; 
for i=1:nSub
    for j=1:length(stims)
        cur_sub = Index_SL07.listener{i};
        cur_cv = stims{j};
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'stim', cur_cv, 'compensate', 'NAL-R');
        if length(data{1})
            cp = calc_CP(data, [cur_cv ' with NAL-R'], 'snr');
            cp.sub = cur_sub; cp.cv = cur_cv; cp.NAL = 1;
            CP_CV_NAL{i,j} = cp;
        end
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'stim', cur_cv, 'compensate', 'NaN');        
        if length(data{1})
            cp = calc_CP(data, [cur_cv ' without NAL-R'], 'snr');
            cp.sub = cur_sub; cp.cv = cur_cv; cp.NAL = 0;
            CP_CV_noNAL{i,j} = cp;
        end
    end
end

%% calculate CP for each subject*utterance with and without NAL-R
utter_list = get_utter_list_SNRLoss07();
for i=1:nSub
    for j=1:length(utter_list)
        cur_sub = Index_SL07.listener{i};
        cur_utter = utter_list{j};
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'utter', cur_utter, 'compensate', 'NAL-R');
        if length(data{1})
            cp = calc_CP(data, [cur_utter ' with NAL-R'], 'snr');
            cp.sub = cur_sub; cp.utter = cur_utter; cp.NAL = 1;
            CP_UTTER_NAL{i,j} = cp;
        end
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'utter', cur_utter, 'compensate', 'NaN');
        if length(data{1})
            cp = calc_CP(data, [cur_utter ' without NAL-R'], 'snr');
            cp.sub = cur_sub; cp.utter = cur_utter; cp.NAL = 0;
            CP_UTTER_noNAL{i,j} = cp;
        end
    end
end

%% calculate Pe for each subject*utterance with and without NAL-R
utter_list = get_utter_list_SNRLoss07();
for i=1:nSub
    for j=1:length(utter_list)
        cur_sub = Index_SL07.listener{i};
        cur_utter = utter_list{j}; cur_cv = cur_utter(6:end);
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'utter', cur_utter, 'compensate', 'NAL-R');
        if length(data{1})
            pe = calc_Pe(data, cur_cv, [cur_utter ' with NAL-R'], 'snr');
            pe.sub = cur_sub; pe.utter = cur_utter; pe.NAL = 1;
            PE_UTTER_NAL{i,j} = pe;
        end
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'utter', cur_utter, 'compensate', 'NaN');
        if length(data{1})
            pe = calc_Pe(data, cur_cv, [cur_utter ' without NAL-R'], 'snr');            
            pe.sub = cur_sub; pe.utter = cur_utter; pe.NAL = 0;
            PE_UTTER_noNAL{i,j} = pe;
        end
    end
end

%% calculate Pe for each subject*CV with and without NAL-R
for i=1:nSub
    for j=1:length(stims)
        cur_sub = Index_SL07.listener{i};
        cur_cv = stims{j};
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'stim', cur_cv, 'compensate', 'NAL-R');
        if length(data{1})
            pe = calc_Pe(data, cur_cv, [cur_cv ' with NAL-R'], 'snr');
            pe.sub = cur_sub; pe.cv = cur_cv; pe.NAL = 1;
            PE_CV_NAL{i,j} = pe;
        end
        
        data = query(SPDB_SL07, 'listener', cur_sub, 'stim', cur_cv, 'compensate', 'NaN');
        if length(data{1})
            pe = calc_Pe(data, cur_cv, [cur_cv ' without NAL-R'], 'snr');            
            pe.sub = cur_sub; pe.cv = cur_cv; pe.NAL = 0;
            PE_CV_noNAL{i,j} = pe;
        end
    end
end

% %% calculate grand Pe for each subject and without NAL-R
% % with NAL-R
% [nSub_NAL, nCV] = size(PE_CV_NAL);
% for i=1:nSub_NAL
%     cur_sub = PE_CV_NAL{i,1}.sub;
%     
%     nsnr = length(PE_CV_NAL{i,1}.score);
%     for j=1:length(stims)
%         if (~isempty(PE_CV_NAL{i,j}))&&(length(PE_CV_NAL{i,j}.score)>nsnr)
%             nsnr = length(PE_CV_NAL{i,j}.score);
%         end
%     end
%     
%     sum_score = zeros(nsnr, 1);
%     for j=1:length(stims)
%         disp(i*100+j);
%         if ~isempty(PE_CV_noNAL{i,j})
%             nskip = nsnr-length(PE_CV_NAL{i,j}.score);
%             sum_score(1:nskip) = sum_score(1:nskip) + 1; % assume that pe = 1 if the snr is skipped
%             sum_score(1+nskip:end) = sum_score(1+nskip:end) + PE_CV_NAL{i,j}.score;
%         end
%     end
%     ave_score = sum_score/length(stims);
%     
%     pe.caption = ['Grand Pe of ' cur_sub ' with NAL-R'];
%     pe.conds = PE_CV_NAL{i,1}.conds;
%     pe.xlabel = PE_CV_NAL{i,1}.xlabel;
%     pe.ylabel = PE_CV_NAL{i,1}.ylabel;    
%     pe.score = ave_score;    
%     pe.sub = cur_sub;
%     pe.NAL = 1;
%     
%     PE_NAL{i} = pe;
%     
% end
% 
% % without NAL-R
% [nSub_noNAL, nCV] = size(PE_CV_noNAL);
% for i=1:nSub_noNAL
%     cur_sub = PE_CV_noNAL{i,1}.sub;
% 
%     nsnr = length(PE_CV_noNAL{i,1}.score);
%     for j=1:length(stims)
%         if (~isempty(PE_CV_noNAL{i,j}))&&(length(PE_CV_noNAL{i,j}.score)>nsnr)    
%             nsnr = length(PE_CV_noNAL{i,j}.score);
%         end
%     end
%     
%     sum_score = zeros(nsnr, 1);
%     for j=1:length(stims)
%         disp(i*100+j);
%         if ~isempty(PE_CV_noNAL{i,j})
%             nskip = nsnr-length(PE_CV_noNAL{i,j}.score);
%             sum_score(1:nskip) = sum_score(1:nskip) + 1; % assume that pe = 1 if the snr is skipped
%             sum_score(1+nskip:end) = sum_score(1+nskip:end) + PE_CV_noNAL{i,j}.score;
%         end
%     end
%     ave_score = sum_score/length(stims);
%     
%     pe.caption = ['Grand Pe of ' cur_sub ' without NAL-R'];
%     pe.conds = PE_CV_noNAL{i,1}.conds;
%     pe.xlabel = PE_CV_noNAL{i,1}.xlabel;
%     pe.ylabel = PE_CV_noNAL{i,1}.ylabel;    
%     pe.score = ave_score;    
%     pe.sub = cur_sub;
%     pe.NAL = 0;
%     
%     PE_noNAL{i} = pe;
%     
% end

save('SL07.mat', 'CM_CV_NAL', 'CM_CV_noNAL', 'CP_CV_NAL', 'CP_CV_noNAL', 'CP_UTTER_NAL', 'CP_UTTER_noNAL',...
     'PE_CV_NAL', 'PE_CV_noNAL', 'PE_UTTER_NAL', 'PE_UTTER_noNAL');