%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is created to pull out the data of the HPLPF07 Normal 
% Hearing human speech perception experiment, HPLPF07.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-26
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [SPDB, Index] = pull_HPLPF07_data(zipfile)

datapath = './data/hplpf07data/';
mkdir(datapath); 
unzip(zipfile, datapath);

files = dir([datapath '*.txt']);

FLTDB_init = 0;

for i=1:length(files)
  filename = [datapath files(i).name];
    
  if isempty(findstr(filename, 'practice'))
  
    disp(filename);
    fid = fopen(filename);

    EOF = 0;
    while ~EOF
        str=[];
    	str=fgetl(fid);
    	
        if ~ischar(str), EOF=1; break, end; % Test for the end of file
    
        if strfind(str, 'Begin User Information')
            str=[]; str=fgetl(fid); Subject.Username = strtrim(str(13:end)); 
            str=[]; str=fgetl(fid); Subject.Firstname = strtrim(str(14:end));
            str=[]; str=fgetl(fid); Subject.Lastname = strtrim(str(13:end));
            str=[]; str=fgetl(fid); Subject.Gender = strtrim(upper(str(12:12)));
            str=[]; str=fgetl(fid); Subject.Birthday = strtrim(str(13:end)); 
            str=[]; str=fgetl(fid); Subject.FirstLanguage = lower(strtrim(str(17:end)));
            str=[]; str=fgetl(fid); Subject.Accent = lower(strtrim(str(11:end))); 
            Subject.Age = 100-str2num(Subject.Birthday(end-1:end))+07;
        end
        
        if strfind(str, 'Begin Data')
            str=[]; str=fgetl(fid); FieldNames = str;  % field names
            
            Data = textscan(fid, '%s %s %s %s %s %f %f %f %f %f %*[^\n]');

            for j=1:length(Data)
                Data{j} = Data{j}(1:end-1);
            end
            
            Correct = strcmp(Data{3},Data{4});

            [nRow, nCol] = size(Data{1});
            
            if ~isempty(Data{1})
              for j = 1:nRow
                Data{1}{j} = Data{1}{j}(1:end-4);   % wav filename
                DataUsername{j} = Subject.Username;         % 
                DataFirstLanguage{j} = Subject.FirstLanguage;
                DataAccent{j} = Subject.Accent;
                DataGender{j} = Subject.Gender; 
                DataAge(j) = Subject.Age;
              end
            end
        end

    end
    fclose(fid);
    
    if ~isempty(Data{1})
        [nRow, nCol] = size(Data);
    
        Data{nCol+1} = DataUsername'; %DataUsername = '';
        Data{nCol+2} = DataFirstLanguage'; %DataFirstLanguage = '';
        Data{nCol+3} = DataAccent'; %DataAccent = '';
        Data{nCol+4} = DataGender'; %DataGender = '';
        Data{nCol+5} = DataAge'; %DataAge = '';
        Data{nCol+6} = Correct; 
    
        if ~FLTDB_init 
            FLTDB = Data;
            FLTDB_init = 1;
        else
            [nRow nCol] = size(Data);
            for k=1:nCol
                FLTDB{k} = cat(1, FLTDB{k}, Data{k});
            end
        end
    end
    
    clear DataUsername DataFirstLanguage DataAccent DataGender DataAge Correct;

  end
end

% save the data in the format of SPDB as specified by create_SPDB_format.m
nRecord = length(FLTDB{1});

Exp = cell(nRecord, 1);         Exp(:) = {'HPLPF07'};
Hearing = cell(nRecord, 1);     Hearing(:) = {'NH'};
Audiogram = cell(nRecord, 1);   Audiogram(:) = {'NaN'};
Compensate = cell(nRecord, 1);  Compensate(:) = {'NaN'};
Trunctime = NaN(nRecord, 1);
n_level = NaN(nRecord, 1);
Noise = cell(nRecord, 1);       Noise(:) = {'WN'};
Comment = cell(nRecord, 1);     Comment(:) = {''};
Hit = NaN(nRecord, 1);    % hit

SPDB{1} = Exp;              % set the first field to be MN16R
SPDB{2} = FLTDB{1};         % utterances
SPDB{3} = FLTDB{2};         % talker
SPDB{4} = FLTDB{11};        % listener
SPDB{5} = Hearing;          % hearing
SPDB{6} = Audiogram;        % Audiogram
SPDB{7} = FLTDB{12};        % first language
SPDB{8} = FLTDB{13};        % accent
SPDB{9} = FLTDB{14};        % gender
SPDB{10} = FLTDB{15};       % age
SPDB{11} = FLTDB{3};        % stim
SPDB{12} = Compensate;      % compensation, NAL-R or other
SPDB{13} = Trunctime;       % truncation time
SPDB{14} = FLTDB{5};        % cutoff frequency
SPDB{15} = FLTDB{6};        % SNR
SPDB{16} = Noise;           % noise type
SPDB{17} = FLTDB{9};        % clean speech level (s_level)
SPDB{18} = n_level;         % noisy speech level (n_level)
SPDB{19} = FLTDB{9};        % actual stimulus level (r_level)
SPDB{20} = FLTDB{4};        % resp
SPDB{21} = FLTDB{7};        % response time
SPDB{22} = FLTDB{8};        % number of repeat
SPDB{23} = Hit;             % hit
SPDB{24} = Comment;         % comment

for j=1:length(SPDB{2})
    SPDB{2}{j} = SPDB{2}{j}(3:end);
end

% for every LP_0, make a copy and change to HP_0 (LP_0 is the same as
% HP_0), but it will make the processing much easier.
p = find(strcmp(SPDB{14}, 'LP_0'));

SPDB_LP_0{1} = SPDB{1}(p);
SPDB_LP_0{2} = SPDB{2}(p);
SPDB_LP_0{3} = SPDB{3}(p);
SPDB_LP_0{4} = SPDB{4}(p);
SPDB_LP_0{5} = SPDB{5}(p);
SPDB_LP_0{6} = SPDB{6}(p);

SPDB_LP_0{7} = SPDB{7}(p);
SPDB_LP_0{8} = SPDB{8}(p);
SPDB_LP_0{9} = SPDB{9}(p);
SPDB_LP_0{10} = SPDB{10}(p);
SPDB_LP_0{11} = SPDB{11}(p);
SPDB_LP_0{12} = SPDB{12}(p);

SPDB_LP_0{13} = SPDB{13}(p);
SPDB_LP_0{14} = SPDB{14}(p);
SPDB_LP_0{15} = SPDB{15}(p);
SPDB_LP_0{16} = SPDB{16}(p);
SPDB_LP_0{17} = SPDB{17}(p);
SPDB_LP_0{18} = SPDB{18}(p);

SPDB_LP_0{19} = SPDB{19}(p);
SPDB_LP_0{20} = SPDB{20}(p);
SPDB_LP_0{21} = SPDB{21}(p);
SPDB_LP_0{22} = SPDB{22}(p);
SPDB_LP_0{23} = SPDB{23}(p);
SPDB_LP_0{24} = SPDB{24}(p);

SPDB_LP_0{14} = strrep(SPDB_LP_0{14}, 'LP_0', 'HP_0');

for j=1:length(SPDB)
    SPDB{j} = cat(1, SPDB{j}, SPDB_LP_0{j});
end

% create a map from Filter_Label to cutoff frequency
Filters = {'LP_0','LP_1','LP_2','LP_3','LP_4','LP_5','LP_6','LP_7','LP_8','LP_9',...        
           'HP_1','HP_2','HP_3','HP_4','HP_5','HP_6','HP_7','HP_8','HP_9','HP_0'};

for i=1:length(Filters)
    [ freq_low, freq_high ] = get_cutfreq_HPLPF07( Filters{i} );
    if strcmp(Filters{i}(1:2), 'LP')
        CutOffFreq(i) = freq_high;
    elseif strcmp(Filters{i}(1:2), 'HP')
        CutOffFreq(i) = freq_low;
    end
end

Filt2Freq{1} = Filters;
Filt2Freq{2} = CutOffFreq;

SPDB{24} = SPDB{14};    % retain the Filter information in Comment field

for i=1:length(Filters)
    SPDB{14} = strrep(SPDB{14}, Filt2Freq{1}{i}, num2str(Filt2Freq{2}(i))); 
end

DataCutFreq = zeros(size(SPDB{14}));
for i=1:length(SPDB{14})
    DataCutFreq(i) = str2num(SPDB{14}{i});
end
SPDB{14} = DataCutFreq;

SPDB{20} = strrep(SPDB{20}, 'Only_Noise', '?');
SPDB{20} = strrep(SPDB{20}, 'tha', 'xda');
SPDB{20} = strrep(SPDB{20}, 'THa', 'xta');
SPDB{20} = strrep(SPDB{20}, 'sha', 'xsa');
SPDB{20} = strrep(SPDB{20}, 'Za', 'xza');

Hit = strcmp(lower(SPDB{11}), lower(SPDB{20}));    % hit
SPDB{23} = Hit;             % hit

for i=1:length(SPDB{24})
    SPDB{24}{i} = SPDB{24}{i}(1:2);
end

% Pull out basic information from the Normal Hearing data FLTDB
Index.exp = unique(sort(SPDB{1}));
Index.utter = unique(sort(SPDB{2}));
Index.talker = unique(sort(SPDB{3}));
Index.listener = unique(sort(SPDB{4}));
Index.hearing = unique(sort(SPDB{5}));
Index.audiogram = unique(sort(SPDB{6}));
Index.L1 = unique(sort(SPDB{7}));
Index.accent = unique(sort(SPDB{8}));
Index.gender = unique(sort(SPDB{9}));
Index.age = unique(sort(SPDB{10}));
Index.stim = unique(sort(SPDB{11}));
Index.compensate = unique(sort(SPDB{12}));
Index.trunctime = unique(sort(SPDB{13}));
Index.frequency = unique(sort(SPDB{14}));
Index.snr = unique(sort(SPDB{15}));
Index.noise = unique(sort(SPDB{16}));
Index.s_level = unique(sort(SPDB{17}));
Index.n_level = unique(sort(SPDB{18}));
Index.r_level = unique(sort(SPDB{19}));
Index.resp = unique(sort(SPDB{20}));
Index.resptime = unique(sort(SPDB{21}));
Index.repeat = unique(sort(SPDB{22}));
Index.hit = unique(sort(SPDB{23}));
Index.comment = unique(sort(SPDB{24}));

rmdir(datapath,'s');
