%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is created to pull out the MN16 data of the Normal 
% Hearing human speech perception experiment, NH-HI-06.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-26
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [SPDB, Index] = pull_MN16R_data(zipfile)

% first read the data from the log files, then save the data in the format
% of SPDB, as specified by creat_SPDB_format.m

disp(['begin unziping ' zipfile '...']);
datapath = './data/mn16rdata/';
mkdir(datapath); 
unzip(zipfile, datapath);
disp(['finish unziping ' zipfile '...']);

disp('begin pulling data from log files ...');
files = dir([datapath '*.log']);
for i=1:length(files)
	filename = [datapath files(i).name];   disp([filename ' finished!']);
    fid = fopen(filename);

    EOF = 0;
    while ~EOF
        str=[];
    	str=fgetl(fid);
    	
        if ~ischar(str), EOF=1; break, end; % Test for the end of file
    
        if strfind(str, 'LNAME')
            Listener = str(9:end); 
            if isempty(Listener), 
                Listener = ['nobody' num2str(i)]; 
            end
            str=[]; str=fgetl(fid); L1 = lower(strtrim(str(9:end)));
            str=[]; str=fgetl(fid); L2 = lower(strtrim(str(9:end)));
            str=[]; str=fgetl(fid); Sex = strtrim(upper(str(9:9)));
            str=[]; str=fgetl(fid); if ~isempty(strtrim(str(7:end)))   Age = round(str2num(strtrim(str(7:end)))); else Age = NaN; end
            str=[]; str=fgetl(fid); 
            str=[]; str=fgetl(fid); 
            str=[]; str=fgetl(fid); Accent = lower(strtrim(str(10:end))); 
        end

        if strfind(str, 'Begin Data')
            str=[]; str=fgetl(fid);
            Data = textscan(fid, 'FILE=s_%s STIM=%s RESP=%s SNR=%f TIME=%f PLAYS=%f          %f  WAVVU=%f NOISEVU=%f VUSNR=%f %*[^\n]');

            [nRow, nCol] = size(Data{1});
            for j = 1:nRow
                Data{1}{j} = Data{1}{j}(1:end-4);
                DataListener{j} = Listener;
                DataL1{j} = L1;
                DataL2{j} = L2;
                DataSex{j} = Sex;
                DataAge(j) = Age;
                DataAccent{j} = Accent;
                DataTalker{j} = Data{1}{j}(1:4);                
            end
        end

    end
    fclose(fid);
    
    [nRow, nCol] = size(Data);
    Data{nCol+1} = DataListener'; DataListener = '';
    Data{nCol+2} = DataL1'; DataL1 = '';
    Data{nCol+3} = DataL2'; DataL2 = '';
    Data{nCol+4} = DataSex'; DataSex = '';
    Data{nCol+5} = DataAge'; DataAge = '';
    Data{nCol+6} = DataAccent'; DataAccent = '';
    Data{nCol+7} = DataTalker'; DataTalker = '';
    
    if isequal(i, 1) 
        NHDB = Data;
    else
        [nRow nCol] = size(Data);
        for k=1:nCol
            NHDB{k} = cat(1, NHDB{k}, Data{k});
        end
    end

end
disp('finish pulling data from log files ...');

% save the data in the format of SPDB as specified by create_SPDB_format.m
nRecord = length(NHDB{1});

Exp = cell(nRecord, 1);         Exp(:) = {'MN16R'};
Hearing = cell(nRecord, 1);     Hearing(:) = {'NH'};
Audiogram = cell(nRecord, 1);   Audiogram(:) = {'NaN'};
Compensate = cell(nRecord, 1);  Compensate(:) = {'NaN'};
Trunctime = NaN(nRecord, 1);
Frequency = NaN(nRecord, 1);
Noise = cell(nRecord, 1);       Noise(:) = {'WN'};
Comment = cell(nRecord, 1);     Comment(:) = {''};
Hit = strcmp(lower(NHDB{2}), lower(NHDB{3}));    % hit

SPDB{1} = Exp;          % set the first field to be MN16R
SPDB{2} = NHDB{1};      % utterances
SPDB{3} = NHDB{17};     % talker
SPDB{4} = NHDB{11};     % listener
SPDB{5} = Hearing;      % hearing
SPDB{6} = Audiogram;    % Audiogram
SPDB{7} = NHDB{12};     % first language
SPDB{8} = NHDB{16};     % accent
SPDB{9} = NHDB{14};     % gender
SPDB{10} = NHDB{15};    % age
SPDB{11} = NHDB{2};     % stim
SPDB{12} = Compensate;  % compensation, NAL-R or other
SPDB{13} = Trunctime;   % truncation time
SPDB{14} = Frequency;   % frequency band
SPDB{15} = NHDB{4};     % SNR
SPDB{16} = Noise;       % noise type
SPDB{17} = NHDB{8};     % clean speech level (s_level)
SPDB{18} = NHDB{9};     % noisy speech level (n_level)
SPDB{19} = NHDB{10};    % actual stimulus level (r_level)
SPDB{20} = NHDB{3};     % resp
SPDB{21} = NHDB{5};     % response time
SPDB{22} = NHDB{6};     % number of repeat
SPDB{23} = Hit;         % hit
SPDB{24} = Comment;     % comment

SPDB{20} = strrep(SPDB{20}, 'noidea', '?');

% fix error introduced by typo
SPDB{7} = strrep(SPDB{7}, 'enhlish', 'english');

% Pull out basic information from the Normal Hearing data NHDB
Index.exp = unique(sort(SPDB{1}));
Index.utter = unique(sort(SPDB{2}));
Index.talker = unique(sort(SPDB{3}));
Index.listener = unique(sort(SPDB{4}));
Index.hearing = unique(sort(SPDB{5}));
Index.audiogram = unique(sort(SPDB{6}));
Index.L1 = unique(sort(SPDB{7}));
Index.accent = unique(sort(SPDB{8}));
Index.gender = unique(sort(SPDB{9}));
Index.age = unique(sort(SPDB{10}));
Index.stim = unique(sort(SPDB{11}));
Index.compensate = unique(sort(SPDB{12}));
Index.trunctime = unique(sort(SPDB{13}));
Index.frequency = unique(sort(SPDB{14}));
Index.snr = unique(sort(SPDB{15}));
Index.noise = unique(sort(SPDB{16}));
Index.s_level = unique(sort(SPDB{17}));
Index.n_level = unique(sort(SPDB{18}));
Index.r_level = unique(sort(SPDB{19}));
Index.resp = unique(sort(SPDB{20}));
Index.resptime = unique(sort(SPDB{21}));
Index.repeat = unique(sort(SPDB{22}));
Index.hit = unique(sort(SPDB{23}));
Index.comment = unique(sort(SPDB{24}));

rmdir(datapath,'s');

