%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program read the pure tone audiogram data from *.csv files
% for the hearing impaired speech perception experiment, SNRLoss06
% and SNRLoss07, and return it in a mat file
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-28
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Audiograms, Index] = create_audiogram(filename)

fid = fopen(filename); 
data = textscan(fid, '%s %s %s %s %s %s %s %s %s %s %s %s %*[^\n]', 'delimiter', ',');
fclose(fid);

for i=2:length(data)-2
    data{i} = strrep(data{i}, 'NaN', '-1'); % first convert all the NaN to -1 for later processing
    temp = zeros(size(data{i}));
    for j=1:length(data{i})
        temp(j) = str2num(data{i}{j});
    end
    p = find(temp==-1); % convert -1 back to NaN;
    if ~isempty(p)
        temp(p) = NaN;
    end
    data{i} = temp;
end

Freq = [ data{2}(1), data{3}(1), data{4}(1), data{5}(1), data{6}(1), ...
         data{7}(1), data{8}(1), data{9}(1), data{10}(1) ];

for i=2:length(data{1})
    
    ID = data{1}{i};
    HL =  [ data{2}(i), data{3}(i), data{4}(i), data{5}(i), data{6}(i), ...
            data{7}(i), data{8}(i), data{9}(i), data{10}(i)];
    PTA = str2num(data{11}{i});
    SNRLoss = str2num(data{12}{i});
    
    audiogram.Freq = Freq;
    audiogram.HL = HL;
    
    subject.ID = ID;
    subject.audiogram = audiogram;
    subject.PTA = PTA;
    subject.SNRLoss = SNRLoss;
    
    Audiograms{i-1} = subject;
    Index{i-1} = ID;
end

save('Audiograms.mat', 'Audiograms', 'Index');