%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is created to pull out the data of the Hearing Impaired 
% human speech perception experiment, NH-HI-06.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2006-11-07
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [SPDB, Index] = pull_SNRLoss06_data(zipfile)

% then pull out the Hearing Impaired data
datapath = './data/';

%mkdir(datapath); 
%unzip(zipfile, datapath);

dirs = dir([datapath '*']);

disp('begin pulling data from log files ...');
for m = 3:length(dirs)-1;
    
files = dir([datapath dirs(m).name '/*.log']);

for i=1:length(files)
	filename = [datapath '/' dirs(m).name '/' files(i).name];   disp([filename ' finished!']);
    fid = fopen(filename);

    EOF = 0;
    while ~EOF
        str=[];
    	str=fgetl(fid);
    	
        if ~ischar(str), EOF=1; break, end; % Test for the end of file
    
        if strfind(str, 'LNAME')
            Listener = dirs(m).name; 

            str=[]; str=fgetl(fid); if length(str)>8, L1 = lower(strtrim(str(9:end))); else L1 = 'english'; end
            str=[]; str=fgetl(fid); if length(str)>8, L2 = lower(strtrim(str(9:end))); else L2 = 'NaN'; end
            str=[]; str=fgetl(fid); if length(str)>8, Sex = strtrim(upper(str(9:9))); else Sex = 'M'; end
            str=[]; str=fgetl(fid); if length(str)>7, Age = round(str2num(strtrim(str(7:end)))); else Age = NaN; end 
            str=[]; str=fgetl(fid); 
            str=[]; str=fgetl(fid); 
            str=[]; str=fgetl(fid); Accent = lower(strtrim(str(10:end))); 
        end

        if strfind(str, 'Begin Data')
            str=[]; str=fgetl(fid);
            Data = textscan(fid, 'FILE=s_%s STIM=%s RESP=%s SNR=%f TIME=%f PLAYS=%f  WAVRMS=%f NOISERMS=%f %*[^\n]');

            [nRow, nCol] = size(Data{1});
            for j = 1:nRow
                Data{1}{j} = Data{1}{j}(1:end-4);
                DataListener{j} = Listener;
                DataL1{j} = L1;
                DataL2{j} = L2;
                DataSex{j} = Sex;
                DataAge(j) = Age;
                DataAccent{j} = Accent;
                DataTalker{j} = Data{1}{j}(1:4);                
            end
        end

    end
    fclose(fid);
    
    [nRow, nCol] = size(Data);
    Data{nCol+1} = DataListener'; DataListener = '';
    Data{nCol+2} = DataL1'; DataL1 = '';
    Data{nCol+3} = DataL2'; DataL2 = '';
    Data{nCol+4} = DataSex'; DataSex = '';
    Data{nCol+5} = DataAge'; DataAge = '';
    Data{nCol+6} = DataAccent'; DataAccent = '';
    Data{nCol+7} = DataTalker'; DataTalker = '';
    
    if isequal(i, 1) 
        HIDBSub = Data;
    else
        [nRow nCol] = size(Data);
        for k=1:nCol
            HIDBSub{k} = cat(1, HIDBSub{k}, Data{k});
        end
    end
end

HIDB{m-2} = HIDBSub;    % HIDB contains data from all subjects
end %%%% for m = 3:length(dirs)-1;

HIDBALL = HIDB{1};
for i=2:length(HIDB)
    for j=1:length(HIDB{1})
        HIDBALL{j} = cat(1, HIDBALL{j}, HIDB{i}{j});
    end
end

disp('finish pulling data from log files ...');

% save the data in the format of SPDB as specified by create_SPDB_format.m
nRecord = length(HIDBALL{1});

Exp = cell(nRecord, 1);         Exp(:) = {'SL06'};
Hearing = cell(nRecord, 1);     Hearing(:) = {'HI'};
Audiogram = cell(nRecord, 1);   Audiogram(:) = {'audiogram.mat'};
Compensate = cell(nRecord, 1);  Compensate(:) = {'NaN'};
Trunctime = NaN(nRecord, 1);
Frequency = NaN(nRecord, 1);
Noise = cell(nRecord, 1);       Noise(:) = {'SWN'};
r_level = NaN(nRecord, 1);       
Comment = cell(nRecord, 1);     Comment(:) = {''};
Hit = strcmp(lower(HIDBALL{2}), lower(HIDBALL{3}));    % hit

SPDB{1} = Exp;          % set the first field to be MN16R
SPDB{2} = HIDBALL{1};      % utterances
SPDB{3} = HIDBALL{15};     % talker
SPDB{4} = HIDBALL{9};     % listener
SPDB{5} = Hearing;      % hearing
SPDB{6} = Audiogram;    % Audiogram
SPDB{7} = HIDBALL{10};     % first language
SPDB{8} = HIDBALL{14};     % accent
SPDB{9} = HIDBALL{12};     % gender
SPDB{10} = HIDBALL{13};    % age
SPDB{11} = HIDBALL{2};     % stim
SPDB{12} = Compensate;  % compensation, NAL-R or other
SPDB{13} = Trunctime;   % truncation time
SPDB{14} = Frequency;   % frequency band
SPDB{15} = HIDBALL{4};     % SNR
SPDB{16} = Noise;       % noise type
SPDB{17} = HIDBALL{7};     % clean speech level (s_level)
SPDB{18} = HIDBALL{8};     % noisy speech level (n_level)
SPDB{19} = r_level;     % actual stimulus level (r_level)
SPDB{20} = HIDBALL{3};     % resp
SPDB{21} = HIDBALL{5};     % response time
SPDB{22} = HIDBALL{6};     % number of repeat
SPDB{23} = Hit;         % hit
SPDB{24} = Comment;     % comment

SPDB{20} = strrep(SPDB{20}, 'noidea', '?');

% Pull out basic information from the Normal Hearing data HIDBALL
Index.exp = unique(sort(SPDB{1}));
Index.utter = unique(sort(SPDB{2}));
Index.talker = unique(sort(SPDB{3}));
Index.listener = unique(sort(SPDB{4}));
Index.hearing = unique(sort(SPDB{5}));
Index.audiogram = unique(sort(SPDB{6}));
Index.L1 = unique(sort(SPDB{7}));
Index.accent = unique(sort(SPDB{8}));
Index.gender = unique(sort(SPDB{9}));
Index.age = unique(sort(SPDB{10}));
Index.stim = unique(sort(SPDB{11}));
Index.compensate = unique(sort(SPDB{12}));
Index.trunctime = unique(sort(SPDB{13}));
Index.frequency = unique(sort(SPDB{14}));
Index.snr = unique(sort(SPDB{15}));
Index.noise = unique(sort(SPDB{16}));
Index.s_level = unique(sort(SPDB{17}));
Index.n_level = unique(sort(SPDB{18}));
Index.r_level = unique(sort(SPDB{19}));
Index.resp = unique(sort(SPDB{20}));
Index.resptime = unique(sort(SPDB{21}));
Index.repeat = unique(sort(SPDB{22}));
Index.hit = unique(sort(SPDB{23}));
Index.comment = unique(sort(SPDB{24}));

%rmdir(datapath,'s');


