%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is created to pull out the data of the Live Speech 
% human speech perception experiment, SNRLoss07.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-09-15
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [SPDB, Index] = pull_LiveSpeech_data()

datapath = './Live/';

fid = fopen([datapath 'Annette_Live.csv']); 
str=fgetl(fid); % skip the first line
HILVDB = textscan(fid, '%s %s %*[^\n]', 'delimiter', ',');
fclose(fid);

Correct = strcmp(HILVDB{1}, HILVDB{2});
HILVDB{end+1} = Correct;

% save the data in the format of SPDB as specified by create_SPDB_format.m
nRecord = length(HILVDB{1});

Utter = cell(nRecord, 1);       Utter(:) = {'NaN'};
Talker = cell(nRecord, 1);      Talker(:) = {'Jont Allen'};
Listener = cell(nRecord, 1);    Listener(:) = {'AS_Live'};
Exp = cell(nRecord, 1);         Exp(:) = {'SNRLoss07-Live'};
Hearing = cell(nRecord, 1);     Hearing(:) = {'HI'};
Audiogram = cell(nRecord, 1);   Audiogram(:) = {'audiogram.mat'};
L1 =  cell(nRecord, 1);         L1(:) = {'german'};
Accent = cell(nRecord, 1);      Accent(:) = {''};
Gender =  cell(nRecord, 1);     Gender(:) = {'F'};
Age = zeros(nRecord, 1);        Age(:) = 80;
SNR = zeros(nRecord, 1);        SNR(:) = 100;
Frequency = NaN(nRecord, 1);
Trunctime = NaN(nRecord, 1);
Compensate = cell(nRecord, 1);  Compensate(:) = {'NaN'};
s_level = NaN(nRecord, 1);
n_level = NaN(nRecord, 1);
r_level = NaN(nRecord, 1);
resptime = NaN(nRecord, 1);
repeat = zeros(nRecord, 1);
Noise = cell(nRecord, 1);       Noise(:) = {'NaN'};
Comment = cell(nRecord, 1);     Comment(:) = {'Live'};

SPDB{1} = Exp;              % set the first field to be MN16R
SPDB{2} = Utter;         % utterances
SPDB{3} = Talker;         % talker
SPDB{4} = Listener;        % listener
SPDB{5} = Hearing;          % hearing
SPDB{6} = Audiogram;        % Audiogram
SPDB{7} = L1;        % first language
SPDB{8} = Accent;        % accent
SPDB{9} = Gender;        % gender
SPDB{10} = Age;       % age
SPDB{11} = HILVDB{1};        % stim
SPDB{12} = Compensate;      % compensation, NAL-R or other
SPDB{13} = Trunctime;       % truncation time
SPDB{14} = Frequency;        % frequency band
SPDB{15} = SNR;        % SNR
SPDB{16} = Noise;           % noise type
SPDB{17} = s_level;        % clean speech level (s_level)
SPDB{18} = n_level;         % noisy speech level (n_level)
SPDB{19} = r_level;        % actual stimulus level (r_level)
SPDB{20} = HILVDB{2};        % resp
SPDB{21} = resptime;        % response time
SPDB{22} = repeat;        % number of repeat
SPDB{23} = HILVDB{3};             % hit
SPDB{24} = Comment;         % comment

% Pull out basic information from the Normal Hearing data HIDB
Index.exp = unique(sort(SPDB{1}));
Index.utter = unique(sort(SPDB{2}));
Index.talker = unique(sort(SPDB{3}));
Index.listener = unique(sort(SPDB{4}));
Index.hearing = unique(sort(SPDB{5}));
Index.audiogram = unique(sort(SPDB{6}));
Index.L1 = unique(sort(SPDB{7}));
Index.accent = unique(sort(SPDB{8}));
Index.gender = unique(sort(SPDB{9}));
Index.age = unique(sort(SPDB{10}));
Index.stim = unique(sort(SPDB{11}));
Index.compensate = unique(sort(SPDB{12}));
Index.trunctime = unique(sort(SPDB{13}));
Index.frequency = unique(sort(SPDB{14}));
Index.snr = unique(sort(SPDB{15}));
Index.noise = unique(sort(SPDB{16}));
Index.s_level = unique(sort(SPDB{17}));
Index.n_level = unique(sort(SPDB{18}));
Index.r_level = unique(sort(SPDB{19}));
Index.resp = unique(sort(SPDB{20}));
Index.resptime = unique(sort(SPDB{21}));
Index.repeat = unique(sort(SPDB{22}));
Index.hit = unique(sort(SPDB{23}));
Index.comment = unique(sort(SPDB{24}));