%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is created to pull out the data of the SNRLoss07EN Hearing
% Impaired speech perception experiment. Only /pa, ta, ka/ and /ba, da, ga/
% are included, with the boosting factor being [0, 1, 10, 50] 
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-09-15
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function [SPDB, Index] = pull_SNRLoss07EN_data(zipfile)
function [SPDB, Index] = pull_SNRLoss07EN_data()

datapath = './data/snrloss07endata/';
%mkdir(datapath); 
%unzip(zipfile, datapath);

files = dir([datapath '*.txt']);

HIDB_init = 0;

for i=1:length(files)
  filename = [datapath files(i).name];
    
  if isempty(findstr(filename, 'practice'))
  
    disp(filename);
    fid = fopen(filename);

    EOF = 0;
    while ~EOF
        str=[];
    	str=fgetl(fid);
    	
        if ~ischar(str), EOF=1; break, end; % Test for the end of file
    
        if strfind(str, 'Begin User Information')
            str=[]; str=fgetl(fid); Subject.Username = strtrim(str(13:end)); 
            str=[]; str=fgetl(fid); Subject.Firstname = strtrim(str(14:end));
            str=[]; str=fgetl(fid); Subject.Lastname = strtrim(str(13:end));
            str=[]; str=fgetl(fid); Subject.Gender = strtrim(upper(str(12:12)));
            str=[]; str=fgetl(fid); Subject.Birthday = strtrim(str(13:end));
            str=[]; str=fgetl(fid); Subject.FirstLanguage = strtrim(str(17:end));
            str=[]; str=fgetl(fid); Subject.Accent = strtrim(str(11:end)); 
            Subject.Age = 100-str2num(Subject.Birthday(end-1:end))+07;
        end
        
        if strfind(str, 'Begin Data')
            str=[]; str=fgetl(fid); FieldNames = str;  % field names

            Data = textscan(fid, '%s %s %s %s %s %f %f %f %f %f %*[^\n]');
            
            % get rid of the last line
            for j=1:length(Data)
                Data{j} = Data{j}(1:end-1);
            end
            
            % convert SNR from string to integer
            for j=1:length(Data{5})
                DataSNR(j) = str2num(Data{5}{j}); 
            end
            Data{5} = DataSNR';
            
            Correct = strcmp(Data{3},Data{4});
            
            NALR = cell(size(Data{1}));  NALR(:) = {'NAL-R'};
            if isempty(findstr(filename, 'NAL-R'))
                NALR(:) = {'NaN'};                
            end
            
            DataEnhanceFactor = cell(size(Data{1})); 
            DataEnhanceFactor(:) = {'NaN'};
            if ~isempty(findstr(filename, 'SNRLoss07EN1'))
                DataEnhanceFactor(:) = {'EN1'};    
            elseif ~isempty(findstr(filename, 'SNRLoss07EN2'))
                DataEnhanceFactor(:) = {'EN2'}; 
            elseif ~isempty(findstr(filename, 'SNRLoss07RM1'))
                DataEnhanceFactor(:) = {'RM1'};   
            elseif ~isempty(findstr(filename, 'SNRLoss07RE1'))
                DataEnhanceFactor(:) = {'RE1'};  
            elseif ~isempty(findstr(filename, 'SNRLoss07SW'))
                DataEnhanceFactor(:) = {'SW'};                   
            end           

            [nRow, nCol] = size(Data{1});
            
            if ~isempty(Data{1})
              for j = 1:nRow
                  
                Data{1}{j} = Data{1}{j}(1:end-4);   % wav filename
                
                DataUsername{j} = Subject.Username;         % 
                DataFirstLanguage{j} = Subject.FirstLanguage;
                DataAccent{j} = Subject.Accent;
                DataGender{j} = Subject.Gender;
                DataAge(j) = Subject.Age;

              end
            end
        end

    end
    fclose(fid);
    
    if ~isempty(Data{1})
        [nRow, nCol] = size(Data);
    
        Data{nCol+1} = DataUsername'; %DataUsername = '';
        Data{nCol+2} = DataFirstLanguage'; %DataFirstLanguage = '';
        Data{nCol+3} = DataAccent'; %DataAccent = '';
        Data{nCol+4} = DataGender'; %DataGender = '';
        Data{nCol+5} = DataAge'; %DataBirthday = '';
        Data{nCol+6} = Correct; % this Correct is not used
        Data{nCol+7} = NALR;
        Data{nCol+8} = DataEnhanceFactor; % the enhance factor (multiply, NOT dB)
    
        if ~HIDB_init 
            HIDB = Data;
            HIDB_init = 1;
        else
            [nRow nCol] = size(Data);
            for k=1:nCol
                HIDB{k} = cat(1, HIDB{k}, Data{k});
            end
        end
    end
    
    clear DataUsername DataFirstLanguage DataAccent DataGender DataBirthday DataSNR DataAge Correct;

  end
end


% save the data in the format of SPDB as specified by create_SPDB_format.m
nRecord = length(HIDB{1});

Exp = cell(nRecord, 1);         Exp(:) = {'SNRLoss07EN'};
Hearing = cell(nRecord, 1);     Hearing(:) = {'HI'};
Audiogram = cell(nRecord, 1);   Audiogram(:) = {'audiogram.mat'};
Frequency = NaN(nRecord, 1);
Trunctime = NaN(nRecord, 1);
Noise = cell(nRecord, 1);       Noise(:) = {'SWN'};
Comment = cell(nRecord, 1);     Comment(:) = {''};
Hit = NaN(nRecord, 1);    % hit

SPDB{1} = Exp;              % set the first field to be MN16R
SPDB{2} = HIDB{1};         % utterances
SPDB{3} = HIDB{2};         % talker
SPDB{4} = HIDB{11};        % listener
SPDB{5} = Hearing;          % hearing
SPDB{6} = Audiogram;        % Audiogram
SPDB{7} = HIDB{12};        % first language
SPDB{8} = HIDB{13};        % accent
SPDB{9} = HIDB{14};        % gender
SPDB{10} = HIDB{15};       % age
SPDB{11} = HIDB{3};        % stim
SPDB{12} = HIDB{17};      % compensation, NAL-R or other
SPDB{13} = Trunctime;       % truncation time
SPDB{14} = Frequency;        % frequency band
SPDB{15} = HIDB{5};        % SNR
SPDB{16} = Noise;           % noise type
SPDB{17} = HIDB{8};        % clean speech level (s_level)
SPDB{18} = HIDB{9};         % noisy speech level (n_level)
SPDB{19} = HIDB{10};        % actual stimulus level (r_level)
SPDB{20} = HIDB{4};        % resp
SPDB{21} = HIDB{6};        % response time
SPDB{22} = HIDB{7};        % number of repeat
SPDB{23} = Hit;             % hit
SPDB{24} = HIDB{18};         % comment

SPDB{20} = strrep(SPDB{20}, 'Only_Noise', '?');
SPDB{20} = strrep(SPDB{20}, 'tha', 'xda');
SPDB{20} = strrep(SPDB{20}, 'THa', 'xta');
SPDB{20} = strrep(SPDB{20}, 'sha', 'xsa');
SPDB{20} = strrep(SPDB{20}, 'Za', 'xza');

Hit = strcmp(lower(SPDB{11}), lower(SPDB{20}));    % hit
SPDB{23} = Hit;             % hit

% fix error in the data
SPDB{7} = strrep(SPDB{7}, 'grman', 'german');
SPDB{9} = strrep(SPDB{9}, 'S', 'F');

% Pull out basic information from the Normal Hearing data HIDB
Index.exp = unique(sort(SPDB{1}));
Index.utter = unique(sort(SPDB{2}));
Index.talker = unique(sort(SPDB{3}));
Index.listener = unique(sort(SPDB{4}));
Index.hearing = unique(sort(SPDB{5}));
Index.audiogram = unique(sort(SPDB{6}));
Index.L1 = unique(sort(SPDB{7}));
Index.accent = unique(sort(SPDB{8}));
Index.gender = unique(sort(SPDB{9}));
Index.age = unique(sort(SPDB{10}));
Index.stim = unique(sort(SPDB{11}));
Index.compensate = unique(sort(SPDB{12}));
Index.trunctime = unique(sort(SPDB{13}));
Index.frequency = unique(sort(SPDB{14}));
Index.snr = unique(sort(SPDB{15}));
Index.noise = unique(sort(SPDB{16}));
Index.s_level = unique(sort(SPDB{17}));
Index.n_level = unique(sort(SPDB{18}));
Index.r_level = unique(sort(SPDB{19}));
Index.resp = unique(sort(SPDB{20}));
Index.resptime = unique(sort(SPDB{21}));
Index.repeat = unique(sort(SPDB{22}));
Index.hit = unique(sort(SPDB{23}));
Index.comment = unique(sort(SPDB{24}));



