%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is created to pull out the Truncation06 data of the Normal 
% Hearing human speech perception experiment.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-10-15
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [SPDB, Index] = pull_Trunc06_data(zipfile)

disp(['begin unziping ' zipfile '...']);
datapath = './data/trunc06data/';
mkdir(datapath); 
unzip(zipfile, datapath);
disp(['finish unziping ' zipfile '...']);

disp('begin pulling data from log files ...');
files = dir([datapath '*.log']);
for i=1:length(files)
	filename = [datapath files(i).name];
    disp(filename);
    fid = fopen(filename);

    EOF = 0;
    while ~EOF
        str=[];
    	str=fgetl(fid);
    	
        if ~ischar(str), EOF=1; break, end; % Test for the end of file
    
        if strfind(str, 'LNAME')
            Listener = str(9:end); 
            if isempty(Listener), 
                Listener = ['nobody' num2str(i)]; 
            end
            str=[]; str=fgetl(fid); L1 = lower(str(9:end));
            str=[]; str=fgetl(fid); L2 = str(9:end);
            str=[]; str=fgetl(fid); Sex = upper(str(9:9));
            str=[]; str=fgetl(fid); Age = strtrim(str(7:end));          
        end

        if strfind(str, 'Begin Data')
            str=[]; str=fgetl(fid);
            Data = textscan(fid, 'FILE=s_%s STIM=%s RESP=%s SNR=%f TIME=%f PLAYS=%f %*[^\n]');
            
            % 1230 means snr = 12dB and time = 30ms ? while 31 means snr =
            % 0 and time = 31ms
            DataTime = zeros(length(Data{4}),1);
            for k=1:length(Data{4})
                DataTime(k) = mod(Data{4}(k), 100);
                Data{4}(k) = round(Data{4}(k)/100);
            end
            
            [nRow, nCol] = size(Data{1});
            for j = 1:nRow
                Data{1}{j} = Data{1}{j}(1:end-4);
                DataListener{j} = Listener;
                DataL1{j} = L1;
                DataL2{j} = L2;
                DataSex{j} = Sex;
                DataAge{j} = Age;
                DataTalker{j} = Data{1}{j}(1:4);   
            end
        end

    end
    fclose(fid);
    
    [nRow, nCol] = size(Data);
    Data{nCol+1} = DataTime; DataTime = '';
    Data{nCol+2} = DataListener'; DataListener = '';
    Data{nCol+3} = DataL1'; DataL1 = '';
    Data{nCol+4} = DataL2'; DataL2 = '';
    Data{nCol+5} = DataSex'; DataSex = '';
    Data{nCol+6} = DataAge'; DataAge = '';
    Data{nCol+7} = DataTalker'; DataTalker = '';
    
    if isequal(i, 1) 
        TRUNCDB = Data;
    else
        [nRow nCol] = size(Data);
        for k=1:nCol
            TRUNCDB{k} = cat(1, TRUNCDB{k}, Data{k});
        end
    end

end

% save the data in the format of SPDB as specified by create_SPDB_format.m
nRecord = length(TRUNCDB{1});

Exp = cell(nRecord, 1);         Exp(:) = {'Trunc06'};
Hearing = cell(nRecord, 1);     Hearing(:) = {'NH'};
Audiogram = cell(nRecord, 1);   Audiogram(:) = {'NaN'};
Compensate = cell(nRecord, 1);  Compensate(:) = {'NaN'};
Trunctime = NaN(nRecord, 1);
Frequency = NaN(nRecord, 1);
Accent = cell(nRecord, 1);      Accent(:) = {'NH'};
Noise = cell(nRecord, 1);       Noise(:) = {'SWN'};
s_level = NaN(nRecord, 1);
n_level = NaN(nRecord, 1);
r_level = NaN(nRecord, 1);
Comment = cell(nRecord, 1);     Comment(:) = {''};

for j=1:length(TRUNCDB{3})
    TRUNCDB{3}{j} = [TRUNCDB{3}{j} 'a'];
end
Hit = strcmp(lower(TRUNCDB{2}), lower(TRUNCDB{3}));    % hit

SPDB{1} = Exp;          % set the first field to be MN16R
SPDB{2} = TRUNCDB{1};      % utterances
SPDB{3} = TRUNCDB{13};     % talker
SPDB{4} = TRUNCDB{8};     % listener
SPDB{5} = Hearing;      % hearing
SPDB{6} = Audiogram;    % Audiogram
SPDB{7} = TRUNCDB{9};     % first language
SPDB{8} = Accent;     % accent
SPDB{9} = TRUNCDB{11};     % gender
SPDB{10} = TRUNCDB{12};    % age
SPDB{11} = TRUNCDB{2};     % stim
SPDB{12} = Compensate;  % compensation, NAL-R or other
SPDB{13} = TRUNCDB{7};   % truncation time
SPDB{14} = Frequency;   % frequency band
SPDB{15} = TRUNCDB{4};     % SNR
SPDB{16} = Noise;       % noise type
SPDB{17} = s_level;     % clean speech level (s_level)
SPDB{18} = n_level;     % noisy speech level (n_level)
SPDB{19} = r_level;    % actual stimulus level (r_level)
SPDB{20} = TRUNCDB{3};     % resp
SPDB{21} = TRUNCDB{5};     % response time
SPDB{22} = TRUNCDB{6};     % number of repeat
SPDB{23} = Hit;         % hit
SPDB{24} = Comment;     % comment

SPDB{20} = strrep(SPDB{20}, 'noideaa', '?');

SPDBAge = zeros(size(SPDB{10}));
for j=1:length(SPDB{10})
    SPDBAge(j) = str2num(SPDB{10}{j});
end
SPDB{10} = SPDBAge;

% calculate real truncation time
load('Trunc06info.mat');
for i=1:length(SPDB{13})
    p = find(strcmp(Trunc06info{1}, ['s_' SPDB{2}{i} '.wav']));
    SPDB{13}(i) = Trunc06info{2}(p) + SPDB{13}(i)*5/1000;
end

% Pull out basic information from the Normal Hearing data TRUNCDB
Index.exp = unique(sort(SPDB{1}));
Index.utter = unique(sort(SPDB{2}));
Index.talker = unique(sort(SPDB{3}));
Index.listener = unique(sort(SPDB{4}));
Index.hearing = unique(sort(SPDB{5}));
Index.audiogram = unique(sort(SPDB{6}));
Index.L1 = unique(sort(SPDB{7}));
Index.accent = unique(sort(SPDB{8}));
Index.gender = unique(sort(SPDB{9}));
Index.age = unique(sort(SPDB{10}));
Index.stim = unique(sort(SPDB{11}));
Index.compensate = unique(sort(SPDB{12}));
Index.trunctime = unique(sort(SPDB{13}));
Index.frequency = unique(sort(SPDB{14}));
Index.snr = unique(sort(SPDB{15}));
Index.noise = unique(sort(SPDB{16}));
Index.s_level = unique(sort(SPDB{17}));
Index.n_level = unique(sort(SPDB{18}));
Index.r_level = unique(sort(SPDB{19}));
Index.resp = unique(sort(SPDB{20}));
Index.resptime = unique(sort(SPDB{21}));
Index.repeat = unique(sort(SPDB{22}));
Index.hit = unique(sort(SPDB{23}));
Index.comment = unique(sort(SPDB{24}));

rmdir(datapath,'s');


