%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is created to pull out the data of the TR08 Normal 
% Hearing human speech perception experiment, TR08.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2008-09-12
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [SPDB, Index] = pull_TR08_data()

datapath = './data/';
% mkdir(datapath); 
% unzip(zipfile, datapath);

files = dir([datapath '*.txt']);

TRNCDB_init = 0;

for i=1:length(files)
  filename = [datapath files(i).name];
    
  if isempty(findstr(filename, 'practice'))
  
    disp(filename);
    fid = fopen(filename);

    EOF = 0;
    while ~EOF
        str=[];
    	str=fgetl(fid);
    	
        if ~ischar(str), EOF=1; break, end; % Test for the end of file
    
        if strfind(str, 'Begin User Information')
            str=[]; str=fgetl(fid); Subject.Username = strtrim(str(13:end)); 
            str=[]; str=fgetl(fid); Subject.Firstname = strtrim(str(14:end));
            str=[]; str=fgetl(fid); Subject.Lastname = strtrim(str(13:end));
            str=[]; str=fgetl(fid); Subject.Gender = strtrim(upper(str(12:12)));
            str=[]; str=fgetl(fid); Subject.Birthday = strtrim(str(13:end));
            str=[]; str=fgetl(fid); Subject.FirstLanguage = lower(strtrim(str(17:end)));
            str=[]; str=fgetl(fid); Subject.Accent = strtrim(str(11:end));
            Subject.Age = 100-str2num(Subject.Birthday(end-1:end))+07;
        end
        
        if strfind(str, 'Begin Data')
            str=[]; str=fgetl(fid); FieldNames = str;  % field names
            
            % DO NOT USE %d for integer number, it will cause trouble!!!
            Data = textscan(fid, '%s %s %s %s %s %s %f %f %f %f %*[^\n]');

            for j=1:length(Data)
                Data{j} = Data{j}(1:end-1);
            end
            
            DataTrunctime = zeros(size(Data{5}));
            for j=1:length(Data{5})
                DataTrunctime(j) = str2num(Data{5}{j})/1000; 
            end
            Data{5} = DataTrunctime;
            
            Correct = strcmp(Data{3},Data{4});

            [nRow, nCol] = size(Data{1});
            
            if ~isempty(Data{1})
              for j = 1:nRow
                Data{1}{j} = Data{1}{j};   % wav filename
                DataUsername{j} = Subject.Username;         % 
                DataFirstLanguage{j} = Subject.FirstLanguage;
                DataAccent{j} = Subject.Accent;
                DataGender{j} = Subject.Gender;
                DataAge(j) = Subject.Age;
              end
            end
        end

    end
    fclose(fid);
    
    if ~isempty(Data{1})
        [nRow, nCol] = size(Data);
    
        Data{nCol+1} = DataUsername'; %DataUsername = '';
        Data{nCol+2} = DataFirstLanguage'; %DataFirstLanguage = '';
        Data{nCol+3} = DataAccent'; %DataAccent = '';
        Data{nCol+4} = DataGender'; %DataGender = '';
        Data{nCol+5} = DataAge'; %DataBirthday = '';
        Data{nCol+6} = Correct; 
    
        if ~TRNCDB_init 
            TRNCDB = Data;
            TRNCDB_init = 1;
        else
            [nRow nCol] = size(Data);
            for k=1:nCol
                TRNCDB{k} = cat(1, TRNCDB{k}, Data{k});
            end
        end
    end
    
    clear DataUsername DataFirstLanguage DataTrunctime DataAccent DataGender DataAge Correct;

  end
end

% save the data in the format of SPDB as specified by create_SPDB_format.m
nRecord = length(TRNCDB{1});

Exp = cell(nRecord, 1);         Exp(:) = {'TR08'};
Hearing = cell(nRecord, 1);     Hearing(:) = {'NH'};
Audiogram = cell(nRecord, 1);   Audiogram(:) = {'NaN'};
Compensate = cell(nRecord, 1);  Compensate(:) = {'NaN'};
Frequency = NaN(nRecord, 1);
n_level = NaN(nRecord, 1);
Noise = cell(nRecord, 1);       Noise(:) = {'WN'};
Comment = cell(nRecord, 1);     Comment(:) = {''};
Hit = NaN(nRecord, 1);    % hit

SPDB{1} = Exp;              % set the first field to be MN16R
SPDB{2} = TRNCDB{1};         % utterances
SPDB{3} = TRNCDB{2};         % talker
SPDB{4} = TRNCDB{11};        % listener
SPDB{5} = Hearing;          % hearing
SPDB{6} = Audiogram;        % Audiogram
SPDB{7} = TRNCDB{12};        % first language
SPDB{8} = TRNCDB{13};        % accent
SPDB{9} = TRNCDB{14};        % gender
SPDB{10} = TRNCDB{15};       % age
SPDB{11} = TRNCDB{3};        % stim
SPDB{12} = Compensate;      % compensation, NAL-R or other
SPDB{13} = TRNCDB{5};       % truncation time
SPDB{14} = Frequency;        % frequency band
SPDB{15} = TRNCDB{6};        % SNR
SPDB{16} = Noise;           % noise type
SPDB{17} = TRNCDB{9};        % clean speech level (s_level)
SPDB{18} = n_level;         % noisy speech level (n_level)
SPDB{19} = TRNCDB{9};        % actual stimulus level (r_level)
SPDB{20} = TRNCDB{4};        % resp
SPDB{21} = TRNCDB{7};        % response time
SPDB{22} = TRNCDB{8};        % number of repeat
SPDB{23} = Hit;             % hit
SPDB{24} = Comment;         % comment

DataSNR = zeros(size(SPDB{15}));
for j=1:length(SPDB{15})
    DataSNR(j) = str2num(SPDB{15}{j});
end
SPDB{15} = DataSNR;

for j=1:length(SPDB{2})
    SPDB{2}{j} = SPDB{2}{j}(3:end);
end

SPDB{20} = strrep(SPDB{20}, 'vowel', '.');
SPDB{20} = strrep(SPDB{20}, 'other', '?');
SPDB{20} = strrep(SPDB{20}, 'th', 'xd');
SPDB{20} = strrep(SPDB{20}, 'TH', 'xt');
SPDB{20} = strrep(SPDB{20}, 'sh', 'xs');
SPDB{20} = strrep(SPDB{20}, 'Z', 'xz');

for i=1:length(SPDB{11})
    cur_resp = lower(SPDB{20}{i});
    cur_stim = lower(SPDB{11}{i}); cur_stim = cur_stim(1:length(cur_resp));    
    Hit(i) = strcmp(cur_stim, cur_resp);    % hit
end
SPDB{23} = Hit;             % hit

% Pull out basic information from the Normal Hearing data TRNCDB
Index.exp = unique(sort(SPDB{1}));
Index.utter = unique(sort(SPDB{2}));
Index.talker = unique(sort(SPDB{3}));
Index.listener = unique(sort(SPDB{4}));
Index.hearing = unique(sort(SPDB{5}));
Index.audiogram = unique(sort(SPDB{6}));
Index.L1 = unique(sort(SPDB{7}));
Index.accent = unique(sort(SPDB{8}));
Index.gender = unique(sort(SPDB{9}));
Index.age = unique(sort(SPDB{10}));
Index.stim = unique(sort(SPDB{11}));
Index.compensate = unique(sort(SPDB{12}));
Index.trunctime = unique(sort(SPDB{13}));
Index.frequency = unique(sort(SPDB{14}));
Index.snr = unique(sort(SPDB{15}));
Index.noise = unique(sort(SPDB{16}));
Index.s_level = unique(sort(SPDB{17}));
Index.n_level = unique(sort(SPDB{18}));
Index.r_level = unique(sort(SPDB{19}));
Index.resp = unique(sort(SPDB{20}));
Index.resptime = unique(sort(SPDB{21}));
Index.repeat = unique(sort(SPDB{22}));
Index.hit = unique(sort(SPDB{23}));
Index.comment = unique(sort(SPDB{24}));

%%%% rmdir(datapath,'s');


