% This program is used to calculate the SNR loss per consonant for
% the Hearing Impaired listeners. 
% SNR_Loss = SNR(Pc_HI=0.5) - SNR(Pc_NH=0.5)
% Pc_Loss = Pc_NH(SNR=quiet) - Pc_HI(SNR=quiet)

function plot_SNRLoss_vs_PILoss(HIPI, snrHI, NHPI, snrNH, stim, marker, markercolor, facecolor)

% first pre-process the Normal Hearing data. 
% If max(HIPI) > max(NHPI), set max(HIPI) = max(NHPI)
% interpolate NHPI and make sure that min(NHPI) = 0 using linear prediction
% so that NHPI covers the range of HIPI.

if snrHI(end)>32, snrHI(end) = 32; end; % quiet condition
if snrNH(end)>32, snrNH(end) = 32; end;

% make sure that NHPI and HIPI starts from 0, so it covers Pc = 50% point.
if NHPI(1)>0
    snr0 = interp1(NHPI([1:2]), snrNH([1:2]), 0, 'linear', 'extrap');
    NHPI = [0, NHPI];
    snrNH = [snr0; snrNH];
end

if HIPI(1)>0
    snr0 = interp1(HIPI([1:2]), snrHI([1:2]), 0, 'linear', 'extrap');
    HIPI = [0; HIPI];
    snrHI = [snr0; snrHI];
end

Pc_thresh = 0.5; % threshold of Pc for the calculation of SNRLoss

% find the first sample greater than Pc_thresh
pHI = min(find(HIPI>Pc_thresh));  
if ~isempty(pHI)
    snr_Pcthresh_HI = interp1(HIPI([pHI-1:pHI]), snrHI([pHI-1:pHI]), Pc_thresh, 'linear');
    Pc_HI = Pc_thresh;
else
    snr_Pcthresh_HI = snrHI(end);
    Pc_HI = HIPI(end);
end

pNH = min(find(NHPI>Pc_HI));  
if ~isempty(pNH)
    snr_Pcthresh_NH = interp1(NHPI([pNH-1:pNH]), snrNH([pNH-1:pNH]), Pc_HI, 'linear');
else
    snr_Pcthresh_NH = snrHI(end);
end

delta_snr = snr_Pcthresh_HI - snr_Pcthresh_NH;
delta_PI  = NHPI(end) - HIPI(end); 

%plot(snrHI, snr_NH4HI, ['--' marker], 'Color', markercolor, 'MarkerFacecolor', facecolor, 'MarkerSize', 6); hold on;
plot(delta_snr, delta_PI, ['--' marker], 'Color', markercolor, 'MarkerFacecolor', facecolor, 'MarkerSize', 9); hold on;
text(delta_snr+0.5, delta_PI+0.02, stim, 'FontSize', 11);

%xlim([-24, 26]); ylim([-24, 26]); 
%set(gca,'XTick',[-20:10:20, 24], 'XTickLabel', {-20:10:20, 'Q'})
%set(gca,'YTick',[-20:10:20, 24], 'YTickLabel', {-20:10:20, 'Q'})
xlabel('SNR Loss [dB]','FontSize',12)
ylabel('PI Loss [%]','FontSize',12)



