% This program is used to calculate the SNR loss per consonant for
% the Hearing Impaired listeners. 
% The idea: for any point on the HI PI curve, find out the snr of the point
% on the NH PI curve that has the same Pc value. 

function plot_SNRloss(HIPI, snrHI, NHPI, snrNH, marker, markercolor, facecolor, diagline)

% first pre-process the Normal Hearing data. 
% If max(HIPI) > max(NHPI), set max(HIPI) = max(NHPI)
% interpolate NHPI and make sure that min(NHPI) = 0 using linear prediction
% so that NHPI covers the range of HIPI.

if diagline
    plot([-20 24], [-20 24], 'k--'); hold on;
else

if snrHI(end)>24, snrHI(end) = 24; end; % quiet condition
if snrNH(end)>24, snrNH(end) = 24; end;

if min(NHPI)>0
    snr0 = interp1(NHPI, snrNH, 0, 'linear', 'extrap');
    NHPI = [0, NHPI];
    snrNH = [snr0; snrNH];
end

p = find(HIPI>max(NHPI)); HIPI(p) = max(NHPI);

for i=1:length(HIPI)
    snr_NH4HI(i) = interp1(NHPI, snrNH, HIPI(i), 'linear');
end

plot(snrHI, snr_NH4HI, ['--' marker], 'Color', markercolor, 'MarkerFacecolor', facecolor, 'MarkerSize', 6); hold on;

xlim([-24, 26]); ylim([-24, 26]); 

set(gca,'XTick',[-20:10:20, 24], 'XTickLabel', {-20:10:20, 'Q'})
set(gca,'YTick',[-20:10:20, 24], 'YTickLabel', {-20:10:20, 'Q'})
xlabel('SNR_{HI} [dB]','FontSize',16)
ylabel('SNR_{ANH} [dB]','FontSize',16)

end

