% This program is used to calculate the SNR loss per consonant for
% the Hearing Impaired listeners. 
% The idea: for any point on the HI PI curve, find out the snr of the point
% on the NH PI curve that has the same Pc value. 

function plot_SNRloss_vs_delta_Pc(HIPI, snrHI, pCH, NHPI, snrNH, stim, marker, markercolor, facecolor)

% first pre-process the Normal Hearing data. 
% If max(HIPI) > max(NHPI), set max(HIPI) = max(NHPI)
% interpolate NHPI and make sure that min(NHPI) = 0 using linear prediction
% so that NHPI covers the range of HIPI.

if snrHI(end)>32, snrHI(end) = 32; end; % quiet condition
if snrNH(end)>32, snrNH(end) = 32; end;

if min(NHPI)>0
    snr0 = interp1(NHPI, snrNH, 0, 'linear', 'extrap');
    NHPI = [0, NHPI];
    snrNH = [snr0; snrNH];
end

p = find(HIPI>max(NHPI)); HIPI(p) = max(NHPI);

snr_NH4HI = interp1(NHPI, snrNH, HIPI(pCH), 'linear');
pi_NH4HI  = interp1(snrNH, NHPI, snrHI(pCH), 'linear');
%delta_PI  = pi_NH4HI-HIPI(pCH);
delta_PI  = pi_NH4HI-max(HIPI);
delta_snr = snrHI(pCH)-snr_NH4HI;

%plot(snrHI, snr_NH4HI, ['--' marker], 'Color', markercolor, 'MarkerFacecolor', facecolor, 'MarkerSize', 6); hold on;
plot(delta_snr, delta_PI, ['--' marker], 'Color', markercolor, 'MarkerFacecolor', facecolor, 'MarkerSize', 9); hold on;
text(delta_snr+0.5, delta_PI+0.02, stim, 'FontSize', 11);

%xlim([-24, 26]); ylim([-24, 26]); 
%set(gca,'XTick',[-20:10:20, 24], 'XTickLabel', {-20:10:20, 'Q'})
%set(gca,'YTick',[-20:10:20, 24], 'YTickLabel', {-20:10:20, 'Q'})
xlabel('SNR Loss [dB]','FontSize',12)
ylabel('PI_{NH} - PI_{HI} [%]','FontSize',12)



