% This function is used for the calculate STFT of one-dimensional signal
% Input: x - signal in time domain (row vector)
%        Fs - sampling rate of the input signal
%        wsize - length of window function(number of samples per frame) 
% Return: X signal in STFT domain (nFrame*L)

function X = STFTk(x, Fs, wsize)

R = 4; % set decimation rate to be 4, i.e. 75% overlapping between windows
shift = floor(wsize/R);     % hamming window moving forward step length

% our experiment shows that Kaiser window with Asl = -91dB genereates the smallest
% reconstruction error in overlap add.
Asl = 91; % the sidelobe is -91dB smaller than the mainlobe. 
beta = 0.12438*(Asl+6.3); % from Discrete-Time Signal Processing, Oppenheim
w = kaiser(wsize,beta); w = w/sum(w);

x1 = zeros(length(x)+2*wsize,1); 
x1(wsize+1:wsize+length(x)) = x; % zero padding to get rid of the error part introduced by the window

nframes = floor((length(x)+2*wsize)/shift)-(R-1);
s = zeros(nframes, wsize);  % input signal x buffer

for i=1:nframes
    s(i,1:wsize) = x1((i-1)*shift+1:(i-1)*shift+wsize).*w;
    X(i,:) = fast(s(i,:));
end

X(nframes+1,:) = length(x); % the last row contains the length of the original signal

