% this script matches two speech signals based on their shape
% step 1: obtain the envelopes env1 adn env2, for both speech signal
% step 2: use a segment of 100 samples in env1, the peak, as landmark
% step 3: sweep landmark through env2 and calculate the error
% s1 - the original speech
% s2 - the shifted speech
function [shift, error] = speech_match(s1, s2, Fs)

% make sure that length(env2) is always greater than length(env1)
env1 = envelope(s1,Fs); 
env2 = envelope(s2,Fs);

n = floor(length(env1)/100);    % downsampling rate
env2 = [zeros(size(env1)); env2; zeros(size(env1))];

env1dn = downsample(env1,n);
env2dn = downsample(env2,n);

env1dn = env1dn/max(env1dn);

for i=1:(length(env2dn)-length(env1dn))
    target = env2dn(i:i+length(env1dn)-1);
    target = target/(max(target)+eps);
    dif_dn(i) = sum(abs(target-env1dn));
end

shift_dn = find(dif_dn==min(dif_dn))-1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
shift_range(1) = max(1, (shift_dn-2)*n);
shift_range(2) = min((shift_dn+2)*n, length(s1)+length(s2)+1);

s2ex = [zeros(size(s1)); s2; zeros(size(s1))];

s1 = s1/max(s1);

dif = nan(size(s2ex));
for i=shift_range(1):shift_range(2)
    target = s2ex(i:i+length(s1)-1);
    target = target/max(target);
    dif(i) = sum(abs(target-s1));
end

shift = (find(dif==min(dif)))-length(s1);
error = min(dif);

% figure(2); clf;
% if (shift>0)
%     plot(s1/max(s1), '-b');  hold on;
%     plot(s2(shift:end)/max(s2), '-r');
% else
%     plot(s1(-shift:end)/max(s1), '-b');  hold on;
%     plot(s2/max(s2), '-r');
% end
